@@ -8,14 +8,8 @@ import sys
 import typing
 from importlib.metadata import version
 from pathlib import Path
-from subprocess import check_call
-from subprocess import DEVNULL
-from subprocess import STDOUT
-from typing import TYPE_CHECKING
 
-import git
 import yaml
-from git.exc import GitCommandError
 from git.exc import InvalidGitRepositoryError
 
 import colrev.exceptions as colrev_exceptions
@@ -23,21 +17,21 @@ import colrev.operation
 from colrev.exit_codes import ExitCodes
 
 
-if TYPE_CHECKING:
-    import colrev.review_manager
+if False:  # pylint: disable=using-constant-test
+    from typing import TYPE_CHECKING
+
+    if TYPE_CHECKING:
+        import colrev.review_manager
 
 
 class Checker:
     """The CoLRev checker makes sure the project setup is ok"""
 
-    __COLREV_HOOKS_URL = "https://github.com/geritwagner/colrev-hooks"
-
     def __init__(
         self,
         *,
         review_manager: colrev.review_manager.ReviewManager,
     ) -> None:
-
         self.review_manager = review_manager
 
         self.review_manager.notified_next_operation = (
@@ -49,19 +43,12 @@ class Checker:
             self.records = self.review_manager.dataset.load_records_dict()
 
     def get_colrev_versions(self) -> list[str]:
-        """Get the colrev version as a list: (last-version, current-version)"""
+        """Get the colrev version as a list: (last_version, current_version)"""
         current_colrev_version = version("colrev")
         last_colrev_version = current_colrev_version
-        try:
-            last_commit_message = self.review_manager.dataset.get_commit_message(
-                commit_nr=0
-            )
-            cmsg_lines = last_commit_message.split("\n")
-            for cmsg_line in cmsg_lines[0:100]:
-                if "colrev:" in cmsg_line and "version" in cmsg_line:
-                    last_colrev_version = cmsg_line[cmsg_line.find("version ") + 8 :]
-        except ValueError:
-            pass
+        last_colrev_version = self.review_manager.settings.project.colrev_version
+        if last_colrev_version.endswith("."):
+            last_colrev_version += "0"
         return [last_colrev_version, current_colrev_version]
 
     def __check_software(self) -> None:
@@ -76,47 +63,6 @@ class Checker:
                 sys.version_info,
             )
 
-    def __lsremote(self, *, url: str) -> dict:
-        remote_refs = {}
-        git_repo = git.cmd.Git()
-        for ref in git_repo.ls_remote(url).split("\n"):
-            hash_ref_list = ref.split("\t")
-            remote_refs[hash_ref_list[1]] = hash_ref_list[0]
-        return remote_refs
-
-    def __colrev_hook_up_to_date(self) -> bool:
-
-        with open(".pre-commit-config.yaml", encoding="utf8") as pre_commit_y:
-            pre_commit_config = yaml.load(pre_commit_y, Loader=yaml.FullLoader)
-
-        local_hooks_version = ""
-        for repository in pre_commit_config["repos"]:
-            if repository["repo"] == self.__COLREV_HOOKS_URL:
-                local_hooks_version = repository["rev"]
-
-        refs = self.__lsremote(url=self.__COLREV_HOOKS_URL)
-        remote_sha = refs["HEAD"]
-        if remote_sha == local_hooks_version:
-            return True
-        return False
-
-    def __update_colrev_hooks(self) -> None:
-        if self.__COLREV_HOOKS_URL not in self.__get_installed_repos():
-            return
-        try:
-            if not self.__colrev_hook_up_to_date():
-                self.review_manager.logger.info("Updating pre-commit hooks")
-                check_call(["pre-commit", "autoupdate"], stdout=DEVNULL, stderr=STDOUT)
-                self.review_manager.dataset.add_changes(
-                    path=Path(".pre-commit-config.yaml")
-                )
-        except GitCommandError:
-            self.review_manager.logger.warning(
-                "No Internet connection, cannot check remote "
-                "colrev-hooks repository for updates."
-            )
-        return
-
     def check_repository_setup(self) -> None:
         """Check the repository setup"""
 
@@ -136,9 +82,6 @@ class Checker:
         # 3. Pre-commit hooks installed?
         self.__require_colrev_hooks_installed()
 
-        # 4. Pre-commit hooks up-to-date?
-        self.__update_colrev_hooks()
-
     @classmethod
     def in_virtualenv(cls) -> bool:
         """Check whether CoLRev operates in a virtual environment"""
@@ -160,7 +103,7 @@ class Checker:
         unmerged_blobs = git_repo.index.unmerged_blobs()
 
         for path, list_of_blobs in unmerged_blobs.items():
-            for (stage, _) in list_of_blobs:
+            for stage, _ in list_of_blobs:
                 if stage != 0:
                     raise colrev_exceptions.GitConflictError(Path(path))
 
@@ -189,14 +132,6 @@ class Checker:
             installed_hooks.extend([hook["id"] for hook in repository["hooks"]])
         return installed_hooks
 
-    def __get_installed_repos(self) -> list:
-        installed_repos = []
-        with open(".pre-commit-config.yaml", encoding="utf8") as pre_commit_y:
-            pre_commit_config = yaml.load(pre_commit_y, Loader=yaml.FullLoader)
-        for repository in pre_commit_config["repos"]:
-            installed_repos.append(repository["repo"])
-        return installed_repos
-
     def __require_colrev_hooks_installed(self) -> bool:
         required_hooks = [
             "colrev-hooks-check",
@@ -212,45 +147,46 @@ class Checker:
                 f"missing hooks in .pre-commit-config.yaml ({', '.join(missing_hooks)})"
             )
 
-        pch_file = Path(".git/hooks/pre-commit")
-        if pch_file.is_file():
-            with open(pch_file, encoding="utf8") as file:
-                if "File generated by pre-commit" not in file.read(4096):
-                    raise colrev_exceptions.RepoSetupError(
-                        "pre-commit hooks not installed (use pre-commit install)"
-                    )
-        else:
-            raise colrev_exceptions.RepoSetupError(
-                "pre-commit hooks not installed (use pre-commit install)"
-            )
+        if not self.review_manager.in_ci_environment():
+            pch_file = Path(".git/hooks/pre-commit")
+            if pch_file.is_file():
+                with open(pch_file, encoding="utf8") as file:
+                    if "File generated by pre-commit" not in file.read(4096):
+                        raise colrev_exceptions.RepoSetupError(
+                            "pre-commit hooks not installed (use pre-commit install)"
+                        )
+            else:
+                raise colrev_exceptions.RepoSetupError(
+                    "pre-commit hooks not installed (use pre-commit install)"
+                )
 
-        psh_file = Path(".git/hooks/pre-push")
-        if psh_file.is_file():
-            with open(psh_file, encoding="utf8") as file:
-                if "File generated by pre-commit" not in file.read(4096):
-                    raise colrev_exceptions.RepoSetupError(
-                        "pre-commit push hooks not installed "
-                        "(use pre-commit install --hook-type pre-push)"
-                    )
-        else:
-            raise colrev_exceptions.RepoSetupError(
-                "pre-commit push hooks not installed "
-                "(use pre-commit install --hook-type pre-push)"
-            )
+            psh_file = Path(".git/hooks/pre-push")
+            if psh_file.is_file():
+                with open(psh_file, encoding="utf8") as file:
+                    if "File generated by pre-commit" not in file.read(4096):
+                        raise colrev_exceptions.RepoSetupError(
+                            "pre-commit push hooks not installed "
+                            "(use pre-commit install --hook-type pre-push)"
+                        )
+            else:
+                raise colrev_exceptions.RepoSetupError(
+                    "pre-commit push hooks not installed "
+                    "(use pre-commit install --hook-type pre-push)"
+                )
 
-        pcmh_file = Path(".git/hooks/prepare-commit-msg")
-        if pcmh_file.is_file():
-            with open(pcmh_file, encoding="utf8") as file:
-                if "File generated by pre-commit" not in file.read(4096):
-                    raise colrev_exceptions.RepoSetupError(
-                        "pre-commit prepare-commit-msg hooks not installed "
-                        "(use pre-commit install --hook-type prepare-commit-msg)"
-                    )
-        else:
-            raise colrev_exceptions.RepoSetupError(
-                "pre-commit prepare-commit-msg hooks not installed "
-                "(use pre-commit install --hook-type prepare-commit-msg)"
-            )
+            pcmh_file = Path(".git/hooks/prepare-commit-msg")
+            if pcmh_file.is_file():
+                with open(pcmh_file, encoding="utf8") as file:
+                    if "File generated by pre-commit" not in file.read(4096):
+                        raise colrev_exceptions.RepoSetupError(
+                            "pre-commit prepare-commit-msg hooks not installed "
+                            "(use pre-commit install --hook-type prepare-commit-msg)"
+                        )
+            else:
+                raise colrev_exceptions.RepoSetupError(
+                    "pre-commit prepare-commit-msg hooks not installed "
+                    "(use pre-commit install --hook-type prepare-commit-msg)"
+                )
 
         return True
 
@@ -358,7 +294,6 @@ class Checker:
             for [record_id, status, screen_crit] in status_data[
                 "screening_criteria_list"
             ]:
-
                 if status not in colrev.record.RecordState.get_post_x_states(
                     state=colrev.record.RecordState.rev_included
                 ):
@@ -510,7 +445,6 @@ class Checker:
     def check_sources(self) -> None:
         """Check the sources"""
         for source in self.review_manager.settings.sources:
-
             if not source.filename.is_file():
                 self.review_manager.logger.debug(
                     f"Search details without file: {source.filename}"
@@ -539,7 +473,6 @@ class Checker:
         status: colrev.record.RecordState,
         status_data: dict,
     ) -> dict:
-
         prior_status = []
         if "colrev_status" in prior:
             prior_status = [
@@ -577,7 +510,6 @@ class Checker:
         return status_transition
 
     def __retrieve_status_data(self, *, prior: dict, records: dict) -> dict:
-
         status_data: dict = {
             "pdf_not_exists": [],
             "status_fields": [],
