@@ -11,7 +11,7 @@ import time
 import typing
 from copy import deepcopy
 from pathlib import Path
-from typing import TYPE_CHECKING
+from typing import Optional
 
 import git
 import pybtex.errors
@@ -21,14 +21,18 @@ from pybtex.database import Person
 from pybtex.database.input import bibtex
 from tqdm import tqdm
 
+import colrev.env.language_service
 import colrev.env.utils
 import colrev.exceptions as colrev_exceptions
 import colrev.operation
 import colrev.record
 import colrev.settings
 
-if TYPE_CHECKING:
-    import colrev.review_manager
+if False:  # pylint: disable=using-constant-test
+    from typing import TYPE_CHECKING
+
+    if TYPE_CHECKING:
+        import colrev.review_manager
 
 # pylint: disable=too-many-public-methods
 # pylint: disable=too-many-lines
@@ -38,13 +42,36 @@ class Dataset:
     """The CoLRev dataset (records and their history in git)"""
 
     RECORDS_FILE_RELATIVE = Path("data/records.bib")
+    GIT_IGNORE_FILE_RELATIVE = Path(".gitignore")
+    DEFAULT_GIT_IGNORE_ITEMS = [
+        "*.bib.sav",
+        "venv",
+        ".corrections",
+        "data/pdfs",
+        ".report.log",
+        "__pycache__",
+        "output",
+        "data/pdf_get_man/missing_pdf_files.csv",
+        "data/.tei/",
+        "data/prep_man/records_prep_man.bib",
+    ]
+    DEPRECATED_GIT_IGNORE_ITEMS = [
+        "missing_pdf_files.csv",
+        "manual_cleansing_statistics.csv",
+        ".references_learned_settings",
+        "pdfs",
+        ".tei",
+        "data.csv",
+        "requests_cache.sqlite",
+    ]
+
     records_file: Path
     __git_repo: git.Repo
 
     def __init__(self, *, review_manager: colrev.review_manager.ReviewManager) -> None:
-
         self.review_manager = review_manager
         self.records_file = review_manager.path / self.RECORDS_FILE_RELATIVE
+        self.git_ignore_file = review_manager.path / self.GIT_IGNORE_FILE_RELATIVE
 
         try:
             self.__git_repo = git.Repo(self.review_manager.path)
@@ -57,8 +84,35 @@ class Dataset:
             pybtex.io.stderr = temp_f
 
         self.masterdata_restrictions = self.__get_masterdata_restrictions()
+        self.update_gitignore(
+            add=self.DEFAULT_GIT_IGNORE_ITEMS, remove=self.DEPRECATED_GIT_IGNORE_ITEMS
+        )
 
-    def get_origin_state_dict(self, *, file_object: io.StringIO = None) -> dict:
+    def update_gitignore(
+        self, *, add: typing.Optional[list] = None, remove: typing.Optional[list] = None
+    ) -> None:
+        """Update the gitignore file by adding or removing particular paths"""
+        # The following may print warnings...
+
+        if self.git_ignore_file.is_file():
+            gitignore_content = self.git_ignore_file.read_text(encoding="utf-8")
+        else:
+            gitignore_content = ""
+        ignored_items = gitignore_content.splitlines()
+        if remove:
+            ignored_items = [x for x in ignored_items if x not in remove]
+        if add:
+            ignored_items = ignored_items + [
+                str(a) for a in add if str(a) not in ignored_items
+            ]
+
+        with self.git_ignore_file.open("w", encoding="utf-8") as file:
+            file.write("\n".join(ignored_items) + "\n")
+        self.add_changes(path=self.git_ignore_file)
+
+    def get_origin_state_dict(
+        self, *, file_object: Optional[io.StringIO] = None
+    ) -> dict:
         """Get the origin_state_dict (to determine state transitions efficiently)
 
         {'30_example_records.bib/Staehr2010': <RecordState.pdf_not_available: 10>,}
@@ -108,7 +162,6 @@ class Dataset:
         found_but_not_changed = False
         skipped_prior_commits = False  # if no commit_sha provided
         for commit in self.__git_repo.iter_commits():
-
             if commit_sha:
                 if not skipped_prior_commits:
                     if not found_but_not_changed:
@@ -233,7 +286,7 @@ class Dataset:
                         print(f"problem with data_provenance_item {item}")
 
         else:
-            print(f"error loading dict_field: {key}")
+            print(f"error loading dict_field: {field}")
 
         return return_dict
 
@@ -296,7 +349,9 @@ class Dataset:
 
         return records_dict
 
-    def __read_record_header_items(self, *, file_object: typing.TextIO = None) -> list:
+    def __read_record_header_items(
+        self, *, file_object: Optional[typing.TextIO] = None
+    ) -> list:
         # Note : more than 10x faster than the pybtex part of load_records_dict()
 
         # pylint: disable=too-many-statements
@@ -388,7 +443,11 @@ class Dataset:
         ]
 
     def load_records_dict(
-        self, *, file_path: Path = None, load_str: str = None, header_only: bool = False
+        self,
+        *,
+        file_path: Optional[Path] = None,
+        load_str: Optional[str] = None,
+        header_only: bool = False,
     ) -> dict:
         """Load the records
 
@@ -405,11 +464,10 @@ class Dataset:
         }
         """
 
-        pybtex.errors.set_strict_mode(False)
-
         if self.review_manager.notified_next_operation is None:
             raise colrev_exceptions.ReviewManagerNotNofiedError()
 
+        pybtex.errors.set_strict_mode(False)
         if header_only:
             # Note : currently not parsing screening_criteria to settings.ScreeningCriterion
             # to optimize performance
@@ -419,12 +477,11 @@ class Dataset:
             record_header_dict = {r["ID"]: r for r in record_header_list}
             return record_header_dict
 
-        parser = bibtex.Parser()
-
         if file_path:
             with open(file_path, encoding="utf-8") as file:
                 load_str = file.read()
 
+        parser = bibtex.Parser()
         if load_str:
             bib_data = parser.parse_string(load_str)
             records_dict = self.parse_records_dict(records_dict=bib_data.entries)
@@ -437,7 +494,12 @@ class Dataset:
 
         return records_dict
 
-    def parse_bibtex_str(self, *, recs_dict_in: dict) -> str:
+    @classmethod
+    def parse_bibtex_str(
+        cls,
+        *,
+        recs_dict_in: dict,
+    ) -> str:
         """Parse a records_dict to a BiBTex string"""
 
         # Note: we need a deepcopy because the parsing modifies dicts
@@ -448,7 +510,7 @@ class Dataset:
             return f",\n   {field} {padd} = {{{value}}}"
 
         bibtex_str = ""
-
+        language_service = colrev.env.language_service.LanguageService()
         first = True
         for record_id, record_dict in recs_dict.items():
             if not first:
@@ -457,22 +519,12 @@ class Dataset:
 
             bibtex_str += f"@{record_dict['ENTRYTYPE']}{{{record_id}"
 
-            if "language" in record_dict:
-                # convert to ISO 639-3
-                # gh_issue https://github.com/geritwagner/colrev/issues/64
-                # other languages/more systematically
-                # (see database_connectors) > in record.py?
-                if "en" == record_dict["language"]:
-                    record_dict["language"] = record_dict["language"].replace(
-                        "en", "eng"
-                    )
-
-                if len(record_dict["language"]) != 3:
-                    self.review_manager.logger.warn(
-                        "language (%s) of %s not in ISO 639-3 format",
-                        record_dict["language"],
-                        record_dict["ID"],
-                    )
+            try:
+                language_service.unify_to_iso_639_3_language_codes(
+                    record=colrev.record.Record(data=record_dict)
+                )
+            except colrev_exceptions.InvalidLanguageCodeException:
+                del record_dict["language"]
 
             field_order = [
                 "colrev_origin",  # must be in second line
@@ -598,7 +650,9 @@ class Dataset:
             return
         self.save_records_dict_to_file(records=records, save_path=self.records_file)
 
-    def read_next_record(self, *, conditions: list = None) -> typing.Iterator[dict]:
+    def read_next_record(
+        self, *, conditions: Optional[list] = None
+    ) -> typing.Iterator[dict]:
         """Read records (Iterator) based on condition"""
 
         # Note : matches conditions connected with 'OR'
@@ -633,7 +687,7 @@ class Dataset:
                         record_dict=record_dict
                     )
                 )
-                record.update_metadata_status(review_manager=self.review_manager)
+                record.update_metadata_status()
 
             if record_dict["colrev_status"] == colrev.record.RecordState.pdf_prepared:
                 record.reset_pdf_provenance_notes()
@@ -672,11 +726,9 @@ class Dataset:
     def __generate_temp_id(
         self, *, local_index: colrev.env.local_index.LocalIndex, record_dict: dict
     ) -> str:
-
         # pylint: disable=too-many-branches
 
         try:
-
             retrieved_record = local_index.retrieve(record_dict=record_dict)
             temp_id = retrieved_record["ID"]
 
@@ -684,8 +736,10 @@ class Dataset:
             if "curated_metadata" in str(self.review_manager.path):
                 raise colrev_exceptions.RecordNotInIndexException()
 
-        except colrev_exceptions.RecordNotInIndexException:
-
+        except (
+            colrev_exceptions.RecordNotInIndexException,
+            colrev_exceptions.NotEnoughDataToIdentifyException,
+        ):
             if "" != record_dict.get("author", record_dict.get("editor", "")):
                 authors_string = record_dict.get(
                     "author", record_dict.get("editor", "Anonymous")
@@ -768,7 +822,7 @@ class Dataset:
         *,
         local_index: colrev.env.local_index.LocalIndex,
         record_dict: dict,
-        existing_ids: list = None,
+        existing_ids: Optional[list] = None,
     ) -> str:
         """Generate a blacklist to avoid setting duplicate IDs"""
 
@@ -794,7 +848,9 @@ class Dataset:
 
         return temp_id
 
-    def set_ids(self, *, records: dict = None, selected_ids: list = None) -> dict:
+    def set_ids(
+        self, *, records: Optional[dict] = None, selected_ids: Optional[list] = None
+    ) -> dict:
         """Set the IDs of records according to predefined formats or
         according to the LocalIndex"""
         # pylint: disable=redefined-outer-name
@@ -811,9 +867,6 @@ class Dataset:
 
         for record_id in tqdm(list(records.keys())):
             record_dict = records[record_id]
-            record = colrev.record.Record(data=record_dict)
-            if record.masterdata_is_curated():
-                continue
             if selected_ids is not None:
                 if record_id not in selected_ids:
                     continue
@@ -874,7 +927,7 @@ class Dataset:
         curated_endpoints = [
             x
             for x in self.review_manager.settings.data.data_package_endpoints
-            if x["endpoint"] == "colrev_built_in.colrev_curation"
+            if x["endpoint"] == "colrev.colrev_curation"
         ]
         if curated_endpoints:
             curated_endpoint = curated_endpoints[0]
@@ -918,7 +971,7 @@ class Dataset:
         return self.__git_repo
 
     def has_changes(
-        self, *, relative_path: Path = None, change_type: str = "all"
+        self, *, relative_path: Optional[Path] = None, change_type: str = "all"
     ) -> bool:
         """Check whether the relative path (or the git repository) has changes"""
 
@@ -947,14 +1000,20 @@ class Dataset:
     def add_changes(self, *, path: Path, remove: bool = False) -> None:
         """Add changed file to git"""
 
+        if path.is_absolute():
+            path = path.relative_to(self.review_manager.path)
+
         while (self.review_manager.path / Path(".git/index.lock")).is_file():
             time.sleep(0.5)
             print("Waiting for previous git operation to complete")
 
-        if remove:
-            self.__git_repo.index.remove([str(path)])
-        else:
-            self.__git_repo.index.add([str(path)])
+        try:
+            if remove:
+                self.__git_repo.index.remove([str(path)])
+            else:
+                self.__git_repo.index.add([str(path)])
+        except GitCommandError:
+            pass
 
     def get_untracked_files(self) -> list:
         """Get the files that are untracked by git"""
@@ -1040,16 +1099,16 @@ class Dataset:
         if not self.__git_repo.is_dirty():
             self.review_manager.reset_report_logger()
 
-    def get_last_commit_sha(self) -> str:
+    def get_last_commit_sha(self) -> str:  # pragma: no cover
         """Get the last commit sha"""
         return str(self.__git_repo.head.commit.hexsha)
 
-    def get_tree_hash(self) -> str:
+    def get_tree_hash(self) -> str:  # pragma: no cover
         """Get the current tree hash"""
         tree_hash = self.__git_repo.git.execute(["git", "write-tree"])
         return str(tree_hash)
 
-    def __get_remote_commit_differences(self) -> list:
+    def __get_remote_commit_differences(self) -> list:  # pragma: no cover
         origin = self.__git_repo.remotes.origin
         if origin.exists():
             try:
@@ -1073,7 +1132,7 @@ class Dataset:
 
         return [nr_commits_behind, nr_commits_ahead]
 
-    def behind_remote(self) -> bool:
+    def behind_remote(self) -> bool:  # pragma: no cover
         """Check whether the repository is behind the remote"""
         nr_commits_behind = 0
         connected_remote = 0 != len(self.__git_repo.remotes)
@@ -1088,7 +1147,7 @@ class Dataset:
             return True
         return False
 
-    def remote_ahead(self) -> bool:
+    def remote_ahead(self) -> bool:  # pragma: no cover
         """Check whether the remote is ahead"""
         connected_remote = 0 != len(self.__git_repo.remotes)
         if connected_remote:
@@ -1102,13 +1161,13 @@ class Dataset:
             return True
         return False
 
-    def pull_if_repo_clean(self) -> None:
+    def pull_if_repo_clean(self) -> None:  # pragma: no cover
         """Pull project if repository is clean"""
         if not self.__git_repo.is_dirty():
             origin = self.__git_repo.remotes.origin
             origin.pull()
 
-    def get_remote_url(self) -> str:
+    def get_remote_url(self) -> str:  # pragma: no cover
         """Get the remote url"""
         remote_url = "NA"
         for remote in self.__git_repo.remotes:
