@@ -6,6 +6,7 @@ import collections.abc
 import dataclasses
 import importlib.util
 import json
+import os
 import sys
 import typing
 from copy import deepcopy
@@ -23,6 +24,9 @@ import colrev.exceptions as colrev_exceptions
 import colrev.operation
 import colrev.record
 import colrev.settings
+import colrev.ui_cli.cli_colors as colors
+
+# pylint: disable=too-many-lines
 
 
 # Inspiration for package descriptions:
@@ -35,18 +39,31 @@ class PackageEndpointType(Enum):
 
     # pylint: disable=C0103
     review_type = "review_type"
+    """Endpoint for review types"""
     load_conversion = "load_conversion"
+    """Endpoint for load conversion"""
     search_source = "search_source"
+    """Endpoint for search sources"""
     prep = "prep"
+    """Endpoint for prep"""
     prep_man = "prep_man"
+    """Endpoint for prep-man"""
     dedupe = "dedupe"
+    """Endpoint for dedupe"""
     prescreen = "prescreen"
+    """Endpoint for prescreen"""
     pdf_get = "pdf_get"
+    """Endpoint for pdf-get"""
     pdf_get_man = "pdf_get_man"
+    """Endpoint for pdf-get-man"""
     pdf_prep = "pdf_prep"
+    """Endpoint for pdf-prep"""
     pdf_prep_man = "pdf_prep_man"
+    """Endpoint for pdf-prep-man"""
     screen = "screen"
+    """Endpoint for screen"""
     data = "data"
+    """Endpoint for data"""
 
 
 class ReviewTypePackageEndpointInterface(
@@ -57,7 +74,7 @@ class ReviewTypePackageEndpointInterface(
     # pylint: disable=no-self-argument
     def initialize(settings: dict) -> dict:  # type: ignore
         """Initialize the review type"""
-        return settings
+        return settings  # pragma: no cover
 
 
 class SearchSourceHeuristicStatus(Enum):
@@ -70,7 +87,7 @@ class SearchSourceHeuristicStatus(Enum):
     todo = "to_be_implemented"
 
     def __str__(self) -> str:
-        return f"{self.name}"
+        return f"{self.name}"  # pragma: no cover
 
 
 class SearchSourcePackageEndpointInterface(
@@ -88,21 +105,17 @@ class SearchSourcePackageEndpointInterface(
         """Main SearchType associated with the SearchSource"""
     )
 
+    api_search_supported = zope.interface.Attribute(
+        """Flag indicating whether API searches are supported (by the run_search method)"""
+    )
+
     heuristic_status: SearchSourceHeuristicStatus = zope.interface.Attribute(
         """The status of the SearchSource heuristic"""
     )
     short_name = zope.interface.Attribute("""Short name of the SearchSource""")
     link = zope.interface.Attribute("""Link to the SearchSource website""")
 
-    # Note : optional method
     # pylint: disable=no-self-argument
-    # def run_search(
-    #     search_operation: colrev.ops.search.Search, rerun: bool
-    # ) -> None:  # type: ignore
-    #     """Run the search operation
-    #       rerun: retrieve and update all records again (default: only receive recent ones)
-    #       """
-
     def validate_source(
         search_operation: colrev.ops.search.Search, source: colrev.settings.SearchSource
     ) -> None:  # type: ignore
@@ -113,6 +126,24 @@ class SearchSourcePackageEndpointInterface(
     def heuristic(filename: Path, data: str):  # type: ignore
         """Heuristic to identify the SearchSource"""
 
+    # pylint: disable=no-self-argument
+    def add_endpoint(search_operation: colrev.ops.search.Search, query: str):  # type: ignore
+        """Add the SearchSource as an endpoint based on a query (passed to colrev search -a)"""
+
+    # pylint: disable=no-self-argument
+    def run_search(search_operation: colrev.ops.search.Search, rerun: bool) -> None:  # type: ignore
+        """Run a search of the SearchSource"""
+
+    # pylint: disable=no-self-argument
+    def get_masterdata(  # type: ignore
+        prep_operation: colrev.ops.prep.Prep,
+        record: colrev.record.Record,
+        save_feed: bool = True,
+        timeout: int = 10,
+    ):
+        """Retrieve masterdata from the SearchSource"""
+
+    # pylint: disable=no-self-argument
     def load_fixes(  # type: ignore
         load_operation: colrev.ops.load.Load,
         source: colrev.settings.SearchSource,
@@ -210,8 +241,7 @@ class PDFGetPackageEndpointInterface(
     # pylint: disable=no-self-argument
     def get_pdf(pdf_get_operation: colrev.ops.pdf_get.PDFGet, record: dict) -> dict:  # type: ignore
         """Run the pdf-get operation"""
-
-        return record
+        return record  # pragma: no cover
 
 
 class PDFGetManPackageEndpointInterface(
@@ -226,7 +256,7 @@ class PDFGetManPackageEndpointInterface(
         pdf_get_man_operation: colrev.ops.pdf_get_man.PDFGetMan, records: dict
     ) -> dict:
         """Run the pdf-get-man operation"""
-        return records
+        return records  # pragma: no cover
 
 
 class PDFPrepPackageEndpointInterface(
@@ -244,7 +274,7 @@ class PDFPrepPackageEndpointInterface(
         pad: int,
     ) -> dict:
         """Run the prep-pdf operation"""
-        return record.data
+        return record.data  # pragma: no cover
 
 
 class PDFPrepManPackageEndpointInterface(
@@ -259,7 +289,7 @@ class PDFPrepManPackageEndpointInterface(
         pdf_prep_man_operation: colrev.ops.prep_man.PrepMan, records: dict
     ) -> dict:
         """Run the prep-man operation"""
-        return records
+        return records  # pragma: no cover
 
 
 class ScreenPackageEndpointInterface(
@@ -287,7 +317,7 @@ class DataPackageEndpointInterface(
     # pylint: disable=no-method-argument
     def get_default_setup() -> dict:  # type: ignore
         """Get the default setup for the data package endpoint"""
-        return {}
+        return {}  # pragma: no cover
 
     def update_data(  # type: ignore
         data_operation: colrev.ops.data.Data,
@@ -303,7 +333,8 @@ class DataPackageEndpointInterface(
         endpoint_identifier: str,
     ) -> None:
         """Update the record status matrix,
-        i.e., indicate whether the record is rev_synthesized for the given endpoint_identifier"""
+        i.e., indicate whether the record is rev_synthesized for the given endpoint_identifier
+        """
 
     def get_advice(  # type: ignore
         review_manager: colrev.review_manager.ReviewManager,
@@ -430,12 +461,21 @@ class PackageManager:
 
     package: typing.Dict[str, typing.Dict[str, typing.Dict]]
 
-    def __init__(self) -> None:
+    def __init__(self, *, verbose: bool = False) -> None:
+        self.verbose = verbose
         self.packages = self.__load_package_endpoints_index()
         self.__flag_installed_packages()
+        colrev_spec = importlib.util.find_spec("colrev")
+        if colrev_spec is None:  # pragma: no cover
+            raise colrev_exceptions.MissingDependencyError(dep="colrev")
+        if colrev_spec.origin is None:  # pragma: no cover
+            raise colrev_exceptions.MissingDependencyError(dep="colrev")
+        self.__colrev_path = Path(colrev_spec.origin).parents[1]
+        self.__package_endpoints_json_file = self.__colrev_path / Path(
+            "colrev/template/package_endpoints.json"
+        )
 
     def __load_package_endpoints_index(self) -> dict:
-
         filedata = colrev.env.utils.get_package_file_content(
             file_path=Path("template/package_endpoints.json")
         )
@@ -467,7 +507,13 @@ class PackageManager:
                         package_type=package_type, package_identifier=package_identifier
                     )
                     package["installed"] = True
-                except (AttributeError, ModuleNotFoundError):
+                except (AttributeError, ModuleNotFoundError) as exc:
+                    if hasattr(exc, "name"):
+                        if package_identifier.split(".")[0] != exc.name:  # type: ignore
+                            if self.verbose:
+                                raise exc
+                            print(f"Error loading package {package_identifier}: {exc}")
+
                     package["installed"] = False
 
     def __replace_path_by_str(self, *, orig_dict):  # type: ignore
@@ -515,7 +561,6 @@ class PackageManager:
         for parameter in [
             i for i in settings_class.__annotations__.keys() if i[:1] != "_"
         ]:
-
             # # default value: determined from class.__dict__
             # # merging_non_dup_threshold: float= 0.7
             # if parameter in settings_class.__dict__:
@@ -556,7 +601,7 @@ class PackageManager:
                             "options"
                         ] = settings_class._details[parameter]["options"]
 
-        # gh_issue https://github.com/geritwagner/colrev/issues/66
+        # gh_issue https://github.com/CoLRev-Ecosystem/colrev/issues/66
         # apply validation when parsing settings during package init (based on _details)
         # later : package version?
 
@@ -651,12 +696,11 @@ class PackageManager:
         package_type: PackageEndpointType,
         ignore_not_available: bool,
     ) -> typing.Dict:
-
         # avoid changes in the config
         selected_packages = deepcopy(selected_packages)
+
         packages_dict: typing.Dict = {}
         for selected_package in selected_packages:
-
             package_identifier = selected_package["endpoint"].lower()
             packages_dict[package_identifier] = {}
 
@@ -723,11 +767,14 @@ class PackageManager:
         operation: colrev.operation.Operation,
         ignore_not_available: bool = False,
         instantiate_objects: bool = True,
+        only_ci_supported: bool = False,
     ) -> typing.Dict[str, typing.Dict[str, typing.Any]]:
         """Load the packages for a particular package_type"""
 
         # pylint: disable=import-outside-toplevel
         # pylint: disable=unnecessary-dict-index-lookup
+        # pylint: disable=too-many-locals
+
         # Note : when iterating over packages_dict.items(),
         # changes to the values (or del k) would not persist
 
@@ -750,6 +797,7 @@ class PackageManager:
 
         package_details = self.package_type_overview[package_type]
         endpoint_class = package_details["import_name"]  # type: ignore
+        to_remove = []
         for package_identifier, package_class in packages_dict.items():
             params = {
                 package_details["operation_name"]: operation,
@@ -764,15 +812,38 @@ class PackageManager:
                 )
 
             if instantiate_objects:
-                packages_dict[package_identifier] = package_class["endpoint"](**params)
-                verifyObject(endpoint_class, packages_dict[package_identifier])
+                try:
+                    packages_dict[package_identifier] = package_class["endpoint"](
+                        **params
+                    )
+                    if only_ci_supported:
+                        if not packages_dict[package_identifier].ci_supported:
+                            to_remove.append(package_identifier)
+                            continue
+                    verifyObject(endpoint_class, packages_dict[package_identifier])
+                except colrev_exceptions.ServiceNotAvailableException as sna_exc:
+                    if "docker" == sna_exc.dep:
+                        print(
+                            f"{colors.ORANGE}Docker not available. Deactivate "
+                            f"{package_identifier}{colors.END}"
+                        )
+                        to_remove.append(package_identifier)
+                    else:
+                        raise sna_exc
             else:
                 packages_dict[package_identifier] = package_class["endpoint"]
 
+        for i_to_remove in to_remove:
+            del packages_dict[i_to_remove]
+
         return packages_dict
 
     def __add_package_endpoints(
-        self, *, package_endpoints_json: dict, package_endpoints: dict
+        self,
+        *,
+        package_endpoints_json: dict,
+        package_endpoints: dict,
+        package_status: dict,
     ) -> None:
         for endpoint_type, endpoint_list in package_endpoints_json.items():
             if endpoint_type not in package_endpoints["endpoints"]:
@@ -784,7 +855,6 @@ class PackageManager:
             )
             print(f" load {endpoint_type}: \n -  {package_list}")
             for endpoint_item in package_endpoints["endpoints"][endpoint_type]:
-
                 self.packages[PackageEndpointType[endpoint_type]][
                     endpoint_item["package_endpoint_identifier"]
                 ] = {"endpoint": endpoint_item["endpoint"], "installed": True}
@@ -793,6 +863,37 @@ class PackageManager:
                     package_identifier=endpoint_item["package_endpoint_identifier"],
                 )
 
+                # Add development status information (if available on package_status)
+                e_list = [
+                    x
+                    for x in package_status[endpoint_type]
+                    if x["package_endpoint_identifier"]
+                    == endpoint_item["package_endpoint_identifier"]
+                ]
+                if e_list:
+                    endpoint_item["status"] = e_list[0]["status"]
+                else:
+                    package_status[endpoint_type].append(
+                        {
+                            "package_endpoint_identifier": endpoint_item[
+                                "package_endpoint_identifier"
+                            ],
+                            "status": "RED",
+                        }
+                    )
+                    endpoint_item["status"] = "RED"
+
+                endpoint_item["status"] = (
+                    endpoint_item["status"]
+                    .replace("GREEN", "🟢")
+                    .replace("ORANGE", "🟡")
+                    .replace("RED", "🔴")
+                )
+                endpoint_item["status_linked"] = (
+                    f"`{endpoint_item['status']} "
+                    + "<https://colrev.readthedocs.io/en/latest/foundations/roadmap.html>`_"
+                )
+
                 # Generate the contents displayed in the docs (see "datatemplate:json")
                 # load short_description dynamically...
                 short_description = endpoint.__doc__
@@ -800,12 +901,15 @@ class PackageManager:
                     short_description = endpoint.__doc__.split("\n")[0]
                 endpoint_item["short_description"] = short_description
 
+                endpoint_item["ci_supported"] = endpoint.ci_supported
+
                 code_link = (
-                    "https://github.com/geritwagner/colrev/blob/main/"
+                    "https://github.com/CoLRev-Ecosystem/colrev/blob/main/"
                     + endpoint_item["endpoint"].replace(".", "/")
                 )
+                # In separate packages, we the main readme.md file should be used
                 code_link = code_link[: code_link.rfind("/")]
-                code_link += ".py"
+                code_link += ".md"
                 if hasattr(endpoint, "link"):
                     link = endpoint.link
                 else:
@@ -824,73 +928,71 @@ class PackageManager:
                         .replace("na", "NA")
                     )
 
-                    if hasattr(endpoint, "run_search"):
+                    if endpoint.api_search_supported:
                         endpoint_item["api_search"] = "Supported"
                     else:
                         endpoint_item["api_search"] = "NA"
-                    endpoint_item[
-                        "link"
-                    ] = f"`{endpoint.short_name} <{endpoint.link}>`_"
+                    endpoint_item["link"] = f"`{endpoint.short_name} <{link}>`_"
                     endpoint_item["instructions"] = f"`Instructions <{code_link}>`_"
 
             endpoint_list += package_endpoints["endpoints"][endpoint_type]
 
+    def __load_packages_json(self) -> list:
+        filedata = colrev.env.utils.get_package_file_content(
+            file_path=Path("template/packages.json")
+        )
+        if not filedata:  # pragma: no cover
+            raise colrev_exceptions.CoLRevException(
+                "Package index not available (colrev/template/packages.json)"
+            )
+        packages = json.loads(filedata.decode("utf-8"))
+        return packages
+
+    def __load_package_status_json(self) -> dict:
+        filedata = colrev.env.utils.get_package_file_content(
+            file_path=Path("template/package_status.json")
+        )
+        if not filedata:  # pragma: no cover
+            raise colrev_exceptions.CoLRevException(
+                "Package index not available (colrev/template/package_status.json)"
+            )
+        packages = json.loads(filedata.decode("utf-8"))
+        return packages
+
     def update_package_list(self) -> None:
         """Generates the template/package_endpoints.json
         based on the packages in template/packages.json
         and the endpoints.json files in the top directory of each package."""
 
-        # pylint: disable=too-many-locals
-        # pylint: disable=too-many-branches
-
-        colrev_spec = importlib.util.find_spec("colrev")
-
-        # Check that we are in the colrev repo
-        if colrev_spec is not None:
-            assert Path.cwd() == Path(colrev_spec.origin).parents[1]  # type: ignore
-        else:
-            print("Could not load colrev module")
-            return
-
-        filedata = colrev.env.utils.get_package_file_content(
-            file_path=Path("template/packages.json")
-        )
-        if not filedata:
-            raise colrev_exceptions.CoLRevException(
-                "Package index not available (colrev/template/package_endpoints.json)"
-            )
-
-        package_endpoints_json_file = Path(Path(colrev_spec.origin).parent) / Path(  # type: ignore
-            "template/package_endpoints.json"
-        )
-        if package_endpoints_json_file.is_file():
-            package_endpoints_json_file.unlink()
+        os.chdir(self.__colrev_path)
+        packages = self.__load_packages_json()
+        package_status = self.__load_package_status_json()
+        self.__package_endpoints_json_file.unlink(missing_ok=True)
 
         package_endpoints_json: typing.Dict[str, list] = {
             x.name: [] for x in self.package_type_overview
         }
-
-        packages = json.loads(filedata.decode("utf-8"))
         for package in packages:
             print(f'Loading package endpoints from {package["module"]}')
             module_spec = importlib.util.find_spec(package["module"])
             endpoints_path = Path(module_spec.origin).parents[1] / Path(  # type:ignore
-                "endpoints.json"
+                ".colrev_endpoints.json"
             )
-            if not endpoints_path.is_file():
+            if not endpoints_path.is_file():  # pragma: no cover
                 print(f"File does not exist: {endpoints_path}")
                 continue
 
             try:
                 with open(endpoints_path, encoding="utf-8") as file:
                     package_endpoints = json.load(file)
-            except json.decoder.JSONDecodeError as exc:
+            except json.decoder.JSONDecodeError as exc:  # pragma: no cover
                 print(f"Invalid json {exc}")
                 continue
 
             self.__add_package_endpoints(
                 package_endpoints_json=package_endpoints_json,
                 package_endpoints=package_endpoints,
+                package_status=package_status,
             )
 
         for key in package_endpoints_json.keys():
@@ -900,8 +1002,16 @@ class PackageManager:
             )
 
         json_object = json.dumps(package_endpoints_json, indent=4)
-        with open(package_endpoints_json_file, "w", encoding="utf-8") as file:
+        with open(self.__package_endpoints_json_file, "w", encoding="utf-8") as file:
             file.write(json_object)
+            file.write("\n")  # to avoid pre-commit/eof-fix changes
+
+        json_object = json.dumps(package_status, indent=4)
+        with open(
+            Path("colrev/template/package_status.json"), "w", encoding="utf-8"
+        ) as file:
+            file.write(json_object)
+            file.write("\n")  # to avoid pre-commit/eof-fix changes
 
 
 if __name__ == "__main__":
