@@ -9,7 +9,6 @@ import statistics
 import typing
 from dataclasses import dataclass
 from pathlib import Path
-from typing import TYPE_CHECKING
 
 import dedupe as dedupe_io
 import pandas as pd
@@ -26,8 +25,11 @@ import colrev.ops.built_in.dedupe.utils
 import colrev.record
 import colrev.ui_cli.cli_colors as colors
 
-if TYPE_CHECKING:
-    import colrev.ops.dedupe
+if False:  # pylint: disable=using-constant-test
+    from typing import TYPE_CHECKING
+
+    if TYPE_CHECKING:
+        import colrev.ops.dedupe
 
 # pylint: disable=too-few-public-methods
 # pylint: disable=too-many-lines
@@ -44,6 +46,8 @@ class ActiveLearningDedupeTraining(JsonSchemaMixin):
 
     deduper: dedupe_io.Deduper
 
+    ci_supported: bool = False
+
     # Code based on
     # https://github.com/dedupeio/dedupe-examples/blob/master/csv_example/csv_example.py
 
@@ -53,7 +57,6 @@ class ActiveLearningDedupeTraining(JsonSchemaMixin):
         dedupe_operation: colrev.ops.dedupe.Dedupe,  # pylint: disable=unused-argument
         settings: dict,
     ):
-
         logging.basicConfig()
         logging.getLogger("dedupe.canopy_index").setLevel(logging.WARNING)
 
@@ -65,6 +68,11 @@ class ActiveLearningDedupeTraining(JsonSchemaMixin):
             dedupe_operation.review_manager.path / self.SETTINGS_FILE_RELATIVE
         )
 
+        if hasattr(dedupe_operation.review_manager, "dataset"):
+            dedupe_operation.review_manager.dataset.update_gitignore(
+                add=[self.SETTINGS_FILE_RELATIVE]
+            )
+
     def __setup_active_learning_dedupe(
         self,
         *,
@@ -105,7 +113,7 @@ class ActiveLearningDedupeTraining(JsonSchemaMixin):
                 f"Selecting a random sample of {max_training_sample_size}"
                 " to avoid memory problems"
             )
-            # gh_issue https://github.com/geritwagner/colrev/issues/37
+            # gh_issue https://github.com/CoLRev-Ecosystem/colrev/issues/37
             # consider similar proportions of post-md_processed/md_prepared?
             keys = random.sample(list(data_d.keys()), max_training_sample_size)
             data_d = {key: data_d[key] for key in keys}
@@ -128,7 +136,7 @@ class ActiveLearningDedupeTraining(JsonSchemaMixin):
 
         # Training
 
-        # gh_issue https://github.com/geritwagner/colrev/issues/37
+        # gh_issue https://github.com/CoLRev-Ecosystem/colrev/issues/37
         # creating a corpus from all fields may create memory issues...
 
         # Define the fields dedupe will pay attention to
@@ -207,7 +215,7 @@ class ActiveLearningDedupeTraining(JsonSchemaMixin):
                 "Sample size too small for active learning. "
                 "Use simple_dedupe instead:\n"
                 f"{colors.ORANGE}  colrev settings -m 'dedupe.dedupe_package_endpoints="
-                f'[{{"endpoint":"colrev_built_in.simple_dedupe"}}]\'{colors.END}'
+                f'[{{"endpoint":"colrev.simple_dedupe"}}]\'{colors.END}'
             )
 
         if self.training_file.is_file():
@@ -242,12 +250,10 @@ class ActiveLearningDedupeTraining(JsonSchemaMixin):
         dedupe_operation: colrev.ops.dedupe.Dedupe,
         results: list,
     ) -> None:
-
         if (
             self.__get_nr_duplicates(result_list=results) > 10
             and self.__get_nr_non_duplicates(result_list=results) > 10
         ):
-
             dedupe_operation.apply_merges(results=results, complete_dedupe=False)
 
             dedupe_operation.review_manager.logger.info("Training deduper.")
@@ -349,7 +355,6 @@ class ActiveLearningDedupeTraining(JsonSchemaMixin):
         manual_dedupe_decision_list = []
 
         while not finished:
-
             if use_previous:
                 record_pair, _ = examples_buffer.pop(0)
                 use_previous = False
@@ -513,6 +518,8 @@ class ActiveLearningDedupeTraining(JsonSchemaMixin):
 class ActiveLearningDedupeAutomated(JsonSchemaMixin):
     """Applies trained (active learning) model"""
 
+    ci_supported: bool = False
+
     @dataclass
     class ActiveLearningSettings(
         colrev.env.package_manager.DefaultSettings, JsonSchemaMixin
@@ -572,7 +579,6 @@ class ActiveLearningDedupeAutomated(JsonSchemaMixin):
             )
 
         for dedupe_decision in dedupe_decision_list:
-
             if len(dedupe_decision["records"]) == 0:
                 continue
 
@@ -590,7 +596,6 @@ class ActiveLearningDedupeAutomated(JsonSchemaMixin):
                 continue
 
             for dupe_rec in dedupe_decision["records"]:
-
                 orig_propagated = dedupe_operation.review_manager.dataset.propagated_id(
                     record_id=orig_rec
                 )
@@ -599,7 +604,6 @@ class ActiveLearningDedupeAutomated(JsonSchemaMixin):
                 )
 
                 if not orig_propagated and not dupe_propagated:
-
                     # Use the record['ID'] without appended letters if possible
                     # Set orig_propagated=True if record_a_ID should be kept
                     if orig_rec[-1:].isnumeric() and not dupe_rec[-1:].isnumeric():
@@ -780,7 +784,6 @@ class ActiveLearningDedupeAutomated(JsonSchemaMixin):
     def __get_collected_dupes_non_dupes_from_clusters(
         self, *, clustered_dupes: list, data_d: dict
     ) -> dict:
-
         # pylint: disable=too-many-locals
 
         cluster_membership = {}
@@ -789,7 +792,6 @@ class ActiveLearningDedupeAutomated(JsonSchemaMixin):
         #  'BhaskaraBawa2021': {'Cluster ID': 353, 'confidence_score': 1.0}}
         for cluster_id, (records, scores) in enumerate(clustered_dupes):
             for record_id, score in zip(records, scores):
-
                 cluster_membership[record_id] = {
                     "cluster_id": cluster_id,
                     "confidence_score": score,
@@ -847,7 +849,6 @@ class ActiveLearningDedupeAutomated(JsonSchemaMixin):
         clustered_dupes: list,
         data_d: dict,
     ) -> None:
-
         results = self.__get_collected_dupes_non_dupes_from_clusters(
             clustered_dupes=clustered_dupes, data_d=data_d
         )
@@ -865,7 +866,6 @@ class ActiveLearningDedupeAutomated(JsonSchemaMixin):
     def __cluster_duplicates(
         self, *, dedupe_operation: colrev.ops.dedupe.Dedupe, data_d: dict
     ) -> list:
-
         # pylint: disable=too-many-locals
 
         dedupe_operation.review_manager.logger.info("Clustering duplicates...")
@@ -917,7 +917,6 @@ class ActiveLearningDedupeAutomated(JsonSchemaMixin):
             #     pass
 
         else:
-
             for field in deduper.fingerprinter.index_fields:
                 field_data = (r[field] for r in data_d.values() if field in r)
                 deduper.fingerprinter.index(field_data, field)
@@ -947,7 +946,6 @@ class ActiveLearningDedupeAutomated(JsonSchemaMixin):
             records_data = {r["ID"]: r for r in data_d.values()}
 
             def record_pairs(result_set: list[tuple]) -> typing.Iterator[tuple]:
-
                 for row in result_set:
                     id_a, id_b = row
                     record_a = (id_a, records_data[id_a])
