@@ -3,7 +3,6 @@
 from __future__ import annotations
 
 from dataclasses import dataclass
-from typing import TYPE_CHECKING
 
 import numpy as np
 import pandas as pd
@@ -17,8 +16,11 @@ import colrev.ops.built_in.pdf_prep.metadata_validation
 import colrev.record
 import colrev.ui_cli.cli_colors as colors
 
-if TYPE_CHECKING:
-    import colrev.ops.dedupe
+if False:  # pylint: disable=using-constant-test
+    from typing import TYPE_CHECKING
+
+    if TYPE_CHECKING:
+        import colrev.ops.dedupe
 
 # pylint: disable=too-many-arguments
 # pylint: disable=too-few-public-methods
@@ -32,6 +34,8 @@ class CurationDedupe(JsonSchemaMixin):
     retrieved from different sources (identifying duplicates in groups of
     volumes/issues or years)"""
 
+    ci_supported: bool = True
+
     @dataclass
     class CurationDedupeSettings(
         colrev.env.package_manager.DefaultSettings, JsonSchemaMixin
@@ -53,11 +57,9 @@ class CurationDedupe(JsonSchemaMixin):
         dedupe_operation: colrev.ops.dedupe.Dedupe,  # pylint: disable=unused-argument
         settings: dict,
     ):
-
         self.settings = self.settings_class.load_settings(data=settings)
 
     def __get_similarity(self, *, df_a: pd.Series, df_b: pd.Series) -> float:
-
         author_similarity = fuzz.ratio(df_a["author"], df_b["author"]) / 100
 
         title_similarity = (
@@ -87,7 +89,6 @@ class CurationDedupe(JsonSchemaMixin):
         references: pd.DataFrame,
         min_similarity: float,
     ) -> tuple:
-
         # Fill out the similarity matrix first
         for base_entry_i in range(1, references.shape[0]):
             for comparison_entry_i in range(1, references.shape[0]):
@@ -103,7 +104,6 @@ class CurationDedupe(JsonSchemaMixin):
         tuples_to_process = []
         maximum_similarity = 1
         while True:
-
             maximum_similarity = np.amax(similarity_array)
             if maximum_similarity < min_similarity:
                 break
@@ -153,11 +153,12 @@ class CurationDedupe(JsonSchemaMixin):
             available_sources = [
                 str(s.filename)
                 for s in dedupe_operation.review_manager.settings.sources
+                if "md_" not in str(s.filename)
             ]
             dedupe_sources = [
                 s["selected_source"]
                 for s in dedupe_operation.review_manager.settings.dedupe.dedupe_package_endpoints
-                if "colrev_built_in.curation_full_outlet_dedupe" == s["endpoint"]
+                if "colrev.curation_full_outlet_dedupe" == s["endpoint"]
             ]
             sources_missing_in_dedupe = [
                 x for x in available_sources if x not in dedupe_sources
@@ -165,7 +166,7 @@ class CurationDedupe(JsonSchemaMixin):
             if len(sources_missing_in_dedupe) > 0:
                 dedupe_operation.review_manager.logger.warning(
                     f"{colors.ORANGE}Sources missing in "
-                    "dedupe.scripts.colrev_built_in.curation_full_outlet_dedupe: "
+                    "dedupe.scripts.colrev.curation_full_outlet_dedupe: "
                     f"{','.join(sources_missing_in_dedupe)}{colors.END}"
                 )
                 if "y" == input("Add sources [y,n]?"):
@@ -175,7 +176,7 @@ class CurationDedupe(JsonSchemaMixin):
                         )
                         penultimate_position = len(dedupe_package_endpoints) - 1
                         dedupe_script_to_add = {
-                            "endpoint": "colrev_built_in.curation_full_outlet_dedupe",
+                            "endpoint": "colrev.curation_full_outlet_dedupe",
                             "selected_source": source_missing_in_dedupe,
                         }
                         dedupe_package_endpoints.insert(
@@ -208,7 +209,6 @@ class CurationDedupe(JsonSchemaMixin):
         toc_items = self.__get_toc_items(records_list=source_records)
 
         for toc_item in toc_items:
-
             # Note : these would be potential errors (duplicates)
             # because they have the same selected_source
             processed_same_toc_same_source_records = [
@@ -243,11 +243,13 @@ class CurationDedupe(JsonSchemaMixin):
                             f"{source_record_dict.get('author', 'NO_AUTHOR')} : "
                             f"{source_record_dict.get('title', 'NO_TITLE')}"
                         )
-
-                if "y" == input(
-                    "No existing records (md_processed*) found. "
-                    "All records unique? Set to md_processed [y]? "
-                ):
+                recs_unique = dedupe_operation.review_manager.force_mode
+                if not recs_unique:
+                    recs_unique = "y" == input(
+                        "No existing records (md_processed*) found. "
+                        "All records unique? Set to md_processed [y]? "
+                    )
+                if recs_unique:
                     for source_record_dict in source_records:
                         if all(
                             source_record_dict.get(k, "NA") == v
@@ -330,7 +332,6 @@ class CurationDedupe(JsonSchemaMixin):
 
         # match based on overlapping  colrev_ids
         for toc_item in tqdm(toc_items):
-
             processed_same_toc_records = [
                 r
                 for r in records.values()
@@ -377,7 +378,6 @@ class CurationDedupe(JsonSchemaMixin):
     def __dedupe_pdf_source(
         self, *, dedupe_operation: colrev.ops.dedupe.Dedupe, records: dict
     ) -> list[dict]:
-
         # pylint: disable=too-many-locals
         # pylint: disable=too-many-branches
         # pylint: disable=too-many-nested-blocks
@@ -408,7 +408,6 @@ class CurationDedupe(JsonSchemaMixin):
         )
 
         for toc_item in tqdm(toc_items):
-
             processed_same_toc_records = [
                 r
                 for r in records.values()
@@ -491,7 +490,6 @@ class CurationDedupe(JsonSchemaMixin):
                     data=rec1
                 ).has_overlapping_colrev_id(record=colrev.record.Record(data=rec2))
                 if validation_info["validates"] or overlapping_colrev_ids:
-
                     # Note : make sure that we merge into the CURATED record
                     if "file" in rec1:
                         if tuple_to_process[0] not in [x["ID1"] for x in decision_list]:
@@ -541,7 +539,7 @@ class CurationDedupe(JsonSchemaMixin):
             if str(s.filename) == self.settings.selected_source
         ]
         if len(relevant_source) > 0:
-            pdf_source = "colrev_built_in.pdfs_dir" == relevant_source[0].endpoint
+            pdf_source = "colrev.pdfs_dir" == relevant_source[0].endpoint
         return pdf_source
 
     def __first_source_selected(
@@ -608,7 +606,7 @@ class CurationDedupe(JsonSchemaMixin):
         preferred_masterdata_sources = [
             s
             for s in dedupe_operation.review_manager.settings.sources
-            if s.endpoint != "colrev_built_in.pdfs_dir"
+            if s.endpoint != "colrev.pdfs_dir"
         ]
         dedupe_operation.apply_merges(
             results=decision_list,
