@@ -22,6 +22,7 @@ class CriticalReview(JsonSchemaMixin):
     """Critical review"""
 
     settings_class = colrev.env.package_manager.DefaultSettings
+    ci_supported: bool = True
 
     def __init__(
         self, *, operation: colrev.operation.CheckOperation, settings: dict
@@ -37,12 +38,11 @@ class CriticalReview(JsonSchemaMixin):
         """Initialize a critical review"""
 
         settings.data.data_package_endpoints = [
-            {"endpoint": "colrev_built_in.prisma", "version": "1.0"},
+            {"endpoint": "colrev.prisma", "version": "1.0"},
             {
-                "endpoint": "colrev_built_in.manuscript",
+                "endpoint": "colrev.paper_md",
                 "version": "1.0",
                 "word_template": "APA-7.docx",
-                "csl_style": "apa.csl",
             },
         ]
 
