@@ -22,6 +22,7 @@ class LiteratureReview(JsonSchemaMixin):
     """Literature review (simple)"""
 
     settings_class = colrev.env.package_manager.DefaultSettings
+    ci_supported: bool = True
 
     def __init__(
         self, *, operation: colrev.operation.CheckOperation, settings: dict
@@ -38,10 +39,9 @@ class LiteratureReview(JsonSchemaMixin):
 
         settings.data.data_package_endpoints = [
             {
-                "endpoint": "colrev_built_in.manuscript",
+                "endpoint": "colrev.paper_md",
                 "version": "1.0",
                 "word_template": "APA-7.docx",
-                "csl_style": "apa.csl",
             }
         ]
         return settings
