@@ -12,11 +12,12 @@ from importlib.metadata import version
 from multiprocessing import Lock
 from pathlib import Path
 from sqlite3 import OperationalError
-from typing import TYPE_CHECKING
+from typing import Optional
 
 import requests
 import zope.interface
 from crossref.restful import Etiquette
+from crossref.restful import Journals
 from dacite import from_dict
 from dataclasses_jsonschema import JsonSchemaMixin
 from thefuzz import fuzz
@@ -28,9 +29,12 @@ import colrev.ops.built_in.search_sources.utils as connector_utils
 import colrev.record
 import colrev.ui_cli.cli_colors as colors
 
-if TYPE_CHECKING:
-    import colrev.ops.search
-    import colrev.ops.prep
+if False:  # pylint: disable=using-constant-test
+    from typing import TYPE_CHECKING
+
+    if TYPE_CHECKING:
+        import colrev.ops.search
+        import colrev.ops.prep
 
 # pylint: disable=unused-argument
 # pylint: disable=duplicate-code
@@ -52,15 +56,19 @@ class CrossrefSearchSource(JsonSchemaMixin):
     source_identifier = "doi"
     # "https://api.crossref.org/works/{{doi}}"
     search_type = colrev.settings.SearchType.DB
+    api_search_supported = True
+    ci_supported: bool = True
     heuristic_status = colrev.env.package_manager.SearchSourceHeuristicStatus.oni
     link = "https://www.crossref.org/"
     short_name = "Crossref"
     __crossref_md_filename = Path("data/search/md_crossref.bib")
 
     def __init__(
-        self, *, source_operation: colrev.operation.Operation, settings: dict = None
+        self,
+        *,
+        source_operation: colrev.operation.Operation,
+        settings: Optional[dict] = None,
     ) -> None:
-
         if settings:
             # Crossref as a search_source
             self.search_source = from_dict(
@@ -77,23 +85,25 @@ class CrossrefSearchSource(JsonSchemaMixin):
                 self.search_source = crossref_md_source_l[0]
             else:
                 self.search_source = colrev.settings.SearchSource(
-                    endpoint="colrev_built_in.crossref",
+                    endpoint="colrev.crossref",
                     filename=self.__crossref_md_filename,
                     search_type=colrev.settings.SearchType.OTHER,
                     search_parameters={},
-                    load_conversion_package_endpoint={
-                        "endpoint": "colrev_built_in.bibtex"
-                    },
+                    load_conversion_package_endpoint={"endpoint": "colrev.bibtex"},
                     comment="",
                 )
 
             self.crossref_lock = Lock()
 
+        self.language_service = colrev.env.language_service.LanguageService()
+
+        _, self.email = source_operation.review_manager.get_committer()
+
         self.etiquette = Etiquette(
             "CoLRev",
             version("colrev"),
-            "https://github.com/geritwagner/colrev",
-            source_operation.review_manager.email,
+            "https://github.com/CoLRev-Ecosystem/colrev",
+            self.email,
         )
         self.review_manager = source_operation.review_manager
 
@@ -148,41 +158,54 @@ class CrossrefSearchSource(JsonSchemaMixin):
         for item in crossref_query_return:
             yield connector_utils.json_to_record(item=item)
 
-    def __query_doi(self, *, doi: str) -> colrev.record.PrepRecord:
+    def query_doi(self, *, doi: str) -> colrev.record.PrepRecord:
         """Get records from Crossref based on a doi query"""
 
         # pylint: disable=import-outside-toplevel
         from crossref.restful import Works
 
         works = Works(etiquette=self.etiquette)
-        crossref_query_return = works.doi(doi)
-        if not crossref_query_return:
+        try:
+            crossref_query_return = works.doi(doi)
+        except (
+            requests.exceptions.JSONDecodeError,
+            requests.exceptions.ConnectTimeout,
+        ) as exc:
+            raise colrev_exceptions.RecordNotFoundInPrepSourceException() from exc
+        if crossref_query_return is None:
             raise colrev_exceptions.RecordNotFoundInPrepSourceException()
         retrieved_record_dict = connector_utils.json_to_record(
             item=crossref_query_return
         )
         retrieved_record = colrev.record.PrepRecord(data=retrieved_record_dict)
+
         return retrieved_record
 
-    def __journal_query(self, *, journal_issn: str) -> typing.Iterator[dict]:
+    def __query_journal(
+        self, *, journal_issn: str, rerun: bool
+    ) -> typing.Iterator[dict]:
         """Get records of a selected journal from Crossref"""
 
         # pylint: disable=import-outside-toplevel
-        from crossref.restful import Journals
 
         assert re.match(self.__issn_regex, journal_issn)
 
         journals = Journals(etiquette=self.etiquette)
-        crossref_query_return = (
-            journals.works(journal_issn).query().sort("deposited").order("desc")
-        )
+        if rerun:
+            # Note : the "deposited" field is not always provided.
+            # only the general query will return all records.
+            crossref_query_return = journals.works(journal_issn).query()
+        else:
+            crossref_query_return = (
+                journals.works(journal_issn).query().sort("deposited").order("desc")
+            )
+
         for item in crossref_query_return:
             yield connector_utils.json_to_record(item=item)
 
     def __create_query_url(
         self, *, record: colrev.record.Record, jour_vol_iss_list: bool
     ) -> str:
-
         if jour_vol_iss_list:
             params = {"rows": "50"}
             container_title = re.sub(r"[\W]+", " ", record.data["journal"])
@@ -196,6 +219,9 @@ class CrossrefSearchSource(JsonSchemaMixin):
             params["query"] = query_field
 
         else:
+            if "title" not in record.data:
+                raise colrev_exceptions.NotEnoughDataToIdentifyException()
+
             params = {"rows": "15"}
             if not isinstance(record.data.get("year", ""), str) or not isinstance(
                 record.data.get("title", ""), str
@@ -251,6 +277,35 @@ class CrossrefSearchSource(JsonSchemaMixin):
         # pp.pprint(retrieved_record_dict)
         return similarity
 
+    def __prep_crossref_record(
+        self,
+        *,
+        record: colrev.record.Record,
+        prep_main_record: bool = True,
+        crossref_source: str = "",
+    ) -> colrev.record.Record:
+        self.language_service.unify_to_iso_639_3_language_codes(record=record)
+        doi_connector.DOIConnector.get_link_from_doi(
+            review_manager=self.review_manager,
+            record=record,
+        )
+
+        if not prep_main_record:
+            # Skip steps for feed records
+            return record
+
+        if "retracted" in record.data.get(
+            "warning", ""
+        ) or "retracted" in record.data.get("prescreen_exclusion", ""):
+            record.prescreen_exclude(reason="retracted")
+            record.remove_field(key="warning")
+        else:
+            assert "" != crossref_source
+            record.set_masterdata_complete(source=crossref_source)
+            record.set_status(target_state=colrev.record.RecordState.md_prepared)
+
+        return record
+
     def crossref_query(
         self,
         *,
@@ -267,15 +322,15 @@ class CrossrefSearchSource(JsonSchemaMixin):
 
         # Note : only returning a multiple-item list for jour_vol_iss_list
 
+        record_list = []
+        most_similar, most_similar_record = 0.0, {}
         try:
-
             record = record_input.copy_prep_rec()
 
             url = self.__create_query_url(
                 record=record, jour_vol_iss_list=jour_vol_iss_list
             )
-            headers = {"user-agent": f"{__name__} (mailto:{review_manager.email})"}
-            record_list = []
+            headers = {"user-agent": f"{__name__} (mailto:{self.email})"}
             session = review_manager.get_cached_session()
 
             # review_manager.logger.debug(url)
@@ -287,7 +342,6 @@ class CrossrefSearchSource(JsonSchemaMixin):
                 # )
                 return []
 
-            most_similar, most_similar_record = 0.0, {}
             data = json.loads(ret.text)
             for item in data["message"]["items"]:
                 if "title" not in item:
@@ -300,26 +354,25 @@ class CrossrefSearchSource(JsonSchemaMixin):
                 )
 
                 retrieved_record = colrev.record.PrepRecord(data=retrieved_record_dict)
-                if "retracted" in retrieved_record.data.get("warning", ""):
-                    retrieved_record.prescreen_exclude(reason="retracted")
-                    retrieved_record.remove_field(key="warning")
 
-                source = (
-                    f'https://api.crossref.org/works/{retrieved_record.data["doi"]}'
-                )
-                retrieved_record.add_provenance_all(source=source)
+                # source = (
+                #     f'https://api.crossref.org/works/{retrieved_record.data["doi"]}'
+                # )
+                # retrieved_record.add_provenance_all(source=source)
 
-                record.set_masterdata_complete(source=source)
+                # record.set_masterdata_complete(source=source)
 
                 if jour_vol_iss_list:
                     record_list.append(retrieved_record)
                 elif most_similar < similarity:
                     most_similar = similarity
                     most_similar_record = retrieved_record.get_data()
-        except json.decoder.JSONDecodeError:
+        except (
+            colrev_exceptions.NotEnoughDataToIdentifyException,
+            json.decoder.JSONDecodeError,
+            requests.exceptions.RequestException,
+        ):
             pass
-        except requests.exceptions.RequestException:
-            return []
         # pylint: disable=duplicate-code
         except OperationalError as exc:
             raise colrev_exceptions.ServiceNotAvailableException(
@@ -328,34 +381,49 @@ class CrossrefSearchSource(JsonSchemaMixin):
             ) from exc
 
         if not jour_vol_iss_list:
-            record_list = [colrev.record.PrepRecord(data=most_similar_record)]
+            if most_similar_record:
+                record_list = [colrev.record.PrepRecord(data=most_similar_record)]
 
         return record_list
 
+    def __check_journal(
+        self,
+        prep_operation: colrev.ops.prep.Prep,
+        record: colrev.record.Record,
+        timeout: int,
+        save_feed: bool,
+    ) -> colrev.record.Record:
+        """When there is no doi, journal names can be checked against crossref"""
+
+        if "article" == record.data["ENTRYTYPE"]:
+            # If type article and doi not in record and
+            # journal name not found in journal-query: notify
+            journals = Journals(etiquette=self.etiquette)
+            # record.data["journal"] = "Information Systems Research"
+            found = False
+            ret = journals.query(record.data["journal"])
+            for rets in ret:
+                if rets["title"]:
+                    found = True
+                    break
+            if not found:
+                record.add_masterdata_provenance_note(
+                    key="journal", note="quality_defect:journal not in crossref"
+                )
+
+        return record
+
     def __get_masterdata_record(
         self,
         prep_operation: colrev.ops.prep.Prep,
         record: colrev.record.Record,
         timeout: int,
-        safe_feed: bool,
+        save_feed: bool,
     ) -> colrev.record.Record:
-
         try:
-
-            retrieved_records = self.crossref_query(
-                review_manager=self.review_manager,
-                record_input=record,
-                jour_vol_iss_list=False,
-                timeout=timeout,
-            )
-            retrieved_record = retrieved_records.pop()
-
-            retries = 0
-            while (
-                not retrieved_record and retries < prep_operation.max_retries_on_error
-            ):
-                retries += 1
-
+            if "doi" in record.data:
+                retrieved_record = self.query_doi(doi=record.data["doi"])
+            else:
                 retrieved_records = self.crossref_query(
                     review_manager=self.review_manager,
                     record_input=record,
@@ -364,7 +432,22 @@ class CrossrefSearchSource(JsonSchemaMixin):
                 )
                 retrieved_record = retrieved_records.pop()
 
-            if 0 == len(retrieved_record.data):
+                retries = 0
+                while (
+                    not retrieved_record
+                    and retries < prep_operation.max_retries_on_error
+                ):
+                    retries += 1
+
+                    retrieved_records = self.crossref_query(
+                        review_manager=self.review_manager,
+                        record_input=record,
+                        jour_vol_iss_list=False,
+                        timeout=timeout,
+                    )
+                    retrieved_record = retrieved_records.pop()
+
+            if 0 == len(retrieved_record.data) or "doi" not in retrieved_record.data:
                 raise colrev_exceptions.RecordNotFoundInPrepSourceException()
 
             similarity = colrev.record.PrepRecord.get_retrieval_similarity(
@@ -400,21 +483,12 @@ class CrossrefSearchSource(JsonSchemaMixin):
                     default_source=retrieved_record.data["colrev_origin"][0],
                 )
 
-                if "retracted" in record.data.get("warning", ""):
-                    record.prescreen_exclude(reason="retracted")
-                    record.remove_field(key="warning")
-                else:
-                    doi_connector.DOIConnector.get_link_from_doi(
-                        review_manager=self.review_manager,
-                        record=record,
-                    )
-                    record.set_masterdata_complete(
-                        source=retrieved_record.data["colrev_origin"][0]
-                    )
-                    record.set_status(
-                        target_state=colrev.record.RecordState.md_prepared
-                    )
-                if safe_feed:
+                record = self.__prep_crossref_record(
+                    record=record,
+                    crossref_source=retrieved_record.data["colrev_origin"][0],
+                )
+
+                if save_feed:
                     crossref_feed.save_feed_file()
                 self.crossref_lock.release()
                 return record
@@ -430,16 +504,17 @@ class CrossrefSearchSource(JsonSchemaMixin):
             OSError,
             IndexError,
             colrev_exceptions.RecordNotFoundInPrepSourceException,
-        ):
-            pass
+        ) as exc:
+            if prep_operation.review_manager.verbose_mode:
+                print(exc)
+
         return record
 
     def __check_doi_masterdata(
         self, record: colrev.record.Record
     ) -> colrev.record.Record:
-
         try:
-            retrieved_record = self.__query_doi(doi=record.data["doi"])
+            retrieved_record = self.query_doi(doi=record.data["doi"])
             similarity = colrev.record.PrepRecord.get_retrieval_similarity(
                 record_original=record,
                 retrieved_record_original=retrieved_record,
@@ -466,10 +541,9 @@ class CrossrefSearchSource(JsonSchemaMixin):
 
     def get_masterdata(
         self,
-        *,
         prep_operation: colrev.ops.prep.Prep,
         record: colrev.record.Record,
-        safe_feed: bool = True,
+        save_feed: bool = True,
         timeout: int = 10,
     ) -> colrev.record.Record:
         """Retrieve masterdata from Crossref based on similarity with the record provided"""
@@ -484,12 +558,19 @@ class CrossrefSearchSource(JsonSchemaMixin):
         if "doi" in record.data:
             record = self.__check_doi_masterdata(record=record)
 
-        else:
-            record = self.__get_masterdata_record(
+        record = self.__get_masterdata_record(
+            prep_operation=prep_operation,
+            record=record,
+            timeout=timeout,
+            save_feed=save_feed,
+        )
+
+        if "doi" not in record.data:
+            record = self.__check_journal(
                 prep_operation=prep_operation,
                 record=record,
                 timeout=timeout,
-                safe_feed=safe_feed,
+                save_feed=save_feed,
             )
 
         return record
@@ -506,7 +587,6 @@ class CrossrefSearchSource(JsonSchemaMixin):
         )
 
         if source.filename.name != self.__crossref_md_filename.name:
-
             if not any(x in source.search_parameters for x in ["query", "scope"]):
                 raise colrev_exceptions.InvalidQueryException(
                     "Crossref search_parameters requires a query or journal_issn field"
@@ -550,13 +630,15 @@ class CrossrefSearchSource(JsonSchemaMixin):
             f"SearchSource {source.filename} validated"
         )
 
-    def __get_crossref_query_return(self) -> typing.Iterator[dict]:
+    def __get_crossref_query_return(self, *, rerun: bool) -> typing.Iterator[dict]:
         params = self.search_source.search_parameters
 
         if "scope" in params:
             if "journal_issn" in params["scope"]:
                 for journal_issn in params["scope"]["journal_issn"].split("|"):
-                    yield from self.__journal_query(journal_issn=journal_issn)
+                    yield from self.__query_journal(
+                        journal_issn=journal_issn, rerun=rerun
+                    )
         else:
             if "query" in params:
                 crossref_query = {"bibliographic": params["query"]}
@@ -573,7 +655,6 @@ class CrossrefSearchSource(JsonSchemaMixin):
         search_operation: colrev.ops.search.Search,
         crossref_feed: colrev.ops.search.GeneralOriginFeed,
     ) -> None:
-
         records = search_operation.review_manager.dataset.load_records_dict()
 
         nr_changed = 0
@@ -581,7 +662,7 @@ class CrossrefSearchSource(JsonSchemaMixin):
             feed_record = colrev.record.Record(data=feed_record_dict)
 
             try:
-                retrieved_record = self.__query_doi(doi=feed_record_dict["doi"])
+                retrieved_record = self.query_doi(doi=feed_record_dict["doi"])
 
                 if retrieved_record.data["doi"] != feed_record.data["doi"]:
                     continue
@@ -633,7 +714,6 @@ class CrossrefSearchSource(JsonSchemaMixin):
         crossref_feed: colrev.ops.search.GeneralOriginFeed,
         rerun: bool,
     ) -> None:
-
         # pylint: disable=too-many-branches
 
         if rerun:
@@ -645,19 +725,17 @@ class CrossrefSearchSource(JsonSchemaMixin):
         nr_retrieved, nr_changed = 0, 0
 
         try:
-
             # for record_dict in tqdm(
             #     self.__get_crossref_query_return(),
             #     total=len(crossref_feed.feed_records),
             # ):
-            for record_dict in self.__get_crossref_query_return():
+            for record_dict in self.__get_crossref_query_return(rerun=rerun):
                 # Note : discard "empty" records
                 if "" == record_dict.get("author", "") and "" == record_dict.get(
                     "title", ""
                 ):
                     continue
                 try:
-
                     crossref_feed.set_id(record_dict=record_dict)
                 except colrev_exceptions.NotFeedIdentifiableException:
                     continue
@@ -668,10 +746,9 @@ class CrossrefSearchSource(JsonSchemaMixin):
                         crossref_feed.feed_records[record_dict["ID"]]
                     )
 
-                prep_record = colrev.record.PrepRecord(data=record_dict)
-                doi_connector.DOIConnector.get_link_from_doi(
-                    record=prep_record,
-                    review_manager=search_operation.review_manager,
+                prep_record = colrev.record.Record(data=record_dict)
+                prep_record = self.__prep_crossref_record(
+                    record=prep_record, prep_main_record=False
                 )
 
                 if "colrev_data_provenance" in prep_record.data:
@@ -730,7 +807,10 @@ class CrossrefSearchSource(JsonSchemaMixin):
             search_operation.review_manager.dataset.save_records_dict(records=records)
             search_operation.review_manager.dataset.add_record_changes()
 
-        except (requests.exceptions.JSONDecodeError) as exc:
+        except (
+            requests.exceptions.JSONDecodeError,
+            requests.exceptions.Timeout,
+        ) as exc:
             # watch github issue:
             # https://github.com/fabiobatalha/crossrefapi/issues/46
             if "504 Gateway Time-out" in str(exc):
@@ -773,6 +853,49 @@ class CrossrefSearchSource(JsonSchemaMixin):
 
         return result
 
+    @classmethod
+    def add_endpoint(
+        cls, search_operation: colrev.ops.search.Search, query: str
+    ) -> typing.Optional[colrev.settings.SearchSource]:
+        """Add SearchSource as an endpoint (based on query provided to colrev search -a )"""
+
+        if "https://search.crossref.org/?q=" in query:
+            query = (
+                query.replace("https://search.crossref.org/?q=", "")
+                .replace("&from_ui=yes", "")
+                .lstrip("+")
+            )
+
+            filename = search_operation.get_unique_filename(
+                file_path_string=f"crossref_{query}"
+            )
+            add_source = colrev.settings.SearchSource(
+                endpoint="colrev.crossref",
+                filename=filename,
+                search_type=colrev.settings.SearchType.DB,
+                search_parameters={"query": query},
+                load_conversion_package_endpoint={"endpoint": "colrev.bibtex"},
+                comment="",
+            )
+            return add_source
+        if "crossref:jissn=" in query.replace("colrev.", ""):
+            query = query.replace("crossref:jissn=", "").replace("colrev.", "")
+
+            filename = search_operation.get_unique_filename(
+                file_path_string=f"crossref_jissn_{query}"
+            )
+            add_source = colrev.settings.SearchSource(
+                endpoint="colrev.crossref",
+                filename=filename,
+                search_type=colrev.settings.SearchType.DB,
+                search_parameters={"scope": {"journal_issn": query}},
+                load_conversion_package_endpoint={"endpoint": "colrev.bibtex"},
+                comment="",
+            )
+            return add_source
+
+        return None
+
     def load_fixes(
         self,
         load_operation: colrev.ops.load.Load,
@@ -788,6 +911,14 @@ class CrossrefSearchSource(JsonSchemaMixin):
     ) -> colrev.record.Record:
         """Source-specific preparation for Crossref"""
 
+        source_item = [
+            x
+            for x in record.data["colrev_origin"]
+            if str(source.filename).replace("data/search/", "") in x
+        ]
+        if source_item:
+            record.set_masterdata_complete(source=source_item[0])
+
         return record
 
 
