@@ -5,13 +5,12 @@ from __future__ import annotations
 import html
 import json
 import re
-import typing
 from dataclasses import dataclass
 from datetime import datetime
 from multiprocessing import Lock
 from pathlib import Path
 from sqlite3 import OperationalError
-from typing import TYPE_CHECKING
+from typing import Optional
 
 import requests
 import zope.interface
@@ -25,8 +24,10 @@ import colrev.record
 import colrev.settings
 import colrev.ui_cli.cli_colors as colors
 
-if TYPE_CHECKING:
-    import colrev.ops.prep
+if False:  # pylint: disable=using-constant-test
+    import typing
+
+    # from typing import TYPE_CHECKING
 
 # pylint: disable=unused-argument
 # pylint: disable=duplicate-code
@@ -44,6 +45,8 @@ class DBLPSearchSource(JsonSchemaMixin):
 
     source_identifier = "dblp_key"
     search_type = colrev.settings.SearchType.DB
+    api_search_supported = True
+    ci_supported: bool = True
     heuristic_status = colrev.env.package_manager.SearchSourceHeuristicStatus.supported
     short_name = "DBLP"
     link = "https://dblp.org/"
@@ -60,7 +63,7 @@ class DBLPSearchSource(JsonSchemaMixin):
         search_type: colrev.settings.SearchType
         search_parameters: dict
         load_conversion_package_endpoint: dict
-        comment: typing.Optional[str]
+        comment: Optional[str]
 
         _details = {
             "search_parameters": {
@@ -75,9 +78,8 @@ class DBLPSearchSource(JsonSchemaMixin):
         self,
         *,
         source_operation: colrev.operation.Operation,
-        settings: dict = None,
+        settings: Optional[dict] = None,
     ) -> None:
-
         if settings:
             # DBLP as a search_source
             self.search_source = from_dict(
@@ -94,18 +96,18 @@ class DBLPSearchSource(JsonSchemaMixin):
                 self.search_source = dblp_md_source_l[0]
             else:
                 self.search_source = colrev.settings.SearchSource(
-                    endpoint="colrev_built_in.dblp",
+                    endpoint="colrev.dblp",
                     filename=self.__dblp_md_filename,
                     search_type=colrev.settings.SearchType.OTHER,
                     search_parameters={},
-                    load_conversion_package_endpoint={
-                        "endpoint": "colrev_built_in.bibtex"
-                    },
+                    load_conversion_package_endpoint={"endpoint": "colrev.bibtex"},
                     comment="",
                 )
         self.dblp_lock = Lock()
         self.origin_prefix = self.search_source.get_origin_prefix()
 
+        _, self.email = source_operation.review_manager.get_committer()
+
     def check_availability(
         self, *, source_operation: colrev.operation.Operation
     ) -> None:
@@ -159,7 +161,7 @@ class DBLPSearchSource(JsonSchemaMixin):
         # https://dblp.org/db/journals/jasis/index.html
         venue = venue_string
         url = self.__api_url_venues + venue_string.replace(" ", "+") + "&format=json"
-        headers = {"user-agent": f"{__name__} (mailto:{review_manager.email})"}
+        headers = {"user-agent": f"{__name__} (mailto:{self.email})"}
         try:
             ret = session.request("GET", url, headers=headers, timeout=timeout)
             ret.raise_for_status()
@@ -273,8 +275,8 @@ class DBLPSearchSource(JsonSchemaMixin):
         self,
         *,
         review_manager: colrev.review_manager.ReviewManager,
-        query: str = None,
-        url: str = None,
+        query: Optional[str] = None,
+        url: Optional[str] = None,
         timeout: int = 10,
     ) -> list:
         """Retrieve records from DBLP based on a query"""
@@ -290,7 +292,7 @@ class DBLPSearchSource(JsonSchemaMixin):
                 query = re.sub(r"[\W]+", " ", query.replace(" ", "_"))
                 url = self.__api_url + query.replace(" ", "+") + "&format=json"
 
-            headers = {"user-agent": f"{__name__}  (mailto:{review_manager.email})"}
+            headers = {"user-agent": f"{__name__}  (mailto:{self.email})"}
             # review_manager.logger.debug(url)
             ret = session.request(
                 "GET", url, headers=headers, timeout=timeout  # type: ignore
@@ -382,7 +384,6 @@ class DBLPSearchSource(JsonSchemaMixin):
         search_operation: colrev.ops.search.Search,
         dblp_feed: colrev.ops.search.GeneralOriginFeed,
     ) -> None:
-
         records = search_operation.review_manager.dataset.load_records_dict()
 
         nr_changed = 0
@@ -441,7 +442,6 @@ class DBLPSearchSource(JsonSchemaMixin):
         dblp_feed: colrev.ops.search.GeneralOriginFeed,
         rerun: bool,
     ) -> None:
-
         # pylint: disable=too-many-branches
         # pylint: disable=too-many-locals
         # pylint: disable=too-many-nested-blocks
@@ -459,7 +459,6 @@ class DBLPSearchSource(JsonSchemaMixin):
                 start = datetime.now().year - 2
 
             for year in range(start, datetime.now().year + 1):
-
                 search_operation.review_manager.logger.debug(f"Retrieve year {year}")
 
                 if "scope" in self.search_source.search_parameters:
@@ -477,7 +476,6 @@ class DBLPSearchSource(JsonSchemaMixin):
                         self.search_source.search_parameters["query"] + "+" + str(year)
                     )
 
-                nr_retrieved = 0
                 batch_size_cumulative = 0
                 batch_size = 250
                 while True:
@@ -533,7 +531,6 @@ class DBLPSearchSource(JsonSchemaMixin):
                             nr_retrieved += 1
 
                         else:
-
                             changed = search_operation.update_existing_record(
                                 records=records,
                                 record_dict=retrieved_record.data,
@@ -620,6 +617,37 @@ class DBLPSearchSource(JsonSchemaMixin):
 
         return result
 
+    @classmethod
+    def add_endpoint(
+        cls, search_operation: colrev.ops.search.Search, query: str
+    ) -> typing.Optional[colrev.settings.SearchSource]:
+        """Add SearchSource as an endpoint (based on query provided to colrev search -a )"""
+
+        if (
+            "https://dblp.org/search?q=" in query
+            or "https://dblp.org/search/publ?q=" in query
+        ):
+            query = query.replace(
+                "https://dblp.org/search?q=", "https://dblp.org/search/publ/api?q="
+            ).replace(
+                "https://dblp.org/search/publ?q=", "https://dblp.org/search/publ/api?q="
+            )
+
+            filename = search_operation.get_unique_filename(
+                file_path_string=f"dblp_{query.replace('https://dblp.org/search/publ/api?q=', '')}"
+            )
+            add_source = colrev.settings.SearchSource(
+                endpoint="colrev.dblp",
+                filename=filename,
+                search_type=colrev.settings.SearchType.DB,
+                search_parameters={"query": query},
+                load_conversion_package_endpoint={"endpoint": "colrev.bibtex"},
+                comment="",
+            )
+            return add_source
+
+        return None
+
     def load_fixes(
         self,
         load_operation: colrev.ops.load.Load,
@@ -647,7 +675,11 @@ class DBLPSearchSource(JsonSchemaMixin):
         return record
 
     def get_masterdata(
-        self, *, prep_operation: colrev.ops.prep.Prep, record: colrev.record.Record
+        self,
+        prep_operation: colrev.ops.prep.Prep,
+        record: colrev.record.Record,
+        save_feed: bool = True,
+        timeout: int = 10,
     ) -> colrev.record.Record:
         """Retrieve masterdata from DBLP based on similarity with the record provided"""
 
@@ -680,6 +712,10 @@ class DBLPSearchSource(JsonSchemaMixin):
                 review_manager=prep_operation.review_manager,
                 query=query,
             ):
+                if "dblp_key" in record.data:
+                    if retrieved_record.data["dblp_key"] != record.data["dblp_key"]:
+                        continue
+
                 similarity = colrev.record.PrepRecord.get_retrieval_similarity(
                     record_original=record,
                     retrieved_record_original=retrieved_record,
