@@ -9,7 +9,7 @@ from dataclasses import dataclass
 from multiprocessing import Lock
 from pathlib import Path
 from sqlite3 import OperationalError
-from typing import TYPE_CHECKING
+from typing import Optional
 from urllib.parse import quote
 from xml.etree.ElementTree import Element
 
@@ -25,8 +25,13 @@ import colrev.ops.search
 import colrev.record
 import colrev.settings
 
-if TYPE_CHECKING:
-    import colrev.ops.prep
+# pylint: disable=duplicate-code
+
+if False:  # pylint: disable=using-constant-test
+    from typing import TYPE_CHECKING  # pylint: disable=ungrouped-imports
+
+    if TYPE_CHECKING:
+        import colrev.ops.prep
 
 
 # pylint: disable=unused-argument
@@ -43,6 +48,8 @@ class EuropePMCSearchSource(JsonSchemaMixin):
     # settings_class = colrev.env.package_manager.DefaultSourceSettings
     source_identifier = "europe_pmc_id"
     search_type = colrev.settings.SearchType.DB
+    api_search_supported = True
+    ci_supported: bool = True
     heuristic_status = colrev.env.package_manager.SearchSourceHeuristicStatus.supported
     short_name = "Europe PMC"
     link = "https://europepmc.org/"
@@ -74,9 +81,8 @@ class EuropePMCSearchSource(JsonSchemaMixin):
         self,
         *,
         source_operation: colrev.operation.Operation,
-        settings: dict = None,
+        settings: Optional[dict] = None,
     ) -> None:
-
         if settings:
             # EuropePMC as a search_source
             self.search_source = from_dict(
@@ -93,13 +99,11 @@ class EuropePMCSearchSource(JsonSchemaMixin):
                 self.search_source = europe_pmc_md_source_l[0]
             else:
                 self.search_source = colrev.settings.SearchSource(
-                    endpoint="colrev_built_in.europe_pmc",
+                    endpoint="colrev.europe_pmc",
                     filename=self.__europe_pmc_md_filename,
                     search_type=colrev.settings.SearchType.OTHER,
                     search_parameters={},
-                    load_conversion_package_endpoint={
-                        "endpoint": "colrev_built_in.bibtex"
-                    },
+                    load_conversion_package_endpoint={"endpoint": "colrev.bibtex"},
                     comment="",
                 )
 
@@ -111,7 +115,6 @@ class EuropePMCSearchSource(JsonSchemaMixin):
 
     @classmethod
     def __europe_pmc_xml_to_record(cls, *, item: Element) -> colrev.record.PrepRecord:
-
         # pylint: disable=too-many-branches
         # pylint: disable=too-many-locals
         # pylint: disable=too-many-statements
@@ -234,15 +237,15 @@ class EuropePMCSearchSource(JsonSchemaMixin):
         # pylint: disable=too-many-locals
 
         try:
-
             record = record_input.copy_prep_rec()
 
             url = (
                 "https://www.ebi.ac.uk/europepmc/webservices/rest/search?query="
                 + quote(record.data["title"])
             )
+            _, email = review_manager.get_committer()
 
-            headers = {"user-agent": f"{__name__} (mailto:{review_manager.email})"}
+            headers = {"user-agent": f"{__name__} (mailto:{email})"}
             record_list = []
             session = review_manager.get_cached_session()
 
@@ -261,7 +264,6 @@ class EuropePMCSearchSource(JsonSchemaMixin):
                 result_list = root.findall("resultList")[0]
 
                 for result_item in result_list.findall("result"):
-
                     retrieved_record = cls.__europe_pmc_xml_to_record(item=result_item)
 
                     if "title" not in retrieved_record.data:
@@ -308,9 +310,9 @@ class EuropePMCSearchSource(JsonSchemaMixin):
 
     def get_masterdata(
         self,
-        *,
         prep_operation: colrev.ops.prep.Prep,
         record: colrev.record.Record,
+        save_feed: bool = True,
         timeout: int = 10,  # pylint: disable=unused-argument
     ) -> colrev.record.Record:
         """Retrieve masterdata from Europe PMC based on similarity with the record provided"""
@@ -320,9 +322,7 @@ class EuropePMCSearchSource(JsonSchemaMixin):
         # https://www.ebi.ac.uk/europepmc/webservices/rest/article/MED/23245604
 
         try:
-
             if len(record.data.get("title", "")) > 35:
-
                 retries = 0
                 while retries < prep_operation.max_retries_on_error:
                     retries += 1
@@ -427,7 +427,6 @@ class EuropePMCSearchSource(JsonSchemaMixin):
         )
 
         try:
-
             for retrieved_record in self.europe_pcmc_query(
                 review_manager=search_operation.review_manager,
                 record_input=colrev.record.Record(
@@ -435,7 +434,6 @@ class EuropePMCSearchSource(JsonSchemaMixin):
                 ),
                 most_similar_only=False,
             ):
-
                 if "colrev_data_provenance" in retrieved_record.data:
                     del retrieved_record.data["colrev_data_provenance"]
                 if "colrev_masterdata_provenance" in retrieved_record.data:
@@ -465,6 +463,14 @@ class EuropePMCSearchSource(JsonSchemaMixin):
 
         return result
 
+    @classmethod
+    def add_endpoint(
+        cls, search_operation: colrev.ops.search.Search, query: str
+    ) -> typing.Optional[colrev.settings.SearchSource]:
+        """Add SearchSource as an endpoint (based on query provided to colrev search -a )"""
+
+        return None
+
     def load_fixes(
         self,
         load_operation: colrev.ops.load.Load,
