@@ -2,11 +2,15 @@
 """SearchSource: LocalIndex"""
 from __future__ import annotations
 
+import difflib
 import typing
 import webbrowser
+from copy import deepcopy
 from dataclasses import dataclass
 from multiprocessing import Lock
 from pathlib import Path
+from typing import Optional
+from urllib.parse import urlparse
 
 import git
 import zope.interface
@@ -34,10 +38,12 @@ class LocalIndexSearchSource(JsonSchemaMixin):
     settings_class = colrev.env.package_manager.DefaultSourceSettings
     source_identifier = "curation_ID"
     search_type = colrev.settings.SearchType.OTHER
+    api_search_supported = True
+    ci_supported: bool = True
     heuristic_status = colrev.env.package_manager.SearchSourceHeuristicStatus.supported
     short_name = "LocalIndex"
     link = (
-        "https://github.com/geritwagner/colrev/blob/main/"
+        "https://github.com/CoLRev-Ecosystem/colrev/blob/main/"
         + "colrev/ops/built_in/search_sources/local_index.py"
     )
     __local_index_md_filename = Path("data/search/md_curated.bib")
@@ -61,9 +67,8 @@ class LocalIndexSearchSource(JsonSchemaMixin):
         self,
         *,
         source_operation: colrev.operation.Operation,
-        settings: dict = None,
+        settings: Optional[dict] = None,
     ) -> None:
-
         if settings:
             # LocalIndex as a search_source
             self.search_source = from_dict(
@@ -81,13 +86,11 @@ class LocalIndexSearchSource(JsonSchemaMixin):
                 self.search_source = li_md_source_l[0]
             else:
                 self.search_source = colrev.settings.SearchSource(
-                    endpoint="colrev_built_in.local_index",
+                    endpoint="colrev.local_index",
                     filename=self.__local_index_md_filename,
                     search_type=colrev.settings.SearchType.OTHER,
                     search_parameters={},
-                    load_conversion_package_endpoint={
-                        "endpoint": "colrev_built_in.bibtex"
-                    },
+                    load_conversion_package_endpoint={"endpoint": "colrev.bibtex"},
                     comment="",
                 )
 
@@ -116,38 +119,30 @@ class LocalIndexSearchSource(JsonSchemaMixin):
         #     )
 
         if "query" in source.search_parameters:
-            if "simple_query_string" in source.search_parameters["query"]:
-                if "query" in source.search_parameters["query"]["simple_query_string"]:
-                    pass
-                else:
-                    raise colrev_exceptions.InvalidQueryException(
-                        "Source missing query/simple_query_string/query "
-                        f"search_parameter ({source.filename})"
-                    )
+            pass
+            # if "simple_query_string" in source.search_parameters["query"]:
+            #     if "query" in source.search_parameters["query"]["simple_query_string"]:
+            #         pass
+            #     else:
+            #         raise colrev_exceptions.InvalidQueryException(
+            #             "Source missing query/simple_query_string/query "
+            #             f"search_parameter ({source.filename})"
+            #         )
 
-            elif "url" in source.search_parameters["query"]:
-                pass
-            else:
-                raise colrev_exceptions.InvalidQueryException(
-                    f"Source missing query/query search_parameter ({source.filename})"
-                )
+            # elif "url" in source.search_parameters["query"]:
+            #     pass
+            # # else:
+            #     raise colrev_exceptions.InvalidQueryException(
+            #         f"Source missing query/query search_parameter ({source.filename})"
+            #     )
 
         search_operation.review_manager.logger.debug(
             f"SearchSource {source.filename} validated"
         )
 
     def __retrieve_from_index(self) -> typing.List[dict]:
-
         params = self.search_source.search_parameters
-        # query = {
-        #     "query": {
-        #         "simple_query_string": {
-        #             "query": "...",
-        #             "fields": ["title", ...],
-        #         },
-        #     }
-        # }
-        query = params
+        query = params["query"]
 
         returned_records = self.local_index.search(query=query)
 
@@ -159,12 +154,6 @@ class LocalIndexSearchSource(JsonSchemaMixin):
             "screening_criteria",
         ]
         for record_dict in records_to_import:
-            identifier_string = (
-                record_dict["colrev_masterdata_provenance"]["CURATED"]["source"]
-                + "#"
-                + record_dict["ID"]
-            )
-            record_dict["curation_ID"] = identifier_string
             record_dict = {
                 key: value
                 for key, value in record_dict.items()
@@ -179,11 +168,11 @@ class LocalIndexSearchSource(JsonSchemaMixin):
         search_operation: colrev.ops.search.Search,
         local_index_feed: colrev.ops.search.GeneralOriginFeed,
     ) -> None:
-
         records = search_operation.review_manager.dataset.load_records_dict()
 
         nr_changed = 0
-        for feed_record_dict in local_index_feed.feed_records.values():
+        for feed_record_dict_id in list(local_index_feed.feed_records.keys()):
+            feed_record_dict = local_index_feed.feed_records[feed_record_dict_id]
             feed_record = colrev.record.Record(data=feed_record_dict)
 
             try:
@@ -215,6 +204,8 @@ class LocalIndexSearchSource(JsonSchemaMixin):
                 source=self.search_source,
                 update_time_variant_fields=True,
             )
+            # Note : changed refers to the data/records.bib.
+            # Records that are not yet imported do not count.
             if changed:
                 nr_changed += 1
 
@@ -241,13 +232,11 @@ class LocalIndexSearchSource(JsonSchemaMixin):
         local_index_feed: colrev.ops.search.GeneralOriginFeed,
         rerun: bool,
     ) -> None:
-
         records = search_operation.review_manager.dataset.load_records_dict()
 
         nr_retrieved, nr_changed = 0, 0
 
         for retrieved_record_dict in self.__retrieve_from_index():
-
             try:
                 local_index_feed.set_id(record_dict=retrieved_record_dict)
             except colrev_exceptions.NotFeedIdentifiableException:
@@ -307,7 +296,6 @@ class LocalIndexSearchSource(JsonSchemaMixin):
         )
 
         if self.search_source.is_md_source() or self.search_source.is_quasi_md_source():
-
             self.__run_md_search_update(
                 search_operation=search_operation,
                 local_index_feed=local_index_feed,
@@ -326,10 +314,36 @@ class LocalIndexSearchSource(JsonSchemaMixin):
 
         result = {"confidence": 0.0}
         if "curation_ID" in data:
-            result["confidence"] = 1
+            result["confidence"] = 1.0
 
         return result
 
+    @classmethod
+    def add_endpoint(
+        cls, search_operation: colrev.ops.search.Search, query: str
+    ) -> typing.Optional[colrev.settings.SearchSource]:
+        """Add SearchSource as an endpoint (based on query provided to colrev search -a )"""
+
+        if query.startswith("local_index:"):
+            query = query.replace("local_index:", "")
+
+            filename = search_operation.get_unique_filename(
+                file_path_string=f"local_index_{query}".replace("%", "").replace(
+                    "'", ""
+                )
+            )
+            add_source = colrev.settings.SearchSource(
+                endpoint="colrev.local_index",
+                filename=filename,
+                search_type=colrev.settings.SearchType.DB,
+                search_parameters={"query": query},
+                load_conversion_package_endpoint={"endpoint": "colrev.bibtex"},
+                comment="",
+            )
+            return add_source
+
+        return None
+
     def load_fixes(
         self,
         load_operation: colrev.ops.load.Load,
@@ -338,7 +352,9 @@ class LocalIndexSearchSource(JsonSchemaMixin):
     ) -> dict:
         """Load fixes for local-index"""
 
-        # Note : should this be covered in the regular load procedure? It is not really a fix...
+        # Note : to avoid modifying the feed-records
+        records = deepcopy(records)
+
         for record in records.values():
             curation_url = record["curation_ID"].split("#")[0]
             record["colrev_masterdata_provenance"] = {
@@ -346,6 +362,7 @@ class LocalIndexSearchSource(JsonSchemaMixin):
             }
             record["colrev_status"] = colrev.record.RecordState.md_prepared
             del curation_url
+
         return records
 
     def prepare(
@@ -357,25 +374,48 @@ class LocalIndexSearchSource(JsonSchemaMixin):
 
     def get_masterdata(
         self,
-        *,
         prep_operation: colrev.ops.prep.Prep,
         record: colrev.record.Record,
+        save_feed: bool = True,
+        timeout: int = 10,
     ) -> colrev.record.Record:
         """Retrieve masterdata from LocalIndex based on similarity with the record provided"""
 
         # pylint: disable=too-many-branches
+        # pylint: disable=too-many-locals
         # pylint: disable=too-many-statements
 
         if any(self.origin_prefix in o for o in record.data["colrev_origin"]):
             # Already linked to a local-index record
             return record
 
+        retrieved_record_dict = {}
+        added_colrev_pdf_id = False
+        # To enable retrieval based on colrev_pdf_id (as part of the global_ids)
+        if "file" in record.data and "colrev_pdf_id" not in record.data:
+            pdf_path = Path(
+                prep_operation.review_manager.path / Path(record.data["file"])
+            )
+            if pdf_path.is_file():
+                try:
+                    record.data.update(
+                        colrev_pdf_id=colrev.record.Record.get_colrev_pdf_id(
+                            review_manager=prep_operation.review_manager,
+                            pdf_path=pdf_path,
+                        )
+                    )
+                    added_colrev_pdf_id = True
+                except colrev_exceptions.PDFHashError:
+                    pass
+
         try:
             retrieved_record_dict = self.local_index.retrieve(
                 record_dict=record.get_data(), include_file=False
             )
-
-        except colrev_exceptions.RecordNotInIndexException:
+        except (
+            colrev_exceptions.RecordNotInIndexException,
+            colrev_exceptions.NotEnoughDataToIdentifyException,
+        ):
             try:
                 # Search within the table-of-content in local_index
                 retrieved_record_dict = self.local_index.retrieve_from_toc(
@@ -413,6 +453,9 @@ class LocalIndexSearchSource(JsonSchemaMixin):
                     return record
             except colrev_exceptions.NotTOCIdentifiableException:
                 return record
+        finally:
+            if added_colrev_pdf_id:
+                del record.data["colrev_pdf_id"]
 
         retrieved_record = colrev.record.PrepRecord(data=retrieved_record_dict)
         if "colrev_status" in retrieved_record.data:
@@ -420,7 +463,9 @@ class LocalIndexSearchSource(JsonSchemaMixin):
 
         # restriction: if we don't restrict to CURATED,
         # we may have to rethink the LocalIndexSearchFeed.set_ids()
-        if "CURATED" not in retrieved_record.data["colrev_masterdata_provenance"]:
+        if "CURATED" not in retrieved_record.data.get(
+            "colrev_masterdata_provenance", ""
+        ):
             return record
 
         default_source = "LOCAL_INDEX"
@@ -449,6 +494,9 @@ class LocalIndexSearchSource(JsonSchemaMixin):
                 merging_record=retrieved_record,
                 default_source=default_source,
             )
+            record.set_status(target_state=colrev.record.RecordState.md_prepared)
+            if "retracted" == retrieved_record.data.get("prescreen_exclusion", "NA"):
+                record.prescreen_exclude(reason="retracted")
 
             git_repo = prep_operation.review_manager.dataset.get_repo()
             cur_project_source_paths = [str(prep_operation.review_manager.path)]
@@ -477,8 +525,6 @@ class LocalIndexSearchSource(JsonSchemaMixin):
         ):
             self.local_index_lock.release()
 
-        record.set_status(target_state=colrev.record.RecordState.md_prepared)
-
         return record
 
     def __get_local_base_repos(self, *, change_itemsets: list) -> dict:
@@ -512,8 +558,23 @@ class LocalIndexSearchSource(JsonSchemaMixin):
 
         local_base_repos = self.__get_local_base_repos(change_itemsets=change_itemsets)
 
+        def print_diff(change: tuple) -> str:
+            diff = difflib.Differ()
+            letters = list(diff.compare(change[1], change[0]))
+            for i, letter in enumerate(letters):
+                if letter.startswith("  "):
+                    letters[i] = letters[i][-1]
+                elif letter.startswith("+ "):
+                    letters[i] = f"{colors.RED}" + letters[i][-1] + f"{colors.END}"
+                elif letter.startswith("- "):
+                    letters[i] = f"{colors.GREEN}" + letters[i][-1] + f"{colors.END}"
+            res = "".join(letters).replace("\n", " ")
+            return res
+
         for local_base_repo in local_base_repos:
             validated_changes = []
+            print()
+            self.review_manager.logger.info(f"Base repository: {local_base_repo}")
             for item in change_itemsets:
                 repo_path = "NA"
                 if "CURATED" in item["original_record"].get(
@@ -528,12 +589,10 @@ class LocalIndexSearchSource(JsonSchemaMixin):
                 if repo_path != local_base_repo:
                     continue
 
-                self.review_manager.logger.info(
-                    f"Base repository: {local_base_repos[repo_path]}"
-                )
-
-                print()
-                self.review_manager.p_printer.pprint(item["original_record"])
+                # self.review_manager.p_printer.pprint(item["original_record"])
+                colrev.record.Record(
+                    data=item["original_record"]
+                ).print_citation_format()
                 for change_item in item["changes"]:
                     if "change" == change_item[0]:
                         edit_type, field, values = change_item
@@ -545,14 +604,22 @@ class LocalIndexSearchSource(JsonSchemaMixin):
                             + " " * max(len(prefix), 30 - len(prefix))
                             + f": {values[0]}"
                         )
-                        print(" " * max(len(prefix), 30) + f"  {values[1]}")
+                        print(
+                            " " * max(len(prefix), 30)
+                            + f"  {colors.ORANGE}{values[1]}{colors.END}"
+                        )
+                        print(
+                            " " * max(len(prefix), 30)
+                            + f"  {print_diff((values[0], values[1]))}"
+                        )
+
                     elif "add" == change_item[0]:
                         edit_type, field, values = change_item
                         prefix = f"{edit_type} {values[0][0]}"
                         print(
                             prefix
                             + " " * max(len(prefix), 30 - len(prefix))
-                            + f": {values[0][1]}"
+                            + f": {colors.GREEN}{values[0][1]}{colors.END}"
                         )
                     else:
                         self.review_manager.p_printer.pprint(change_item)
@@ -591,7 +658,7 @@ class LocalIndexSearchSource(JsonSchemaMixin):
             else:
                 self.review_manager.logger.error(
                     "Repo behind remote. Pull first to avoid conflicts.\n"
-                    f"colrev env --update {check_operation.review_manager.path}"
+                    "colrev env --pull"
                 )
                 return False
 
@@ -600,7 +667,6 @@ class LocalIndexSearchSource(JsonSchemaMixin):
     def __retrieve_by_colrev_id(
         self, *, indexed_record_dict: dict, records: list[dict]
     ) -> dict:
-
         indexed_record = colrev.record.Record(data=indexed_record_dict)
 
         if "colrev_id" in indexed_record.data:
@@ -628,6 +694,23 @@ class LocalIndexSearchSource(JsonSchemaMixin):
     ) -> dict:
         original_record = change_item["original_record"]
 
+        local_index_feed = self.search_source.get_feed(
+            review_manager=self.review_manager,
+            source_identifier=self.source_identifier,
+            update_only=True,
+        )
+
+        try:
+            md_curated_origin_id = [
+                x for x in original_record["colrev_origin"] if "md_curated.bib/" in x
+            ][0].replace("md_curated.bib/", "")
+            curation_origin_record = local_index_feed.feed_records[md_curated_origin_id]
+            curation_id = curation_origin_record["curation_ID"]
+            curation_id = curation_id[curation_id.find("#") + 1 :]
+            return records[curation_id]
+        except KeyError:
+            pass
+
         try:
             record_dict = self.__retrieve_by_colrev_id(
                 indexed_record_dict=original_record,
@@ -635,7 +718,6 @@ class LocalIndexSearchSource(JsonSchemaMixin):
             )
             return record_dict
         except colrev_exceptions.RecordNotInRepoException:
-
             matching_doi_rec_l = [
                 r
                 for r in records.values()
@@ -654,8 +736,10 @@ class LocalIndexSearchSource(JsonSchemaMixin):
                 record_dict = matching_url_rec_l[0]
                 return record_dict
 
-        print(f'Record not found: {original_record["ID"]}')
-        return {}
+        self.review_manager.logger.error(
+            f'{colors.RED}Record not found: {original_record["ID"]}{colors.END}'
+        )
+        raise colrev_exceptions.RecordNotInIndexException()
 
     def __create_correction_branch(
         self, *, git_repo: git.Repo, record_dict: dict
@@ -679,8 +763,7 @@ class LocalIndexSearchSource(JsonSchemaMixin):
         record_dict: dict,
         change_item: dict,
     ) -> None:
-
-        for (edit_type, key, change) in list(change_item["changes"]):
+        for edit_type, key, change in list(change_item["changes"]):
             # Note : by retricting changes to self.essential_md_keys,
             # we also prevent changes in
             # "colrev_status", "colrev_origin", "file"
@@ -697,7 +780,7 @@ class LocalIndexSearchSource(JsonSchemaMixin):
                 if key not in self.essential_md_keys:
                     continue
                 record_dict[key] = value
-            # gh_issue https://github.com/geritwagner/colrev/issues/63
+            # gh_issue https://github.com/CoLRev-Ecosystem/colrev/issues/63
             # deal with remove/merge
 
         check_operation.review_manager.dataset.save_records_dict(records=records)
@@ -714,7 +797,6 @@ class LocalIndexSearchSource(JsonSchemaMixin):
         prev_branch_name: str,
         source_url: str,
     ) -> None:
-
         git_repo.remotes.origin.push(
             refspec=f"{record_branch_name}:{record_branch_name}"
         )
@@ -752,66 +834,81 @@ class LocalIndexSearchSource(JsonSchemaMixin):
         check_operation: colrev.operation.Operation,
         source_url: str,
         change_list: list,
-    ) -> None:
+    ) -> bool:
+        # pylint: disable=too-many-locals
 
         git_repo = check_operation.review_manager.dataset.get_repo()
         records = check_operation.review_manager.dataset.load_records_dict()
 
+        success = False
         pull_request_msgs = []
+        pull_request_links = []
         for change_item in change_list:
-
-            record_dict = self.__retrieve_record_for_correction(
-                records=records,
-                change_item=change_item,
-            )
-            if not record_dict:
-                continue
-
-            record_branch_name = self.__create_correction_branch(
-                git_repo=git_repo, record_dict=record_dict
-            )
-            prev_branch_name = git_repo.active_branch.name
-
-            remote = git_repo.remote()
-            for head in git_repo.heads:
-                if head.name == record_branch_name:
-                    head.checkout()
-
-            rec_for_reset = record_dict.copy()
-
-            self.__apply_record_correction(
-                check_operation=check_operation,
-                records=records,
-                record_dict=record_dict,
-                change_item=change_item,
-            )
-
-            self.__push_corrections_and_reset_branch(
-                git_repo=git_repo,
-                record_branch_name=record_branch_name,
-                prev_branch_name=prev_branch_name,
-                source_url=source_url,
-            )
-
-            self.__reset_record_after_correction(
-                record_dict=record_dict,
-                rec_for_reset=rec_for_reset,
-                change_item=change_item,
-            )
-
-            if "github.com" in remote.url:
-                webbrowser.open(remote.url, new=2)
-                pull_request_msgs.append(
-                    "\nTo create a pull request for your changes go "
-                    f"to \n{colors.ORANGE}{str(remote.url).rstrip('.git')}/"
-                    f"compare/{record_branch_name}{colors.END}"
+            try:
+                record_dict = self.__retrieve_record_for_correction(
+                    records=records,
+                    change_item=change_item,
                 )
+                if not record_dict:
+                    continue
+
+                record_branch_name = self.__create_correction_branch(
+                    git_repo=git_repo, record_dict=record_dict
+                )
+                prev_branch_name = git_repo.active_branch.name
+
+                remote = git_repo.remote()
+                for head in git_repo.heads:
+                    if head.name == record_branch_name:
+                        head.checkout()
+
+                rec_for_reset = record_dict.copy()
+
+                self.__apply_record_correction(
+                    check_operation=check_operation,
+                    records=records,
+                    record_dict=record_dict,
+                    change_item=change_item,
+                )
+
+                self.__push_corrections_and_reset_branch(
+                    git_repo=git_repo,
+                    record_branch_name=record_branch_name,
+                    prev_branch_name=prev_branch_name,
+                    source_url=source_url,
+                )
+
+                self.__reset_record_after_correction(
+                    record_dict=record_dict,
+                    rec_for_reset=rec_for_reset,
+                    change_item=change_item,
+                )
+
+                host = urlparse(remote.url).hostname
+                if host and host.endswith("github.com"):
+                    link = (
+                        str(remote.url).rstrip(".git")
+                        + "/compare/"
+                        + record_branch_name
+                    )
+                    pull_request_links.append(link)
+                    pull_request_msgs.append(
+                        "\nTo create a pull request for your changes go "
+                        f"to \n{colors.ORANGE}{link}{colors.END}"
+                    )
+                success = True
+            except colrev_exceptions.RecordNotInIndexException:
+                pass
 
         for pull_request_msg in pull_request_msgs:
             print(pull_request_msg)
+        for pull_request_link in pull_request_links:
+            webbrowser.open(pull_request_link, new=2)
+
         # https://github.com/geritwagner/information_systems_papers/compare/update?expand=1
-        # gh_issue https://github.com/geritwagner/colrev/issues/63
+        # gh_issue https://github.com/CoLRev-Ecosystem/colrev/issues/63
         # handle cases where update branch already exists
+        return success
 
     def __apply_correction(self, *, source_url: str, change_list: list) -> None:
         """Apply a (list of) corrections"""
@@ -847,16 +944,17 @@ class LocalIndexSearchSource(JsonSchemaMixin):
             "Precondition for correction (pull-request) checked."
         )
 
-        self.__apply_change_item_correction(
+        success = self.__apply_change_item_correction(
             check_operation=check_operation,
             source_url=source_url,
             change_list=change_list,
         )
 
-        print(
-            f"\n{colors.GREEN}Thank you for supporting other researchers "
-            f"by sharing your corrections ❤{colors.END}\n"
-        )
+        if success:
+            print(
+                f"\n{colors.GREEN}Thank you for supporting other researchers "
+                f"by sharing your corrections ❤{colors.END}\n"
+            )
 
 
 if __name__ == "__main__":
