@@ -5,12 +5,9 @@ from __future__ import annotations
 import re
 import typing
 from dataclasses import dataclass
-from importlib.metadata import version
 from pathlib import Path
 
 import zope.interface
-from crossref.restful import Etiquette
-from crossref.restful import Works
 from dacite import from_dict
 from dataclasses_jsonschema import JsonSchemaMixin
 from pdfminer.pdfdocument import PDFDocument
@@ -19,8 +16,8 @@ from pdfminer.pdfparser import PDFParser
 
 import colrev.env.package_manager
 import colrev.exceptions as colrev_exceptions
+import colrev.ops.built_in.search_sources.crossref
 import colrev.ops.built_in.search_sources.pdf_backward_search as bws
-import colrev.ops.built_in.search_sources.utils as connector_utils
 import colrev.ops.search
 import colrev.record
 import colrev.ui_cli.cli_colors as colors
@@ -41,6 +38,8 @@ class PDFSearchSource(JsonSchemaMixin):
     settings_class = colrev.env.package_manager.DefaultSourceSettings
     source_identifier = "file"
     search_type = colrev.settings.SearchType.PDFS
+    api_search_supported = True
+    ci_supported: bool = False
     heuristic_status = colrev.env.package_manager.SearchSourceHeuristicStatus.supported
     short_name = "PDF directory"
     link = "https://github.com/kermitt2/grobid"
@@ -50,14 +49,15 @@ class PDFSearchSource(JsonSchemaMixin):
     def __init__(
         self, *, source_operation: colrev.operation.CheckOperation, settings: dict
     ) -> None:
-
         self.search_source = from_dict(data_class=self.settings_class, data=settings)
         self.source_operation = source_operation
-        self.pdf_preparation_operation = (
-            source_operation.review_manager.get_pdf_prep_operation(
-                notify_state_transition_operation=False
+
+        if not source_operation.review_manager.in_ci_environment():
+            self.pdf_preparation_operation = (
+                source_operation.review_manager.get_pdf_prep_operation(
+                    notify_state_transition_operation=False
+                )
             )
-        )
 
         self.pdfs_path = source_operation.review_manager.path / Path(
             self.search_source.search_parameters["scope"]["path"]
@@ -79,12 +79,10 @@ class PDFSearchSource(JsonSchemaMixin):
                 self.r_subdir_pattern = re.compile("([0-9]{1,3})(_|/)([0-9]{1,2})")
             if "volume" == self.subdir_pattern:
                 self.r_subdir_pattern = re.compile("([0-9]{1,4})")
-
-        self.etiquette = Etiquette(
-            "CoLRev",
-            version("colrev"),
-            "https://github.com/geritwagner/colrev",
-            source_operation.review_manager.email,
+        self.crossref_connector = (
+            colrev.ops.built_in.search_sources.crossref.CrossrefSearchSource(
+                source_operation=source_operation
+            )
         )
 
     def __update_if_pdf_renamed(
@@ -136,7 +134,6 @@ class PDFSearchSource(JsonSchemaMixin):
     def __remove_records_if_pdf_no_longer_exists(
         self, *, search_operation: colrev.ops.search.Search
     ) -> None:
-
         # search_operation.review_manager.logger.debug(
         #     "Checking for PDFs that no longer exist"
         # )
@@ -145,7 +142,6 @@ class PDFSearchSource(JsonSchemaMixin):
             return
 
         with open(self.search_source.filename, encoding="utf8") as target_db:
-
             search_rd = search_operation.review_manager.dataset.load_records_dict(
                 load_str=target_db.read()
             )
@@ -214,7 +210,6 @@ class PDFSearchSource(JsonSchemaMixin):
             record_dict["ENTRYTYPE"] = "inproceedings"
 
         if self.subdir_pattern:
-
             # Note : no file access here (just parsing the patterns)
             # no absolute paths needed
             partial_path = Path(record_dict["file"]).parents[0]
@@ -256,7 +251,6 @@ class PDFSearchSource(JsonSchemaMixin):
     def __get_record_from_pdf_grobid(
         self, *, search_operation: colrev.ops.search.Search, record_dict: dict
     ) -> dict:
-
         if colrev.record.RecordState.md_prepared == record_dict.get(
             "colrev_status", "NA"
         ):
@@ -311,7 +305,6 @@ class PDFSearchSource(JsonSchemaMixin):
     def __get_grobid_metadata(
         self, *, search_operation: colrev.ops.search.Search, pdf_path: Path
     ) -> dict:
-
         record_dict: typing.Dict[str, typing.Any] = {
             "file": str(pdf_path),
             "ENTRYTYPE": "misc",
@@ -371,7 +364,6 @@ class PDFSearchSource(JsonSchemaMixin):
         self,
         pdf_path: Path,
     ) -> bool:
-
         if ";" in str(pdf_path):
             self.review_manager.logger.error(
                 f'skipping PDF with ";" in filepath: \n{pdf_path}'
@@ -384,7 +376,6 @@ class PDFSearchSource(JsonSchemaMixin):
             or "_wo_lp.pdf" == str(pdf_path)[-10:]
             or "_backup.pdf" == str(pdf_path)[-11:]
         ):
-
             self.review_manager.logger.info(
                 f"Skipping PDF with _ocr.pdf/_wo_cp.pdf: {pdf_path}"
             )
@@ -432,7 +423,6 @@ class PDFSearchSource(JsonSchemaMixin):
         )
 
     def __add_md_string(self, *, record_dict: dict) -> dict:
-
         md_copy = record_dict.copy()
         try:
             fsize = str(
@@ -447,6 +437,16 @@ class PDFSearchSource(JsonSchemaMixin):
         record_dict["md_string"] = str(fsize) + md_string
         return record_dict
 
+    def get_masterdata(
+        self,
+        prep_operation: colrev.ops.prep.Prep,
+        record: colrev.record.Record,
+        save_feed: bool = True,
+        timeout: int = 10,
+    ) -> colrev.record.Record:
+        """Not implemented"""
+        return record
+
     def run_search(
         self, search_operation: colrev.ops.search.Search, rerun: bool
     ) -> None:
@@ -456,6 +456,10 @@ class PDFSearchSource(JsonSchemaMixin):
         # pylint: disable=too-many-branches
         # pylint: disable=too-many-statements
 
+        # Do not run in continuous-integration environment
+        if search_operation.review_manager.in_ci_environment():
+            return
+
         # Removing records/origins for which PDFs were removed makes sense for curated repositories
         # In regular repositories, it may be confusing (e.g., if PDFs are renamed)
         # In these cases, we may simply print a warning instead of modifying/removing records?
@@ -493,12 +497,10 @@ class PDFSearchSource(JsonSchemaMixin):
         ]
         nr_added, nr_changed = 0, 0
         for pdf_batch in pdf_batches:
-
             for record in pdfs_dir_feed.feed_records.values():
                 record = self.__add_md_string(record_dict=record)
 
             for pdf_path in pdf_batch:
-
                 if self.__is_broken_filepath(pdf_path=pdf_path):
                     continue
 
@@ -524,22 +526,29 @@ class PDFSearchSource(JsonSchemaMixin):
                     f" extract metadata from {pdf_path}"
                 )
                 try:
-                    # retrieve_based_on_colrev_pdf_id
-                    pdf_hash_service = (
-                        search_operation.review_manager.get_pdf_hash_service()
-                    )
-                    colrev_pdf_id = pdf_hash_service.get_pdf_hash(
-                        pdf_path=Path(pdf_path),
-                        page_nr=1,
-                        hash_size=32,
-                    )
-                    new_record = local_index.retrieve_based_on_colrev_pdf_id(
-                        colrev_pdf_id="cpid1:" + colrev_pdf_id
-                    )
-                    new_record["file"] = str(pdf_path)
-                    # Note : an alternative to replacing all data with the curated version
-                    # is to just add the curation_ID
-                    # (and retrieve the curated metadata separately/non-redundantly)
+                    if (
+                        not search_operation.review_manager.settings.is_curated_masterdata_repo()
+                    ):
+                        # retrieve_based_on_colrev_pdf_id
+                        pdf_hash_service = (
+                            search_operation.review_manager.get_pdf_hash_service()
+                        )
+                        colrev_pdf_id = pdf_hash_service.get_pdf_hash(
+                            pdf_path=Path(pdf_path),
+                            page_nr=1,
+                            hash_size=32,
+                        )
+                        new_record = local_index.retrieve_based_on_colrev_pdf_id(
+                            colrev_pdf_id="cpid1:" + colrev_pdf_id
+                        )
+                        new_record["file"] = str(pdf_path)
+                        # Note : an alternative to replacing all data with the curated version
+                        # is to just add the curation_ID
+                        # (and retrieve the curated metadata separately/non-redundantly)
+                    else:
+                        new_record = self.__get_grobid_metadata(
+                            search_operation=search_operation, pdf_path=pdf_path
+                        )
                 except (
                     colrev_exceptions.PDFHashError,
                     colrev_exceptions.RecordNotInIndexException,
@@ -590,7 +599,7 @@ class PDFSearchSource(JsonSchemaMixin):
                         )
                         res = re.findall(self.__doi_regex, record.data["text_from_pdf"])
                         if res:
-                            record["doi"] = res[0]
+                            record["doi"] = res[0].upper()
                         del record.data["text_from_pdf"]
 
                 elif search_operation.review_manager.force_mode:
@@ -644,6 +653,27 @@ class PDFSearchSource(JsonSchemaMixin):
 
         return result
 
+    @classmethod
+    def add_endpoint(
+        cls, search_operation: colrev.ops.search.Search, query: str
+    ) -> typing.Optional[colrev.settings.SearchSource]:
+        """Add SearchSource as an endpoint (based on query provided to colrev search -a )"""
+
+        if "pdfs" == query:
+            filename = search_operation.get_unique_filename(file_path_string="pdfs")
+            # pylint: disable=no-value-for-parameter
+            add_source = colrev.settings.SearchSource(
+                endpoint="colrev.pdfs_dir",
+                filename=filename,
+                search_type=colrev.settings.SearchType.PDFS,
+                search_parameters={"scope": {"path": "data/pdfs"}},
+                load_conversion_package_endpoint={"endpoint": "colrev.bibtex"},
+                comment="",
+            )
+            return add_source
+
+        return None
+
     def load_fixes(
         self,
         load_operation: colrev.ops.load.Load,
@@ -656,24 +686,24 @@ class PDFSearchSource(JsonSchemaMixin):
             if "grobid-version" in record:
                 del record["grobid-version"]
 
-            # TODO : better merge with md_crossref feed record
-            # (temporarily lowering the similarty threshold?)
             if "doi" in record:
-                works = Works(etiquette=self.etiquette)
-                crossref_query_return = works.doi(record["doi"])
-                retrieved_record_dict = connector_utils.json_to_record(
-                    item=crossref_query_return
-                )
-                for key in [
-                    "journal",
-                    "booktitle",
-                    "volume",
-                    "number",
-                    "year",
-                    "pages",
-                ]:
-                    if key in retrieved_record_dict:
-                        record[key] = retrieved_record_dict[key]
+                try:
+                    retrieved_record = self.crossref_connector.query_doi(
+                        doi=record["doi"]
+                    )
+
+                    for key in [
+                        "journal",
+                        "booktitle",
+                        "volume",
+                        "number",
+                        "year",
+                        "pages",
+                    ]:
+                        if key in retrieved_record.data:
+                            record[key] = retrieved_record.data[key]
+                except colrev_exceptions.RecordNotFoundInPrepSourceException:
+                    pass
 
         return records
 
