@@ -7,7 +7,7 @@ import string
 import typing
 from collections import Counter
 from pathlib import Path
-from typing import TYPE_CHECKING
+from typing import Optional
 
 import pandas as pd
 
@@ -17,8 +17,11 @@ import colrev.record
 import colrev.settings
 import colrev.ui_cli.cli_colors as colors
 
-if TYPE_CHECKING:
-    import colrev.review_manager
+if False:  # pylint: disable=using-constant-test
+    from typing import TYPE_CHECKING
+
+    if TYPE_CHECKING:
+        import colrev.review_manager
 
 
 class Dedupe(colrev.operation.Operation):
@@ -36,7 +39,6 @@ class Dedupe(colrev.operation.Operation):
         review_manager: colrev.review_manager.ReviewManager,
         notify_state_transition_operation: bool = True,
     ) -> None:
-
         super().__init__(
             review_manager=review_manager,
             operations_type=colrev.operation.OperationsType.dedupe,
@@ -44,17 +46,20 @@ class Dedupe(colrev.operation.Operation):
         )
 
         self.non_dupe_file_xlsx = (
-            self.review_manager.path / self.NON_DUPLICATE_FILE_XLSX
+            self.review_manager.dedupe_dir / self.NON_DUPLICATE_FILE_XLSX
         )
-        self.non_dupe_file_txt = self.review_manager.path / self.NON_DUPLICATE_FILE_TXT
-        self.dupe_file = self.review_manager.path / self.DUPLICATES_TO_VALIDATE
+        self.non_dupe_file_txt = (
+            self.review_manager.dedupe_dir / self.NON_DUPLICATE_FILE_TXT
+        )
+        self.dupe_file = self.review_manager.dedupe_dir / self.DUPLICATES_TO_VALIDATE
 
         self.same_source_merge_file = (
-            self.review_manager.path / self.SAME_SOURCE_MERGE_FILE
+            self.review_manager.dedupe_dir / self.SAME_SOURCE_MERGE_FILE
         )
         self.prevented_same_source_merge_file = (
-            self.review_manager.path / self.PREVENTED_SAME_SOURCE_MERGE_FILE
+            self.review_manager.dedupe_dir / self.PREVENTED_SAME_SOURCE_MERGE_FILE
         )
+        self.review_manager.dedupe_dir.mkdir(exist_ok=True, parents=True)
 
     def __pre_process(self, *, key: str, value: str) -> str | None:
         if key in ["ID", "ENTRYTYPE", "colrev_status", "colrev_origin"]:
@@ -83,7 +88,8 @@ class Dedupe(colrev.operation.Operation):
     def prep_records(self, *, records_df: pd.DataFrame) -> dict:
         """Prepare records for dedupe"""
 
-        assert records_df.shape[0] > 0
+        if 0 == records_df.shape[0]:
+            return {}
 
         required_fields = [
             "journal",
@@ -237,7 +243,6 @@ class Dedupe(colrev.operation.Operation):
         return records
 
     def __select_primary_merge_record(self, rec_1: dict, rec_2: dict) -> list:
-
         # pylint: disable=too-many-branches
 
         # Heuristic
@@ -302,7 +307,6 @@ class Dedupe(colrev.operation.Operation):
     def __same_source_merge(
         self, *, main_record: colrev.record.Record, dupe_record: colrev.record.Record
     ) -> bool:
-
         main_rec_sources = [x.split("/")[0] for x in main_record.data["colrev_origin"]]
         dupe_rec_sources = [x.split("/")[0] for x in dupe_record.data["colrev_origin"]]
         same_sources = set(main_rec_sources).intersection(set(dupe_rec_sources))
@@ -343,7 +347,6 @@ class Dedupe(colrev.operation.Operation):
             colrev.settings.SameSourceMergePolicy.warn
             == self.review_manager.settings.dedupe.same_source_merges
         ):
-
             self.review_manager.logger.warning(
                 f"\n{colors.ORANGE}"
                 "Applying same source merge "
@@ -360,7 +363,6 @@ class Dedupe(colrev.operation.Operation):
             colrev.settings.SameSourceMergePolicy.prevent
             == self.review_manager.settings.dedupe.same_source_merges
         ):
-
             self.review_manager.logger.warning(
                 f"Prevented same-source merge: ({merge_info})"
             )
@@ -400,22 +402,31 @@ class Dedupe(colrev.operation.Operation):
         duplicate_id_mappings: dict,
         removed_duplicates: list,
         complete_dedupe: bool,
+        set_to_md_processed: list,
     ) -> None:
         for record_id, duplicate_ids in duplicate_id_mappings.items():
-            if complete_dedupe:
-                self.review_manager.logger.info(
-                    f" {colors.GREEN}{record_id} ({','.join(duplicate_ids)})".ljust(46)
-                    + f"md_prepared →  md_processed{colors.END}"
-                )
-            else:
-                self.review_manager.logger.info(
-                    f" {colors.GREEN}{record_id} ({','.join(duplicate_ids)})".ljust(46)
-                    + f"md_prepared →  md_prepared{colors.END}"
-                )
+            if (
+                colrev.record.RecordState.md_prepared
+                == records[record_id]["colrev_status"]
+            ):
+                if complete_dedupe:
+                    self.review_manager.logger.info(
+                        f" {colors.GREEN}{record_id} ({','.join(duplicate_ids)})".ljust(
+                            46
+                        )
+                        + f"md_prepared →  md_processed{colors.END}"
+                    )
+                else:
+                    self.review_manager.logger.info(
+                        f" {colors.GREEN}{record_id} ({','.join(duplicate_ids)})".ljust(
+                            46
+                        )
+                        + f"md_prepared →  md_prepared{colors.END}"
+                    )
         if complete_dedupe:
-            for record in records.values():
+            for rid in set_to_md_processed:
                 self.review_manager.logger.info(
-                    f" {colors.GREEN}{record['ID']}".ljust(46)
+                    f" {colors.GREEN}{rid}".ljust(46)
                     + f"md_prepared →  md_processed{colors.END}"
                 )
 
@@ -428,7 +439,7 @@ class Dedupe(colrev.operation.Operation):
         *,
         results: list,
         complete_dedupe: bool = False,
-        preferred_masterdata_sources: list = None,
+        preferred_masterdata_sources: Optional[list] = None,
     ) -> None:
         """Apply automated deduplication decisions
 
@@ -469,10 +480,8 @@ class Dedupe(colrev.operation.Operation):
         removed_duplicates = []
         duplicate_id_mappings = {}
         record_to_merge = self.__get_records_to_merge(records=records, results=results)
-        for (main_record, dupe_record, dupe) in record_to_merge:
-
+        for main_record, dupe_record, dupe in record_to_merge:
             try:
-
                 if self.__is_cross_level_merge(
                     main_record=main_record, dupe_record=dupe_record
                 ):
@@ -551,18 +560,21 @@ class Dedupe(colrev.operation.Operation):
             if removed_duplicate in records:
                 del records[removed_duplicate]
 
+        set_to_md_processed = []
         if complete_dedupe:
             # Set remaining records to md_processed (not duplicate) because all records
             # have been considered by dedupe
             for record in records.values():
                 if record["colrev_status"] == colrev.record.RecordState.md_prepared:
                     record["colrev_status"] = colrev.record.RecordState.md_processed
+                    set_to_md_processed.append(record["ID"])
 
         self.__print_merge_stats(
             records=records,
             duplicate_id_mappings=duplicate_id_mappings,
             removed_duplicates=removed_duplicates,
             complete_dedupe=complete_dedupe,
+            set_to_md_processed=set_to_md_processed,
         )
 
         self.review_manager.dataset.save_records_dict(records=records)
@@ -576,7 +588,6 @@ class Dedupe(colrev.operation.Operation):
 
         duplicates_to_process = [x for x in results if "duplicate" == x["decision"]]
         for dupe in duplicates_to_process:
-
             try:
                 rec_1 = records[dupe.pop("ID1")]
                 rec_2 = records[dupe.pop("ID2")]
@@ -702,7 +713,10 @@ class Dedupe(colrev.operation.Operation):
         return records
 
     def unmerge_records(
-        self, *, current_record_ids: list = None, previous_id_lists: list = None
+        self,
+        *,
+        current_record_ids: Optional[list] = None,
+        previous_id_lists: Optional[list] = None,
     ) -> None:
         """Unmerge duplicate decision of the records, as identified by their ids.
 
@@ -752,7 +766,6 @@ class Dedupe(colrev.operation.Operation):
                 self.unmerge_records(previous_id_lists=previous_ids_to_unmerge)
 
         if self.non_dupe_file_xlsx.is_file() or self.non_dupe_file_txt.is_file():
-
             ids_to_merge = []
             if self.non_dupe_file_xlsx.is_file():
                 non_dupes = pd.read_excel(self.non_dupe_file_xlsx)
@@ -819,7 +832,6 @@ class Dedupe(colrev.operation.Operation):
         same_source_merges = []
 
         for record in records.values():
-
             rec_sources = [x.split("/")[0] for x in record["colrev_origin"]]
 
             duplicated_sources = [
@@ -855,6 +867,9 @@ class Dedupe(colrev.operation.Operation):
         self.review_manager.logger.info(
             "Identifies duplicate records and merges them (keeping traces to their origins)."
         )
+        self.review_manager.logger.info(
+            "See https://colrev.readthedocs.io/en/latest/manual/metadata_retrieval/dedupe.html"
+        )
 
         if not self.review_manager.high_level_operation:
             print()
@@ -863,14 +878,26 @@ class Dedupe(colrev.operation.Operation):
         for (
             dedupe_package_endpoint
         ) in self.review_manager.settings.dedupe.dedupe_package_endpoints:
-
             # Note : load package/script at this point because the same script
             # may run with different parameters
             endpoint_dict = package_manager.load_packages(
                 package_type=colrev.env.package_manager.PackageEndpointType.dedupe,
                 selected_packages=[dedupe_package_endpoint],
                 operation=self,
+                only_ci_supported=self.review_manager.in_ci_environment(),
             )
+            if dedupe_package_endpoint["endpoint"] not in endpoint_dict:
+                self.review_manager.logger.info(
+                    f'Skip {dedupe_package_endpoint["endpoint"]} (not available)'
+                )
+                if self.review_manager.in_ci_environment():
+                    raise colrev_exceptions.ServiceNotAvailableException(
+                        dep="colrev dedupe",
+                        detailed_trace="dedupe not available in ci environment",
+                    )
+                raise colrev_exceptions.ServiceNotAvailableException(
+                    dep="colrev dedupe", detailed_trace="dedupe not available"
+                )
 
             endpoint = endpoint_dict[dedupe_package_endpoint["endpoint"]]
 
@@ -878,6 +905,14 @@ class Dedupe(colrev.operation.Operation):
             if not self.review_manager.high_level_operation:
                 print()
 
+        dedupe_commit_id = self.review_manager.dataset.get_repo().head.commit.hexsha
+        self.review_manager.logger.info("To validate the changes, use")
+
+        self.review_manager.logger.info(
+            f"{colors.ORANGE}colrev validate {dedupe_commit_id}{colors.END}"
+        )
+        print()
+
         self.review_manager.logger.info(
             f"{colors.GREEN}Completed dedupe operation{colors.END}"
         )
@@ -895,6 +930,9 @@ class Dedupe(colrev.operation.Operation):
             self.review_manager.dataset.add_record_changes()
             self.review_manager.create_commit(msg="Skip prescreen/include all")
 
+        if self.review_manager.in_ci_environment():
+            print("\n\n")
+
 
 if __name__ == "__main__":
     pass
