@@ -15,6 +15,8 @@ import colrev.record
 import colrev.settings
 import colrev.ui_cli.cli_colors as colors
 
+# pylint: disable=too-many-lines
+
 
 class Load(colrev.operation.Operation):
     """Load the records"""
@@ -45,7 +47,6 @@ class Load(colrev.operation.Operation):
         notify_state_transition_operation: bool = True,
         hide_load_explanation: bool = False,
     ) -> None:
-
         super().__init__(
             review_manager=review_manager,
             operations_type=colrev.operation.OperationsType.load,
@@ -62,6 +63,9 @@ class Load(colrev.operation.Operation):
             self.review_manager.logger.info(
                 "Original records (search results) are stored in the directory data/search"
             )
+            self.review_manager.logger.info(
+                "See https://colrev.readthedocs.io/en/latest/manual/metadata_retrieval/load.html"
+            )
 
     def __get_new_search_files(self) -> list[Path]:
         """Retrieve new search files (not yet registered in settings)"""
@@ -107,8 +111,9 @@ class Load(colrev.operation.Operation):
     ) -> list[typing.Dict]:
         """Apply heuristics to identify source"""
 
-        def get_load_conversion_package_endpoint(*, filepath: Path) -> dict:
+        # pylint: disable=too-many-statements
 
+        def get_load_conversion_package_endpoint(*, filepath: Path) -> dict:
             filetype = filepath.suffix.replace(".", "")
 
             for (
@@ -138,49 +143,73 @@ class Load(colrev.operation.Operation):
                 continue
             res = endpoint_class.heuristic(filepath, data)  # type: ignore
             self.review_manager.logger.debug(f"- {endpoint}: {res['confidence']}")
+            try:
+                if res["confidence"] > 0:
+                    result_item = {}
 
-            if res["confidence"] > 0:
-                result_item = {}
+                    res["endpoint"] = endpoint
 
-                res["endpoint"] = endpoint
+                    search_type = endpoint_class.search_type
+                    # Note : as the identifier, we use the filename
+                    # (if search results are added by file/not via the API)
 
-                search_type = endpoint_class.search_type
-                # Note : as the identifier, we use the filename
-                # (if search results are added by file/not via the API)
-
-                if "filename" not in res:
                     # Correct the file extension if necessary
-                    if re.search("%0", data) and filepath.suffix not in [".enl"]:
+                    if re.findall(
+                        r"^%0", data, re.MULTILINE
+                    ) and filepath.suffix not in [".enl"]:
                         new_filename = filepath.with_suffix(".enl")
                         self.review_manager.logger.info(
                             f"{colors.GREEN}Rename to {new_filename} "
                             f"(because the format is .enl){colors.END}"
                         )
                         filepath.rename(new_filename)
+                        self.review_manager.dataset.add_changes(
+                            path=filepath, remove=True
+                        )
                         filepath = new_filename
                         res["filename"] = filepath
                         self.review_manager.dataset.add_changes(path=new_filename)
+                        self.review_manager.create_commit(msg=f"Rename {filepath}")
 
-                if "load_conversion_package_endpoint" not in res:
-                    res[
-                        "load_conversion_package_endpoint"
-                    ] = get_load_conversion_package_endpoint(filepath=filepath)
+                    if re.findall(
+                        r"^TI ", data, re.MULTILINE
+                    ) and filepath.suffix not in [".ris"]:
+                        new_filename = filepath.with_suffix(".ris")
+                        self.review_manager.logger.info(
+                            f"{colors.GREEN}Rename to {new_filename} "
+                            f"(because the format is .ris){colors.END}"
+                        )
+                        filepath.rename(new_filename)
+                        self.review_manager.dataset.add_changes(
+                            path=filepath, remove=True
+                        )
+                        filepath = new_filename
+                        res["filename"] = filepath
+                        self.review_manager.dataset.add_changes(path=new_filename)
+                        self.review_manager.create_commit(msg=f"Rename {filepath}")
 
-                source_candidate = colrev.settings.SearchSource(
-                    endpoint=endpoint,
-                    filename=filepath,
-                    search_type=search_type,
-                    search_parameters={},
-                    load_conversion_package_endpoint=res[
-                        "load_conversion_package_endpoint"
-                    ],
-                    comment="",
-                )
+                    if "load_conversion_package_endpoint" not in res:
+                        res[
+                            "load_conversion_package_endpoint"
+                        ] = get_load_conversion_package_endpoint(filepath=filepath)
 
-                result_item["source_candidate"] = source_candidate
-                result_item["confidence"] = res["confidence"]
+                    source_candidate = colrev.settings.SearchSource(
+                        endpoint=endpoint,
+                        filename=filepath,
+                        search_type=search_type,
+                        search_parameters={},
+                        load_conversion_package_endpoint=res[
+                            "load_conversion_package_endpoint"
+                        ],
+                        comment="",
+                    )
 
-                results_list.append(result_item)
+                    result_item["source_candidate"] = source_candidate
+                    result_item["confidence"] = res["confidence"]
+
+                    results_list.append(result_item)
+            except colrev_exceptions.UnsupportedImportFormatError:
+                continue
 
         # Reduce the results_list when there are results with very high confidence
         if [r for r in results_list if r["confidence"] > 0.95]:
@@ -191,7 +220,7 @@ class Load(colrev.operation.Operation):
             or len([r for r in results_list if r["confidence"] > 0.5]) == 0
         ):
             source_candidate = colrev.settings.SearchSource(
-                endpoint="colrev_built_in.unknown_source",
+                endpoint="colrev.unknown_source",
                 filename=Path(filepath),
                 search_type=colrev.settings.SearchType("DB"),
                 search_parameters={},
@@ -263,7 +292,6 @@ class Load(colrev.operation.Operation):
 
         new_sources = []
         for sfp in new_search_files:
-
             sfp_name = sfp
             if sfp_name in [
                 str(source.filename) for source in self.review_manager.settings.sources
@@ -290,18 +318,19 @@ class Load(colrev.operation.Operation):
             if 1 == len(heuristic_result_list):
                 heuristic_source = heuristic_result_list[0]
             else:
-                print("Select search source:")
-                for i, heuristic_source in enumerate(heuristic_result_list):
-                    highlight_color = ""
-                    if heuristic_source["confidence"] >= 0.7:
-                        highlight_color = colors.GREEN
-                    elif heuristic_source["confidence"] >= 0.5:
-                        highlight_color = colors.ORANGE
-                    print(
-                        f"{highlight_color}{i+1} "
-                        f"(confidence: {round(heuristic_source['confidence'], 2)}):"
-                        f" {heuristic_source['source_candidate'].endpoint}{colors.END}"
-                    )
+                if not skip_query:
+                    print(f"{colors.ORANGE}Select search source{colors.END}:")
+                    for i, heuristic_source in enumerate(heuristic_result_list):
+                        highlight_color = ""
+                        if heuristic_source["confidence"] >= 0.7:
+                            highlight_color = colors.GREEN
+                        elif heuristic_source["confidence"] >= 0.5:
+                            highlight_color = colors.ORANGE
+                        print(
+                            f"{highlight_color}{i+1} "
+                            f"(confidence: {round(heuristic_source['confidence'], 2)}):"
+                            f" {heuristic_source['source_candidate'].endpoint}{colors.END}"
+                        )
 
                 while True:
                     if skip_query:
@@ -315,11 +344,7 @@ class Load(colrev.operation.Operation):
                         heuristic_source = heuristic_result_list[int(selection) - 1]
                         break
 
-            if (
-                "colrev_built_in.unknown_source"
-                == heuristic_source["source_candidate"].endpoint
-            ):
-
+            if "colrev.unknown_source" == heuristic_source["source_candidate"].endpoint:
                 cmd = "Enter the search query (or NA)".ljust(25, " ") + ": "
                 query_input = ""
                 if not skip_query:
@@ -376,7 +401,6 @@ class Load(colrev.operation.Operation):
 
     def __resolve_non_unique_ids(self, *, source: colrev.settings.SearchSource) -> None:
         def get_unique_id(*, non_unique_id: str, id_list: list[str]) -> str:
-
             order = 0
             letters = list(string.ascii_lowercase)
             temp_id = non_unique_id
@@ -486,6 +510,85 @@ class Load(colrev.operation.Operation):
             input_str = re.sub(r"<.*?>", "", input_str)
         return input_str
 
+    def import_provenance(
+        self,
+        *,
+        record: colrev.record.Record,
+    ) -> None:
+        """Set the provenance for an imported record"""
+
+        def set_initial_import_provenance(*, record: colrev.record.Record) -> None:
+            # Initialize colrev_masterdata_provenance
+            colrev_masterdata_provenance, colrev_data_provenance = {}, {}
+
+            for key in record.data.keys():
+                if key in colrev.record.Record.identifying_field_keys:
+                    if key not in colrev_masterdata_provenance:
+                        colrev_masterdata_provenance[key] = {
+                            "source": record.data["colrev_origin"][0],
+                            "note": "",
+                        }
+                elif key not in colrev.record.Record.provenance_keys and key not in [
+                    "colrev_source_identifier",
+                    "ID",
+                    "ENTRYTYPE",
+                ]:
+                    colrev_data_provenance[key] = {
+                        "source": record.data["colrev_origin"][0],
+                        "note": "",
+                    }
+
+            record.data["colrev_data_provenance"] = colrev_data_provenance
+            record.data["colrev_masterdata_provenance"] = colrev_masterdata_provenance
+
+        def set_initial_non_curated_import_provenance(
+            *, record: colrev.record.Record
+        ) -> None:
+            masterdata_restrictions = (
+                self.review_manager.dataset.get_applicable_restrictions(
+                    record_dict=record.get_data()
+                )
+            )
+            if masterdata_restrictions:
+                record.update_masterdata_provenance(
+                    masterdata_restrictions=masterdata_restrictions
+                )
+            else:
+                record.apply_fields_keys_requirements()
+
+            if (
+                record.data["ENTRYTYPE"]
+                in colrev.record.Record.record_field_inconsistencies
+            ):
+                inconsistent_fields = colrev.record.Record.record_field_inconsistencies[
+                    record.data["ENTRYTYPE"]
+                ]
+                for inconsistent_field in inconsistent_fields:
+                    if inconsistent_field in record.data:
+                        inconsistency_note = (
+                            f"inconsistent with entrytype ({record.data['ENTRYTYPE']})"
+                        )
+                        record.add_masterdata_provenance_note(
+                            key=inconsistent_field, note=inconsistency_note
+                        )
+
+            incomplete_fields = record.get_incomplete_fields()
+            for incomplete_field in incomplete_fields:
+                record.add_masterdata_provenance_note(
+                    key=incomplete_field, note="incomplete"
+                )
+
+            defect_fields = record.get_quality_defects()
+            if defect_fields:
+                for defect_field in defect_fields:
+                    record.add_masterdata_provenance_note(
+                        key=defect_field, note="quality_defect"
+                    )
+
+        if not record.masterdata_is_curated():
+            set_initial_import_provenance(record=record)
+            set_initial_non_curated_import_provenance(record=record)
+
     def __import_record(self, *, record_dict: dict) -> dict:
         # pylint: disable=too-many-branches
 
@@ -495,7 +598,6 @@ class Load(colrev.operation.Operation):
         )
 
         if colrev.record.RecordState.md_retrieved == record_dict["colrev_status"]:
-
             # Consistently set keys to lower case
             lower_keys = [k.lower() for k in list(record_dict.keys())]
             for key, n_key in zip(list(record_dict.keys()), lower_keys):
@@ -548,6 +650,10 @@ class Load(colrev.operation.Operation):
                     .rstrip(".")
                 )
 
+            if "year" in record_dict:
+                if str(record_dict["year"]).endswith(".0"):
+                    record_dict["year"] = str(record_dict["year"])[:-2]
+
             if "pages" in record_dict:
                 record_dict["pages"] = record_dict["pages"].replace("–", "--")
                 if record_dict["pages"].count("-") == 1:
@@ -592,8 +698,8 @@ class Load(colrev.operation.Operation):
                 doi=record.data["doi"].replace("http://dx.doi.org/", "").upper()
             )
 
-        record.import_provenance(
-            review_manager=self.review_manager,
+        self.import_provenance(
+            record=record,
         )
         record.set_status(target_state=colrev.record.RecordState.md_imported)
 
@@ -602,8 +708,6 @@ class Load(colrev.operation.Operation):
                 f"{colors.GREEN}Found paper retract: "
                 f"{record.data['ID']}{colors.END}"
             )
-            record.prescreen_exclude(reason="retracted", print_warning=True)
-            record.remove_field(key="warning")
 
         return record.get_data()
 
@@ -638,12 +742,21 @@ class Load(colrev.operation.Operation):
                 record.update(colrev_status=colrev.record.RecordState.md_retrieved)
 
             if "doi" in record:
-                record.update(
-                    doi=record["doi"].replace("http://dx.doi.org/", "").upper()
+                formatted_doi = (
+                    record["doi"]
+                    .lower()
+                    .replace("https://", "http://")
+                    .replace("dx.doi.org", "doi.org")
+                    .replace("http://doi.org/", "")
+                    .upper()
                 )
+                record.update(doi=formatted_doi)
                 # https://www.crossref.org/blog/dois-and-matching-regular-expressions/
                 doi_match = re.match(r"^10.\d{4,9}\/", record["doi"])
                 if not doi_match:
+                    self.review_manager.logger.info(
+                        f"remove doi (not matching regex): {record['doi']}"
+                    )
                     del record["doi"]
 
             self.review_manager.logger.debug(
@@ -661,6 +774,11 @@ class Load(colrev.operation.Operation):
                 f"Loaded {source.get_corresponding_bib_file().name} "
                 f"with {len(search_records)} records"
             )
+        else:
+            self.review_manager.logger.error(
+                f"Did not find bib file {source.get_corresponding_bib_file().name} "
+            )
+            return []
 
         if len(search_records) == 0:
             self.review_manager.logger.info(
@@ -690,7 +808,6 @@ class Load(colrev.operation.Operation):
     def __load_source_records(
         self, *, source: colrev.settings.SearchSource, keep_ids: bool
     ) -> None:
-
         search_records = self.__get_search_records(source=source)
         if len(search_records) == 0:
             return
@@ -755,7 +872,6 @@ class Load(colrev.operation.Operation):
         self.review_manager.dataset.add_record_changes()
 
     def __validate_load(self, *, source: colrev.settings.SearchSource) -> None:
-
         imported_origins = self.__get_currently_imported_origin_list()
         imported = len(imported_origins) - source.len_before
 
@@ -809,18 +925,23 @@ class Load(colrev.operation.Operation):
             return records
 
         def drop_empty_fields(*, records: dict) -> dict:
+            for record_id in records:
+                records[record_id] = {
+                    k: v for k, v in records[record_id].items() if v is not None
+                }
+                records[record_id] = {
+                    k: v for k, v in records[record_id].items() if v != "nan"
+                }
+            return records
 
-            records_list = list(records.values())
-            records_list = [
-                {k: v for k, v in record.items() if v is not None}
-                for record in records_list
-            ]
-            records_list = [
-                {k: v for k, v in record.items() if v != "nan"}
-                for record in records_list
-            ]
-
-            return {r["ID"]: r for r in records_list}
+        def drop_fields(*, records: dict) -> dict:
+            for record_id in records:
+                records[record_id] = {
+                    k: v
+                    for k, v in records[record_id].items()
+                    if k not in ["colrev_status", "colrev_masterdata_provenance"]
+                }
+            return records
 
         if len(records) == 0:
             self.review_manager.report_logger.debug("No records loaded")
@@ -830,6 +951,7 @@ class Load(colrev.operation.Operation):
         records = fix_keys(records=records)
         records = set_incremental_ids(records=records)
         records = drop_empty_fields(records=records)
+        records = drop_fields(records=records)
 
         self.review_manager.dataset.save_records_dict_to_file(
             records=records, save_path=corresponding_bib_file
@@ -844,8 +966,6 @@ class Load(colrev.operation.Operation):
     ) -> None:
         """Load records (main entrypoint)"""
 
-        # TODO : provide -s option
-
         def load_active_sources() -> list:
             checker = self.review_manager.get_checker()
             checker.check_sources()
@@ -862,7 +982,6 @@ class Load(colrev.operation.Operation):
         git_repo = self.review_manager.dataset.get_repo()
         part_exact_call = self.review_manager.exact_call
         for source in load_active_sources():
-
             try:
                 self.review_manager.logger.info(f"Load {source.filename}")
 
@@ -923,7 +1042,7 @@ class Load(colrev.operation.Operation):
                 git_repo.git.stash("pop")
                 if not self.review_manager.high_level_operation:
                     print()
-            except (colrev_exceptions.ImportException) as exc:
+            except colrev_exceptions.ImportException as exc:
                 print(exc)
 
         if combine_commits and self.review_manager.dataset.has_changes():
@@ -932,6 +1051,8 @@ class Load(colrev.operation.Operation):
         self.review_manager.logger.info(
             f"{colors.GREEN}Completed load operation{colors.END}"
         )
+        if self.review_manager.in_ci_environment():
+            print("\n\n")
 
 
 if __name__ == "__main__":
