@@ -14,6 +14,7 @@ import timeout_decorator
 
 import colrev.exceptions as colrev_exceptions
 import colrev.operation
+import colrev.ops.built_in.pdf_prep.tei_prep
 import colrev.record
 import colrev.ui_cli.cli_colors as colors
 
@@ -25,6 +26,8 @@ class PDFPrep(colrev.operation.Operation):
     pdf_prepared: int
     not_prepared: int
 
+    pdf_prep_package_endpoints: dict
+
     def __init__(
         self,
         *,
@@ -32,7 +35,6 @@ class PDFPrep(colrev.operation.Operation):
         reprocess: bool = False,
         notify_state_transition_operation: bool = True,
     ) -> None:
-
         super().__init__(
             review_manager=review_manager,
             operations_type=colrev.operation.OperationsType.pdf_prep,
@@ -45,30 +47,26 @@ class PDFPrep(colrev.operation.Operation):
 
         self.cpus = 4
 
-        package_manager = self.review_manager.get_package_manager()
-        self.pdf_prep_package_endpoints = package_manager.load_packages(
-            package_type=colrev.env.package_manager.PackageEndpointType.pdf_prep,
-            selected_packages=self.review_manager.settings.pdf_prep.pdf_prep_package_endpoints,
-            operation=self,
-        )
-
     def __complete_successful_pdf_prep(
         self, *, record: colrev.record.Record, original_filename: str
     ) -> None:
-
         record.data.update(colrev_status=colrev.record.RecordState.pdf_prepared)
         pdf_path = self.review_manager.path / Path(record.data["file"])
         if pdf_path.suffix == ".pdf":
-            record.data.update(
-                colrev_pdf_id=record.get_colrev_pdf_id(
-                    review_manager=self.review_manager, pdf_path=pdf_path
+            try:
+                record.data.update(
+                    colrev_pdf_id=record.get_colrev_pdf_id(
+                        review_manager=self.review_manager, pdf_path=pdf_path
+                    )
+                )
+            except colrev_exceptions.ServiceNotAvailableException:
+                self.review_manager.logger.error(
+                    "Cannot create pdf-hash (Docker service not available)"
                 )
-            )
 
         # colrev_status == pdf_imported : means successful
         # create *_backup.pdf if record["file"] was changed
         if original_filename != record.data["file"]:
-
             current_file = self.review_manager.path / Path(record.data["file"])
             original_file = self.review_manager.path / Path(original_filename)
             if current_file.is_file() and original_file.is_file():
@@ -140,8 +138,15 @@ class PDFPrep(colrev.operation.Operation):
         for (
             pdf_prep_package_endpoint
         ) in self.review_manager.settings.pdf_prep.pdf_prep_package_endpoints:
-
             try:
+                if (
+                    pdf_prep_package_endpoint["endpoint"]
+                    not in self.pdf_prep_package_endpoints
+                ):
+                    self.review_manager.logger.error(
+                        f'Skip {pdf_prep_package_endpoint["endpoint"]} (not available)'
+                    )
+                    continue
                 endpoint = self.pdf_prep_package_endpoints[
                     pdf_prep_package_endpoint["endpoint"]  # type: ignore
                 ]
@@ -166,7 +171,7 @@ class PDFPrep(colrev.operation.Operation):
             ) as err:
                 self.review_manager.logger.error(
                     f'Error for {record.data["ID"]} '  # type: ignore
-                    f"(in {endpoint.settings.endpoint} : {err})"
+                    f"(in {endpoint.settings.endpoint} : {err})"  # type: ignore
                 )
                 record.data[
                     "colrev_status"
@@ -179,7 +184,7 @@ class PDFPrep(colrev.operation.Operation):
 
             if failed:
                 msg_str = f"{endpoint.settings.endpoint}"  # type: ignore
-                msg_str = msg_str.replace("colrev_built_in.", "")
+                msg_str = msg_str.replace("colrev.", "")
                 detailed_msgs.append(f"{colors.ORANGE}{msg_str}{colors.END}")
 
             if failed:
@@ -218,7 +223,6 @@ class PDFPrep(colrev.operation.Operation):
         return record.get_data()
 
     def __get_data(self, *, batch_size: int) -> dict:
-
         records_headers = self.review_manager.dataset.load_records_dict(
             header_only=True
         )
@@ -251,7 +255,6 @@ class PDFPrep(colrev.operation.Operation):
         return prep_data
 
     def __set_to_reprocess(self) -> None:
-
         records = self.review_manager.dataset.load_records_dict()
         for record_dict in records.values():
             if (
@@ -291,7 +294,6 @@ class PDFPrep(colrev.operation.Operation):
         self.review_manager.create_commit(msg="Update colrev_pdf_ids")
 
     def _print_stats(self, *, pdf_prep_record_list: list) -> None:
-
         self.pdf_prepared = len(
             [
                 r
@@ -352,6 +354,30 @@ class PDFPrep(colrev.operation.Operation):
 
         self.review_manager.save_settings()
 
+    def generate_tei(self) -> None:
+        """Generate TEI documents for included records"""
+
+        self.review_manager.logger.info("Generate TEI documents")
+        endpoint = colrev.ops.built_in.pdf_prep.tei_prep.TEIPDFPrep(
+            pdf_prep_operation=self, settings={"endpoint": "colrev.create_tei"}
+        )
+        records = self.review_manager.dataset.load_records_dict()
+        for record_dict in records.values():
+            if record_dict["colrev_status"] not in [
+                colrev.record.RecordState.rev_included,
+                colrev.record.RecordState.rev_synthesized,
+            ]:
+                continue
+            self.review_manager.logger.info(record_dict["ID"])
+            try:
+                endpoint.prep_pdf(
+                    pdf_prep_operation=self,
+                    record=colrev.record.Record(data=record_dict),
+                    pad=0,
+                )
+            except colrev_exceptions.TEIException:
+                self.review_manager.logger.error("Eror generating TEI")
+
     def main(
         self,
         *,
@@ -360,11 +386,20 @@ class PDFPrep(colrev.operation.Operation):
     ) -> None:
         """Prepare PDFs (main entrypoint)"""
 
+        if self.review_manager.in_ci_environment():
+            raise colrev_exceptions.ServiceNotAvailableException(
+                dep="colrev pdf-prep",
+                detailed_trace="pdf-prep not available in ci environment",
+            )
+
         self.review_manager.logger.info("Prep PDFs")
         self.review_manager.logger.info(
             "Prepare PDFs, validating them against their metadata, "
             "removing additional pages, ensuring machine readability."
         )
+        self.review_manager.logger.info(
+            "See https://colrev.readthedocs.io/en/latest/manual/pdf_retrieval/pdf_prep.html"
+        )
 
         # temporary fix: remove all lines containing PDFType1Font from log.
         # https://github.com/pdfminer/pdfminer.six/issues/282
@@ -380,6 +415,14 @@ class PDFPrep(colrev.operation.Operation):
 
         pdf_prep_data = self.__get_data(batch_size=batch_size)
 
+        package_manager = self.review_manager.get_package_manager()
+        self.pdf_prep_package_endpoints = package_manager.load_packages(
+            package_type=colrev.env.package_manager.PackageEndpointType.pdf_prep,
+            selected_packages=self.review_manager.settings.pdf_prep.pdf_prep_package_endpoints,
+            operation=self,
+            only_ci_supported=self.review_manager.in_ci_environment(),
+        )
+
         self.review_manager.logger.info(
             "PDFs to prep".ljust(38) + f'{pdf_prep_data["nr_tasks"]} PDFs'
         )
@@ -393,12 +436,11 @@ class PDFPrep(colrev.operation.Operation):
                 )
 
         else:
-
             endpoint_names = [
                 s["endpoint"]
                 for s in self.review_manager.settings.pdf_prep.pdf_prep_package_endpoints
             ]
-            if "colrev_built_in.create_tei" in endpoint_names:  # type: ignore
+            if "colrev.create_tei" in endpoint_names:  # type: ignore
                 pool = Pool(mp.cpu_count() // 2)
             else:
                 pool = Pool(self.cpus)
