@@ -13,8 +13,10 @@ from datetime import timedelta
 from multiprocessing import Value
 from multiprocessing.pool import ThreadPool as Pool
 from pathlib import Path
+from typing import Optional
 
 import timeout_decorator
+from requests.exceptions import ConnectionError as requests_ConnectionError
 from requests.exceptions import ReadTimeout
 
 import colrev.env.utils
@@ -24,6 +26,7 @@ import colrev.record
 import colrev.settings
 import colrev.ui_cli.cli_colors as colors
 
+# pylint: disable=too-many-lines
 
 # logging.getLogger("urllib3").setLevel(logging.ERROR)
 logging.getLogger("requests_cache").setLevel(logging.ERROR)
@@ -48,6 +51,8 @@ class Prep(colrev.operation.Operation):
 
     pad: int
 
+    polish: bool
+
     prep_package_endpoints: dict[str, typing.Any]
 
     requests_headers = {
@@ -124,6 +129,7 @@ class Prep(colrev.operation.Operation):
 
         self.retrieval_similarity = retrieval_similarity
 
+        self.polish = False
         self.debug_mode = False
         self.pad = 0
         self.__stats: typing.Dict[str, typing.List[timedelta]] = {}
@@ -176,7 +182,6 @@ class Prep(colrev.operation.Operation):
         preparation_record: colrev.record.PrepRecord,
         prep_package_endpoint: colrev.env.package_manager.PrepPackageEndpointInterface,
     ) -> None:
-
         if not self.debug_mode:
             return
 
@@ -193,7 +198,7 @@ class Prep(colrev.operation.Operation):
             self.review_manager.logger.info(
                 "To correct errors in the endpoint,"
                 " open an issue at "
-                "https://github.com/geritwagner/colrev/issues"
+                "https://github.com/CoLRev-Ecosystem/colrev/issues"
             )
             self.review_manager.logger.info(
                 "To correct potential errors at source,"
@@ -219,7 +224,7 @@ class Prep(colrev.operation.Operation):
 
         record: colrev.record.PrepRecord = item["record"]
 
-        if not record.status_to_prepare():
+        if not record.status_to_prepare() and not self.polish:
             return record.get_data()
 
         if self.review_manager.verbose_mode:
@@ -233,8 +238,12 @@ class Prep(colrev.operation.Operation):
         for prep_round_package_endpoint in deepcopy(
             item["prep_round_package_endpoints"]
         ):
-
             try:
+                if (
+                    prep_round_package_endpoint["endpoint"].lower()
+                    not in self.prep_package_endpoints
+                ):
+                    continue
                 endpoint = self.prep_package_endpoints[
                     prep_round_package_endpoint["endpoint"].lower()
                 ]
@@ -264,9 +273,10 @@ class Prep(colrev.operation.Operation):
 
                 if preparation_record.preparation_save_condition():
                     record.update_by_record(update_record=preparation_record)
-                    record.update_masterdata_provenance()
+                    if not self.polish:
+                        record.update_masterdata_provenance()
 
-                if preparation_record.preparation_break_condition():
+                if preparation_record.preparation_break_condition() and not self.polish:
                     record.update_by_record(update_record=preparation_record)
                     break
             except (timeout_decorator.timeout_decorator.TimeoutError, ReadTimeout):
@@ -291,13 +301,14 @@ class Prep(colrev.operation.Operation):
                     raise exc
 
         if not self.review_manager.verbose_mode:
-
             # pylint: disable=redefined-outer-name,invalid-name
             with PREP_COUNTER.get_lock():
-                PREP_COUNTER.value += 1
+                PREP_COUNTER.value += 1  # type: ignore
             progress = ""
             if item["nr_items"] > 100:
-                progress = f"({PREP_COUNTER.value}/{item['nr_items']}) ".rjust(12, " ")
+                progress = f"({PREP_COUNTER.value}/{item['nr_items']}) ".rjust(  # type: ignore
+                    12, " "
+                )
 
             if record.preparation_break_condition():
                 if (
@@ -310,33 +321,44 @@ class Prep(colrev.operation.Operation):
                             + f"Detected: {record.data.get('prescreen_exclusion', 'NA')}"
                             + f"{colors.END}"
                         )
-
+                    target_state = "rev_prescreen_excluded"
+                    if self.polish:
+                        target_state = prior_state
                     self.review_manager.logger.info(
                         f" {colors.RED}{record.data['ID']}".ljust(46)
-                        + f"{progress}{prior_state} →  rev_prescreen_excluded"
+                        + f"{progress}{prior_state} →  {target_state}"
                         + f"{colors.END}"
                     )
                 else:
+                    target_state = "md_needs_manual_preparation"
+                    if self.polish:
+                        target_state = prior_state
                     self.review_manager.logger.info(
                         f" {colors.ORANGE}{record.data['ID']}".ljust(46)
-                        + f"{progress}{prior_state} →  md_needs_manual_preparation{colors.END}"
+                        + f"{progress}{prior_state} →  {target_state}{colors.END}"
                     )
 
             elif record.preparation_save_condition():
                 curation_addition = "   "
                 if record.masterdata_is_curated():
                     curation_addition = " ✔ "
+                target_state = "md_prepared"
+                if self.polish:
+                    target_state = prior_state
                 self.review_manager.logger.info(
                     f" {colors.GREEN}{record.data['ID']}".ljust(46)
-                    + f"{progress}{prior_state} →  md_prepared{colors.END}{curation_addition}"
+                    + f"{progress}{prior_state} →  {target_state}{colors.END}{curation_addition}"
                 )
             else:
+                target_state = "md_needs_manual_preparation"
+                if self.polish:
+                    target_state = prior_state
                 self.review_manager.logger.info(
                     f" {colors.ORANGE}{record.data['ID']}".ljust(46)
-                    + f"{progress}{prior_state} →  md_needs_manual_preparation{colors.END}"
+                    + f"{progress}{prior_state} →  {target_state}{colors.END}"
                 )
 
-        if self.last_round:
+        if self.last_round and not self.polish:
             if record.status_to_prepare():
                 for key in list(record.data.keys()):
                     if key not in self.fields_to_keep:
@@ -347,6 +369,9 @@ class Prep(colrev.operation.Operation):
                 # Note: update_masterdata_provenance sets to md_needs_manual_preparation
                 record.update_masterdata_provenance()
 
+        if self.polish:
+            record.data["colrev_status"] = prior_state
+
         return record.get_data()
 
     def __select_record_list_for_reset(self, *, record_list: list[dict]) -> list[dict]:
@@ -394,7 +419,6 @@ class Prep(colrev.operation.Operation):
         return revlist
 
     def __reset(self, *, record_list: list[dict]) -> None:
-
         record_list = self.__select_record_list_for_reset(record_list=record_list)
         revlist = self.__get_revlist_for_reset()
 
@@ -416,7 +440,6 @@ class Prep(colrev.operation.Operation):
                 ):
                     continue
                 for record_to_unmerge, record in record_reset_list:
-
                     if any(
                         o in prior_record["colrev_origin"]
                         for o in record["colrev_origin"]
@@ -559,8 +582,7 @@ class Prep(colrev.operation.Operation):
         prep_round.prep_package_endpoints.append({"endpoint": "custom_prep_script"})
         self.review_manager.save_settings()
 
-    def __load_prep_data(self) -> dict:
-
+    def __load_prep_data(self, *, polish: bool = False) -> dict:
         records_headers = self.review_manager.dataset.load_records_dict(
             header_only=True
         )
@@ -576,6 +598,9 @@ class Prep(colrev.operation.Operation):
             colrev.record.RecordState.md_imported,
             colrev.record.RecordState.md_needs_manual_preparation,
         ]
+        if polish:
+            r_states_to_prepare = list(colrev.record.RecordState)
+
         items = list(
             self.review_manager.dataset.read_next_record(
                 conditions=[{"colrev_status": s} for s in r_states_to_prepare]
@@ -594,8 +619,10 @@ class Prep(colrev.operation.Operation):
         self,
         *,
         prep_round: colrev.settings.PrepRound,
-        debug_file: Path = None,
+        debug_file: Optional[Path] = None,
         debug_ids: str,
+        max_n: int = 2500,
+        polish: bool = False,
     ) -> list:
         if self.debug_mode:
             prepare_data = self.__load_prep_data_for_debug(
@@ -604,7 +631,7 @@ class Prep(colrev.operation.Operation):
             if prepare_data["nr_tasks"] == 0:
                 print("ID not found in history.")
         else:
-            prepare_data = self.__load_prep_data()
+            prepare_data = self.__load_prep_data(polish=polish)
 
         if self.debug_mode:
             self.review_manager.logger.info(
@@ -640,7 +667,6 @@ class Prep(colrev.operation.Operation):
         original_records: list[dict],
         condition_state: colrev.record.RecordState,
     ) -> list:
-
         retrieved, prior_records = [], []
         for (
             prior_records_dict
@@ -665,9 +691,8 @@ class Prep(colrev.operation.Operation):
         return prior_records
 
     def __load_prep_data_for_debug(
-        self, *, debug_ids: str, debug_file: Path = None
+        self, *, debug_ids: str, debug_file: Optional[Path] = None
     ) -> dict:
-
         if debug_file:
             with open(debug_file, encoding="utf8") as target_db:
                 records_dict = self.review_manager.dataset.load_records_dict(
@@ -712,11 +737,10 @@ class Prep(colrev.operation.Operation):
     def __setup_prep_round(
         self, *, i: int, prep_round: colrev.settings.PrepRound
     ) -> None:
-
         # pylint: disable=redefined-outer-name,invalid-name
         PREP_COUNTER = Value("i", 0)
         with PREP_COUNTER.get_lock():
-            PREP_COUNTER.value = 0
+            PREP_COUNTER.value = 0  # type: ignore
 
         self.first_round = bool(i == 0)
 
@@ -728,7 +752,7 @@ class Prep(colrev.operation.Operation):
         # because it must always be executed at the end
         if prep_round.name not in ["source_specific_prep", "exclusion"]:
             prep_round.prep_package_endpoints.append(
-                {"endpoint": "colrev_built_in.update_metadata_status"}
+                {"endpoint": "colrev.update_metadata_status"}
             )
 
         if self.debug_mode:
@@ -744,7 +768,7 @@ class Prep(colrev.operation.Operation):
 
         required_prep_package_endpoints = list(prep_round.prep_package_endpoints)
         required_prep_package_endpoints.append(
-            {"endpoint": "colrev_built_in.update_metadata_status"}
+            {"endpoint": "colrev.update_metadata_status"}
         )
 
         package_manager = self.review_manager.get_package_manager()
@@ -754,13 +778,28 @@ class Prep(colrev.operation.Operation):
             package_type=colrev.env.package_manager.PackageEndpointType.prep,
             selected_packages=required_prep_package_endpoints,
             operation=self,
+            only_ci_supported=self.review_manager.in_ci_environment(),
         )
+        non_available_endpoints = [
+            x["endpoint"].lower()
+            for x in required_prep_package_endpoints
+            if x["endpoint"].lower() not in self.prep_package_endpoints
+        ]
+        if non_available_endpoints:
+            if self.review_manager.in_ci_environment():
+                raise colrev_exceptions.ServiceNotAvailableException(
+                    dep=f"colrev prep ({','.join(non_available_endpoints)})",
+                    detailed_trace="prep not available in ci environment",
+                )
+            raise colrev_exceptions.ServiceNotAvailableException(
+                dep="colrev prep", detailed_trace="prep not available"
+            )
 
-        for endpoin_name, endpoint in self.prep_package_endpoints.items():
+        for endpoint_name, endpoint in self.prep_package_endpoints.items():
             check_function = getattr(endpoint, "check_availability", None)
             if callable(check_function):
                 self.review_manager.logger.debug(
-                    f"Check availability of {endpoin_name}"
+                    f"Check availability of {endpoint_name}"
                 )
                 endpoint.check_availability(source_operation=self)  # type: ignore
 
@@ -840,23 +879,47 @@ class Prep(colrev.operation.Operation):
                 + f"{colors.RED}{nr_recs}{colors.END}".rjust(20, " ")
             )
 
+    def skip_prep(self) -> None:
+        """Skip the preparation"""
+
+        records = self.review_manager.dataset.load_records_dict()
+
+        for record in records.values():
+            if colrev.record.RecordState.md_imported == record["colrev_status"]:
+                record["colrev_status"] = colrev.record.RecordState.md_prepared
+        self.review_manager.dataset.save_records_dict(records=records)
+        self.review_manager.dataset.add_record_changes()
+        self.review_manager.create_commit(msg="Skip prep")
+
     def main(
         self,
         *,
         keep_ids: bool = False,
         debug_ids: str = "NA",
-        debug_file: Path = None,
+        debug_file: Optional[Path] = None,
+        cpu: int = 4,
+        polish: bool = False,
     ) -> None:
         """Preparation of records (main entrypoint)"""
 
         # pylint: disable=too-many-locals
         # pylint: disable=too-many-branches
         # pylint: disable=too-many-statements
-
-        self.review_manager.logger.info("Prep")
+        if not polish:
+            self.review_manager.logger.info("Prep")
+        else:
+            self.review_manager.logger.info("Prep (polish mode)")
         self.review_manager.logger.info(
             "Prep completes and corrects record metadata based on APIs and preparation rules."
         )
+        self.polish = polish
+        if self.polish:
+            self.review_manager.logger.info(
+                "Polish mode: consider all records but prevent state transitions."
+            )
+        self.review_manager.logger.info(
+            "See https://colrev.readthedocs.io/en/latest/manual/metadata_retrieval/prep.html"
+        )
 
         # Note: for unit testing, we use a simple loop (instead of parallel)
         # to ensure that the IDs of feed records don't change
@@ -878,65 +941,87 @@ class Prep(colrev.operation.Operation):
 
         prep_commit_id = "HEAD"
 
-        for i, prep_round in enumerate(self.review_manager.settings.prep.prep_rounds):
+        try:
+            for i, prep_round in enumerate(
+                self.review_manager.settings.prep.prep_rounds
+            ):
+                self.__setup_prep_round(i=i, prep_round=prep_round)
 
-            self.__setup_prep_round(i=i, prep_round=prep_round)
-
-            preparation_data = self.__get_preparation_data(
-                prep_round=prep_round, debug_file=debug_file, debug_ids=debug_ids
-            )
-            previous_preparation_data = deepcopy(preparation_data)
-
-            if len(preparation_data) == 0:
-                print("No records to prepare.")
-                return
-
-            if self.debug_mode or __unit_testing:
-                # Note: preparation_data is not turned into a list of records.
-                prepared_records = []
-                for item in preparation_data:
-                    record = self.prepare(item)
-                    prepared_records.append(record)
-            else:
-
-                prep_pe_names = [
-                    r["endpoint"] for r in prep_round.prep_package_endpoints
-                ]
-                if "colrev_built_in.exclude_languages" in prep_pe_names:  # type: ignore
-                    self.review_manager.logger.info(
-                        "Info: The language detector requires RAM and may take longer"
-                    )
-                    pool = Pool(mp.cpu_count() // 2)
-                else:
-                    # Note : if we use too many CPUS, a "too many open files" exception is thrown
-                    pool = Pool(self.cpus * 4)
-                prepared_records = pool.map(self.prepare, preparation_data)
-                pool.close()
-                pool.join()
-
-            self.__log_record_change_scores(
-                preparation_data=previous_preparation_data,
-                prepared_records=prepared_records,
-            )
-
-            if not self.debug_mode:
-                self.review_manager.dataset.save_records_dict(
-                    records={r["ID"]: r for r in prepared_records}, partial=True
+                preparation_data = self.__get_preparation_data(
+                    prep_round=prep_round,
+                    debug_file=debug_file,
+                    debug_ids=debug_ids,
+                    polish=polish,
                 )
+                previous_preparation_data = deepcopy(preparation_data)
 
-                self.__log_details(prepared_records=prepared_records)
-
-                self.review_manager.create_commit(
-                    msg=f"Prepare records ({prep_round.name})",
-                )
-                prep_commit_id = (
-                    self.review_manager.dataset.get_repo().head.commit.hexsha
-                )
-                if not self.review_manager.high_level_operation:
+                if len(preparation_data) == 0:
+                    self.review_manager.logger.info("No records to prepare.")
                     print()
-            self.review_manager.reset_report_logger()
+                    return
 
-            self.__print_stats()
+                if self.debug_mode or __unit_testing:
+                    # Note: preparation_data is not turned into a list of records.
+                    prepared_records = []
+                    for item in preparation_data:
+                        record = self.prepare(item)
+                        prepared_records.append(record)
+                else:
+                    prep_pe_names = [
+                        r["endpoint"] for r in prep_round.prep_package_endpoints
+                    ]
+                    if "colrev.exclude_languages" in prep_pe_names:  # type: ignore
+                        self.review_manager.logger.info(
+                            "Info: The language detector requires RAM and may take longer"
+                        )
+                        pool = Pool(mp.cpu_count() // 2)
+                    else:
+                        # Note : if we use too many CPUS,
+                        # a "too many open files" exception is thrown
+                        pool = Pool(cpu)
+                    prepared_records = pool.map(self.prepare, preparation_data)
+                    pool.close()
+                    pool.join()
+
+                self.__log_record_change_scores(
+                    preparation_data=previous_preparation_data,
+                    prepared_records=prepared_records,
+                )
+
+                if not self.debug_mode:
+                    self.review_manager.dataset.save_records_dict(
+                        records={r["ID"]: r for r in prepared_records}, partial=True
+                    )
+
+                    self.__log_details(prepared_records=prepared_records)
+
+                    self.review_manager.create_commit(
+                        msg=f"Prepare records ({prep_round.name})",
+                    )
+                    prep_commit_id = (
+                        self.review_manager.dataset.get_repo().head.commit.hexsha
+                    )
+                    if not self.review_manager.high_level_operation:
+                        print()
+                self.review_manager.reset_report_logger()
+
+                self.__print_stats()
+
+        except requests_ConnectionError as exc:
+            if "OSError(24, 'Too many open files" in str(exc):
+                raise colrev_exceptions.ServiceNotAvailableException(
+                    "Too many files opened (OSError, Errno24). "
+                    "To use a smaller number of parallel processes, run colrev prep --cpu 1"
+                ) from exc
+            raise exc
+
+        except OSError as exc:
+            if 24 == exc.errno:
+                raise colrev_exceptions.ServiceNotAvailableException(
+                    "Too many files opened (OSError, Errno24). "
+                    "To use a smaller number of parallel processes, run colrev prep --cpu 1"
+                ) from exc
+            raise exc
 
         if not keep_ids and not self.debug_mode:
             self.review_manager.logger.info("Set record IDs")
@@ -957,6 +1042,8 @@ class Prep(colrev.operation.Operation):
         self.review_manager.logger.info(
             f"{colors.GREEN}Completed prep operation{colors.END}"
         )
+        if self.review_manager.in_ci_environment():
+            print("\n\n")
 
 
 if __name__ == "__main__":
