@@ -7,6 +7,7 @@ import io
 import typing
 from dataclasses import dataclass
 from pathlib import Path
+from typing import Optional
 
 import yaml
 
@@ -73,7 +74,7 @@ class Status(colrev.operation.Operation):
         return analytics_dict
 
     def get_review_status_report(
-        self, *, records: dict = None, colors: bool = True
+        self, *, records: Optional[dict] = None, colors: bool = True
     ) -> str:
         """Get the review status report"""
 
@@ -107,9 +108,8 @@ class StatusStats:
         self,
         *,
         review_manager: colrev.review_manager.ReviewManager,
-        records: dict = None,
+        records: Optional[dict] = None,
     ) -> None:
-
         self.review_manager = review_manager
         colrev.operation.CheckOperation(review_manager=review_manager)
 
@@ -345,7 +345,6 @@ class StatusStats:
             committed_origin,
             committed_colrev_status,
         ) in committed_origin_states_dict.items():
-
             transitioned_record = {
                 "origin": committed_origin,
                 "source": committed_colrev_status,
@@ -412,10 +411,11 @@ class StatusStats:
     def get_active_operations(self, *, current_origin_states_dict: dict) -> list:
         """Get the active processing functions"""
 
-        active_operations = []
+        active_operations: typing.List[str] = []
         for state in set(current_origin_states_dict.values()):
-            srec = colrev.record.RecordStateModel(state=state)
-            valid_transitions = srec.get_valid_transitions()
+            valid_transitions = colrev.record.RecordStateModel.get_valid_transitions(
+                state=state
+            )
             active_operations.extend(valid_transitions)
 
         self.review_manager.logger.debug(f"active_operations: {set(active_operations)}")
@@ -553,7 +553,6 @@ class StatusStats:
             )
 
         def __get_nr_search(self, *, search_dir: Path) -> int:
-
             if not search_dir.is_dir():
                 return 0
             bib_files = search_dir.glob("*.bib")
