@@ -6,6 +6,7 @@ import datetime
 import re
 import typing
 from pathlib import Path
+from typing import Optional
 
 from tqdm import tqdm
 
@@ -18,7 +19,6 @@ class Validate(colrev.operation.Operation):
     """Validate changes"""
 
     def __init__(self, *, review_manager: colrev.review_manager.ReviewManager) -> None:
-
         super().__init__(
             review_manager=review_manager,
             operations_type=colrev.operation.OperationsType.check,
@@ -27,7 +27,6 @@ class Validate(colrev.operation.Operation):
         self.cpus = 4
 
     def __load_prior_records_dict(self, *, target_commit: str) -> dict:
-
         git_repo = self.review_manager.dataset.get_repo()
 
         revlist = (
@@ -115,7 +114,6 @@ class Validate(colrev.operation.Operation):
         change_diff = []
         merged_records = False
         for record in records:
-
             if "changed_in_target_commit" not in record:
                 continue
             del record["changed_in_target_commit"]
@@ -196,7 +194,9 @@ class Validate(colrev.operation.Operation):
 
         return change_diff
 
-    def load_changed_records(self, *, target_commit: str = None) -> list[dict]:
+    def load_changed_records(
+        self, *, target_commit: Optional[str] = None
+    ) -> list[dict]:
         """Load the records that were changed in the target commit"""
         if target_commit is None:
             self.review_manager.logger.info("Loading data...")
@@ -255,6 +255,7 @@ class Validate(colrev.operation.Operation):
         return validation_details
 
     def __set_scope_based_on_target_commit(self, *, target_commit: str) -> str:
+        # pylint: disable=too-many-branches
 
         if not target_commit:
             target_commit = self.review_manager.dataset.get_last_commit_sha()
@@ -266,9 +267,10 @@ class Validate(colrev.operation.Operation):
                 commit.hexsha,
                 commit.message,
             )
-            for commit in git_repo.iter_commits(paths="status.yaml")
+            for commit in git_repo.iter_commits()
         )
-        scope = "NA"
+
+        scope = ""
         # Note : simple heuristic: commit messages
         for commit_id, msg in revlist:
             if commit_id == target_commit:
@@ -276,11 +278,25 @@ class Validate(colrev.operation.Operation):
                     scope = "prepare"
                 elif "colrev dedupe" in msg:
                     scope = "dedupe"
+                elif any(
+                    x in msg
+                    for x in [
+                        "colrev init",
+                        "colrev load",
+                        "colrev pdf-get",
+                        "colrev pdf-prep",
+                        "colrev screen",
+                        "colrev prescreen",
+                    ]
+                ):
+                    scope = "general"
                 else:
-                    scope = "unspecified"
+                    scope = "general"
+            if scope != "":
+                break
 
         # Otherwise: compare records
-        if scope in ["NA", "unspecified"]:
+        if scope in ["general"]:
             # detect transition types in the respective commit and
             # use them to calculate the validation_details
             records: typing.Dict[str, typing.Dict] = {}
@@ -299,8 +315,6 @@ class Validate(colrev.operation.Operation):
             # Note : still very simple heuristics...
             if len(records) != len(hist_records):
                 scope = "dedupe"
-            else:
-                scope = "prepare"
 
         return scope
 
@@ -380,7 +394,6 @@ class Validate(colrev.operation.Operation):
                 .decode("utf-8")
             )
             if "screen" in commit.message or "prescreen" in commit.message:
-
                 prescreen_validation = self.validate_merge_prescreen_screen(
                     commit_sha=commit.hexsha,
                     current_branch_records=records_branch_1,
@@ -413,7 +426,6 @@ class Validate(colrev.operation.Operation):
                 commit_object = git_repo.commit(scope)
                 commit = commit_object.hexsha
             except ValueError:
-
                 for commit_candidate in git_repo.iter_commits():
                     if str(commit_candidate.tree) == scope:
                         commit = commit_candidate.hexsha
@@ -426,13 +438,17 @@ class Validate(colrev.operation.Operation):
                         parameter="validate.scope", value=scope, options=valid_options
                     )
 
-        assert re.match(r"[0-9a-f]{5,40}", commit)
+        if not re.match(r"[0-9a-f]{5,40}", commit):
+            raise colrev_exceptions.ParameterError(
+                parameter="commit",
+                value=scope,
+                options=[x.hexsha for x in git_repo.iter_commits()],
+            )
         return commit
 
     def __deduplicated_records(
         self, *, records: list[dict], prior_records_dict: dict
     ) -> bool:
-
         return {",".join(sorted(x)) for x in [r["colrev_origin"] for r in records]} != {
             ",".join(sorted(x))
             for x in [r["colrev_origin"] for r in prior_records_dict.values()]
@@ -483,6 +499,17 @@ class Validate(colrev.operation.Operation):
             )
         return validation_details
 
+    def __get_relative_commit(self, target_commit: str) -> str:
+        git_repo = self.review_manager.dataset.get_repo()
+
+        relative_to_head = 0
+        for commit in git_repo.iter_commits():
+            if target_commit == commit.hexsha:
+                return f"HEAD~{relative_to_head}"
+            relative_to_head += 1
+
+        return target_commit
+
     def main(
         self,
         *,
@@ -512,6 +539,15 @@ class Validate(colrev.operation.Operation):
             filter_setting = self.__set_scope_based_on_target_commit(
                 target_commit=target_commit
             )
+        if "general" == filter_setting:
+            validation_details["general"] = {
+                "commit": target_commit,
+                "commit_relative": self.__get_relative_commit(
+                    target_commit=target_commit
+                ),
+            }
+            return validation_details
+
         self.review_manager.logger.info(f"Filter: {filter_setting} changes")
 
         # extension: filter_setting for changes of contributor (git author)
