@@ -5,9 +5,11 @@ from __future__ import annotations
 import logging
 import os
 import pprint
+import typing
 from dataclasses import asdict
 from datetime import timedelta
 from pathlib import Path
+from typing import Optional
 
 import requests_cache
 import yaml
@@ -27,6 +29,7 @@ class ReviewManager:
     """Class for managing individual CoLRev review project (repositories)"""
 
     # pylint: disable=too-many-instance-attributes
+    # pylint: disable=too-many-statements
     # pylint: disable=too-many-public-methods
     # pylint: disable=import-outside-toplevel
     # pylint: disable=redefined-outer-name
@@ -38,12 +41,18 @@ class ReviewManager:
     SETTINGS_RELATIVE = Path("settings.json")
     REPORT_RELATIVE = Path(".report.log")
     CORRECTIONS_PATH_RELATIVE = Path(".corrections")
-    PDF_DIR_RELATIVE = Path("data/pdfs")
-    SEARCHDIR_RELATIVE = Path("data/search")
     README_RELATIVE = Path("readme.md")
     STATUS_RELATIVE = Path("status.yaml")
+
+    SEARCHDIR_RELATIVE = Path("data/search")
+    PREP_RELATIVE = Path("data/prep")
+    DEDUPE_RELATIVE = Path("data/dedupe")
+    PRESCREEN_RELATIVE = Path("data/prescreen")
+    PDF_DIR_RELATIVE = Path("data/pdfs")
+    SCREEN_RELATIVE = Path("data/screen")
+    DATA_DIR_RELATIVE = Path("data/data")
+
     OUTPUT_DIR_RELATIVE = Path("output")
-    DATA_DIR_RELATIVE = Path("data")
 
     dataset: colrev.dataset.Dataset
     """The review dataset object"""
@@ -54,7 +63,7 @@ class ReviewManager:
     def __init__(
         self,
         *,
-        path_str: str = None,
+        path_str: Optional[str] = None,
         force_mode: bool = False,
         verbose_mode: bool = False,
         debug_mode: bool = False,
@@ -62,7 +71,6 @@ class ReviewManager:
         navigate_to_home_dir: bool = True,
         exact_call: str = "",
     ) -> None:
-
         self.force_mode = force_mode
         """Force mode variable (bool)"""
         self.verbose_mode = verbose_mode
@@ -81,12 +89,19 @@ class ReviewManager:
         self.settings_path = self.path / self.SETTINGS_RELATIVE
         self.report_path = self.path / self.REPORT_RELATIVE
         self.corrections_path = self.path / self.CORRECTIONS_PATH_RELATIVE
-        self.pdf_dir = self.path / self.PDF_DIR_RELATIVE
-        self.search_dir = self.path / self.SEARCHDIR_RELATIVE
         self.readme = self.path / self.README_RELATIVE
         self.status = self.path / self.STATUS_RELATIVE
-        self.output_dir = self.path / self.OUTPUT_DIR_RELATIVE
+
+        self.search_dir = self.path / self.SEARCHDIR_RELATIVE
+        self.prep_dir = self.path / self.PREP_RELATIVE
+        self.dedupe_dir = self.path / self.DEDUPE_RELATIVE
+        self.prescreen_dir = self.path / self.PRESCREEN_RELATIVE
+        self.pdf_dir = self.path / self.PDF_DIR_RELATIVE
+        self.screen_dir = self.path / self.SCREEN_RELATIVE
         self.data_dir = self.path / self.DATA_DIR_RELATIVE
+
+        self.output_dir = self.path / self.OUTPUT_DIR_RELATIVE
+
         self.exact_call = exact_call
 
         try:
@@ -96,9 +111,6 @@ class ReviewManager:
                 self.pdf_dir.mkdir(exist_ok=True)
                 self.output_dir.mkdir(exist_ok=True)
 
-            # Start LocalIndex to prevent waiting times
-            self.get_local_index(startup_without_waiting=True)
-
             if self.debug_mode:
                 self.report_logger = colrev.logger.setup_report_logger(
                     review_manager=self, level=logging.DEBUG
@@ -117,10 +129,6 @@ class ReviewManager:
                 )
 
             self.environment_manager = self.get_environment_manager()
-            (
-                self.committer,
-                self.email,
-            ) = self.environment_manager.get_name_mail_from_git()
 
             self.p_printer = pprint.PrettyPrinter(indent=4, width=140, compact=False)
             self.settings = self.load_settings()
@@ -133,7 +141,23 @@ class ReviewManager:
             else:
                 raise exc
 
-    def __get_project_home_dir(self, *, path_str: str = None) -> Path:
+        self.__check_update()
+
+    def __check_update(self) -> None:
+        if not hasattr(self, "dataset"):
+            return
+        # Once the following has run for all repositories,
+        # it should only be called when the versions differ.
+        # last_version, current_version = self.get_colrev_versions()
+        # if last_version != current_version:
+        upgrade_operation = self.get_upgrade()
+        upgrade_operation.main()
+
+    def get_committer(self) -> typing.Tuple[str, str]:
+        """Get the committer name and email"""
+        return self.environment_manager.get_name_mail_from_git()
+
+    def __get_project_home_dir(self, *, path_str: Optional[str] = None) -> Path:
         if path_str:
             return Path(path_str)
 
@@ -147,7 +171,7 @@ class ReviewManager:
 
     def load_settings(self) -> colrev.settings.Settings:
         """Load the settings"""
-        self.settings = colrev.settings.load_settings(review_manager=self)
+        self.settings = colrev.settings.load_settings(settings_path=self.settings_path)
         return self.settings
 
     def save_settings(self) -> None:
@@ -256,9 +280,9 @@ class ReviewManager:
         """Notify the review_manager about the next operation"""
 
         try:
+            self.notified_next_operation = operation.type
             if state_transition:
                 operation.check_precondition()
-            self.notified_next_operation = operation.type
             self.dataset.reset_log_if_no_changes()
         except AttributeError as exc:
             if self.force_mode:
@@ -267,7 +291,7 @@ class ReviewManager:
                 raise exc
 
     def update_status_yaml(
-        self, *, add_to_git: bool = True, records: dict = None
+        self, *, add_to_git: bool = True, records: Optional[dict] = None
     ) -> None:
         """Update the status.yaml"""
 
@@ -284,7 +308,7 @@ class ReviewManager:
         msg: str,
         manual_author: bool = False,
         script_call: str = "",
-        saved_args: dict = None,
+        saved_args: Optional[dict] = None,
     ) -> bool:
         """Create a commit (including a commit report)"""
         import colrev.ops.commit
@@ -343,7 +367,7 @@ class ReviewManager:
         return colrev.checker.Checker(review_manager=self)
 
     def get_status_stats(
-        self, *, records: dict = None
+        self, *, records: Optional[dict] = None
     ) -> colrev.ops.status.StatusStats:
         """Get a status stats object"""
 
@@ -358,25 +382,23 @@ class ReviewManager:
 
     @classmethod
     def get_local_index(
-        cls, *, startup_without_waiting: bool = False, verbose_mode: bool = False
+        cls, *, verbose_mode: bool = False
     ) -> colrev.env.local_index.LocalIndex:
         """Get a local-index object"""
 
         import colrev.env.local_index
 
-        return colrev.env.local_index.LocalIndex(
-            startup_without_waiting=startup_without_waiting, verbose_mode=verbose_mode
-        )
+        return colrev.env.local_index.LocalIndex(verbose_mode=verbose_mode)
 
     @classmethod
-    def get_package_manager(  # type: ignore
-        cls, **kwargs
+    def get_package_manager(
+        cls, *, verbose_mode: bool = True
     ) -> colrev.env.package_manager.PackageManager:
         """Get a package manager object"""
 
         import colrev.env.package_manager
 
-        return colrev.env.package_manager.PackageManager(**kwargs)
+        return colrev.env.package_manager.PackageManager(verbose=verbose_mode)
 
     @classmethod
     def get_grobid_service(cls) -> colrev.env.grobid_service.GrobidService:
@@ -389,7 +411,7 @@ class ReviewManager:
         )
 
     def get_tei(
-        self, *, pdf_path: Path = None, tei_path: Path = None
+        self, *, pdf_path: Optional[Path] = None, tei_path: Optional[Path] = None
     ) -> colrev.env.tei_parser.TEIParser:  # type: ignore
         """Get a tei object"""
 
@@ -443,7 +465,7 @@ class ReviewManager:
         """Get the pdf-hash-service object"""
         import colrev.env.pdf_hash_service
 
-        return colrev.env.pdf_hash_service.PDFHashService(review_manager=self)
+        return colrev.env.pdf_hash_service.PDFHashService(logger=self.logger)
 
     @classmethod
     def get_resources(cls) -> colrev.env.resources.Resources:
@@ -458,8 +480,9 @@ class ReviewManager:
         cls,
         review_type: str,
         example: bool = False,
+        light: bool = False,
         local_pdf_collection: bool = False,
-        target_path: Path = None,
+        target_path: Optional[Path] = None,
         exact_call: str = "",
     ) -> colrev.ops.init.Initializer:
         """Get an init operation object"""
@@ -468,6 +491,7 @@ class ReviewManager:
         return colrev.ops.init.Initializer(
             review_type=review_type,
             example=example,
+            light=light,
             local_pdf_collection=local_pdf_collection,
             target_path=target_path,
             exact_call=exact_call,
@@ -683,7 +707,7 @@ class ReviewManager:
         return colrev.ops.search_sources.SearchSources(review_manager=self)
 
     def get_review_types(
-        self, *, review_type: str = None
+        self, *, review_type: str
     ) -> colrev.ops.review_types.ReviewTypes:
         """Get a ReviewTypes object"""
         import colrev.ops.review_types
@@ -695,7 +719,7 @@ class ReviewManager:
     def get_review_manager(
         self,
         *,
-        path_str: str = None,
+        path_str: Optional[str] = None,
         force_mode: bool = False,
         verbose_mode: bool = False,
     ) -> ReviewManager:
@@ -704,6 +728,21 @@ class ReviewManager:
             path_str=path_str, force_mode=force_mode, verbose_mode=verbose_mode
         )
 
+    def in_ci_environment(
+        self,
+        *,
+        identifier: Optional[str] = None,
+    ) -> bool:
+        """Check whether CoLRev runs in a continuous-integration environment"""
+
+        if "pytest" in os.getcwd():
+            return False
+
+        identifier_list = ["GITHUB_ACTIONS", "CIRCLECI", "TRAVIS", "GITLAB_CI"]
+        if identifier:
+            identifier_list = [identifier]
+        return any("true" == os.getenv(x) for x in identifier_list)
+
 
 if __name__ == "__main__":
     pass
