@@ -10,7 +10,6 @@ from dataclasses import asdict
 from dataclasses import dataclass
 from enum import Enum
 from pathlib import Path
-from typing import TYPE_CHECKING
 
 import dacite
 from dacite import from_dict
@@ -22,15 +21,16 @@ from dataclasses_jsonschema import JsonSchemaMixin
 import colrev.env.utils
 import colrev.exceptions as colrev_exceptions
 
-if TYPE_CHECKING:
-    import colrev.review_manager
+if False:  # pylint: disable=using-constant-test
+    from typing import TYPE_CHECKING
+
+    if TYPE_CHECKING:
+        import colrev.review_manager
 
 
 # Note : to avoid performance issues on startup (ReviewManager, parsing settings)
 # the settings dataclasses should be in one file (13s compared to 0.3s)
 
-# https://stackoverflow.com/questions/66807878/pretty-print-dataclasses-prettier
-
 
 # Project
 
@@ -120,7 +120,10 @@ class ProjectSettings(JsonSchemaMixin):
     colrev_version: str
 
     def __str__(self) -> str:
-        return f"Review ({self.review_type}): {self.title}"
+        project_str = f"Review ({self.review_type}):"
+        if self.title:
+            project_str += f" {self.title}"
+        return project_str
 
 
 # Search
@@ -302,7 +305,7 @@ class PrepSettings(JsonSchemaMixin):
 
     def __str__(self) -> str:
         return (
-            " - prep_rounds: \n   - "
+            " - prep_rounds:\n   - "
             + "\n   - ".join([str(prep_round) for prep_round in self.prep_rounds])
             + f"\n - fields_to_keep: {self.fields_to_keep}"
         )
@@ -473,7 +476,9 @@ class ScreenSettings(JsonSchemaMixin):
     screen_package_endpoints: list
 
     def __str__(self) -> str:
-        return " - " + "\n - ".join([str(c) for c in self.criteria])
+        if self.criteria:
+            return " - " + "\n - ".join([str(c) for c in self.criteria])
+        return " -"
 
 
 # Data
@@ -507,15 +512,29 @@ class Settings(JsonSchemaMixin):
     screen: ScreenSettings
     data: DataSettings
 
+    def is_curated_repo(self) -> bool:
+        """Check whether data is curated in this repository"""
+
+        curation_endpoints = [
+            x
+            for x in self.data.data_package_endpoints
+            if x["endpoint"] == "colrev.colrev_curation"
+        ]
+        return bool(curation_endpoints)
+
     def is_curated_masterdata_repo(self) -> bool:
         """Check whether the masterdata is curated in this repository"""
 
         curation_endpoints = [
             x
             for x in self.data.data_package_endpoints
-            if x["endpoint"] == "colrev_built_in.colrev_curation"
+            if x["endpoint"] == "colrev.colrev_curation"
         ]
-        return bool(curation_endpoints)
+        if curation_endpoints:
+            curation_endpoint = curation_endpoints[0]
+            if curation_endpoint["curated_masterdata"]:
+                return True
+        return False
 
     def __str__(self) -> str:
         return (
@@ -542,7 +561,8 @@ class Settings(JsonSchemaMixin):
             + str(self.data)
         )
 
-    def get_settings_schema(self) -> dict:
+    @classmethod
+    def get_settings_schema(cls) -> dict:
         """Get the json-schema for the settings"""
 
         class PathField(FieldEncoder):
@@ -556,7 +576,7 @@ class Settings(JsonSchemaMixin):
 
         with warnings.catch_warnings():
             warnings.simplefilter("ignore")
-            schema = self.json_schema()
+            schema = cls.json_schema()
 
         sdefs = schema["definitions"]
         sdefs["SearchSource"]["properties"]["load_conversion_package_endpoint"] = {  # type: ignore
@@ -609,35 +629,12 @@ class Settings(JsonSchemaMixin):
         return schema
 
 
-def load_settings(*, review_manager: colrev.review_manager.ReviewManager) -> Settings:
-    """Load the settings from file"""
-
-    # https://tech.preferred.jp/en/blog/working-with-configuration-in-python/
-    # possible extension : integrate/merge global, default settings
-    # from colrev.environment import EnvironmentManager
-    # def selective_merge(base_obj, delta_obj):
-    #     if not isinstance(base_obj, dict):
-    #         return delta_obj
-    #     common_keys = set(base_obj).intersection(delta_obj)
-    #     new_keys = set(delta_obj).difference(common_keys)
-    #     for k in common_keys:
-    #         base_obj[k] = selective_merge(base_obj[k], delta_obj[k])
-    #     for k in new_keys:
-    #         base_obj[k] = delta_obj[k]
-    #     return base_obj
-    # print(selective_merge(default_settings, project_settings))
-
-    if not review_manager.settings_path.is_file():
-        raise colrev_exceptions.RepoSetupError()
-
-    with open(review_manager.settings_path, encoding="utf-8") as file:
-        loaded_settings = json.load(file)
-
+def __load_settings_from_dict(*, loaded_dict: dict) -> Settings:
     try:
         converters = {Path: Path, Enum: Enum}
         settings = from_dict(
             data_class=Settings,
-            data=loaded_settings,
+            data=loaded_dict,
             config=dacite.Config(type_hooks=converters, cast=[Enum]),  # type: ignore
         )
         for source in settings.sources:
@@ -652,17 +649,32 @@ def load_settings(*, review_manager: colrev.review_manager.ReviewManager) -> Set
             raise colrev_exceptions.InvalidSettingsError(msg=msg, fix_per_upgrade=False)
 
     except (ValueError, MissingValueError, WrongTypeError, AssertionError) as exc:
-        raise colrev_exceptions.InvalidSettingsError(msg=str(exc)) from exc
+        raise colrev_exceptions.InvalidSettingsError(
+            msg=str(exc)
+        ) from exc  # pragma: no cover
 
     return settings
 
 
+def load_settings(*, settings_path: Path) -> Settings:
+    """Load the settings from file"""
+
+    if not settings_path.is_file():
+        raise colrev_exceptions.RepoSetupError()
+
+    with open(settings_path, encoding="utf-8") as file:
+        loaded_dict = json.load(file)
+
+    return __load_settings_from_dict(loaded_dict=loaded_dict)
+
+
 def save_settings(*, review_manager: colrev.review_manager.ReviewManager) -> None:
     """Save the settings"""
 
     exported_dict = asdict(
         review_manager.settings, dict_factory=colrev.env.utils.custom_asdict_factory
     )
+
     with open("settings.json", "w", encoding="utf-8") as outfile:
         json.dump(exported_dict, outfile, indent=4)
     review_manager.dataset.add_changes(path=Path("settings.json"))
