@@ -102,7 +102,7 @@ def main(ctx: click.core.Context) -> None:
 
     Recommended workflow: colrev status > colrev OPERATION > colrev validate
 
-    Documentation:  https://github.com/geritwagner/colrev/docs
+    Documentation:  https://colrev.readthedocs.io/
     """
 
 
@@ -113,6 +113,12 @@ def main(ctx: click.core.Context) -> None:
     default="literature_review",
     help="Review type (e.g., literature_review (default), scoping_review, theoretical_review)",
 )
+@click.option(
+    "--light",
+    is_flag=True,
+    default=False,
+    help="Setup a lightweight repository (not requiring Docker services)",
+)
 @click.option(
     "--example",
     is_flag=True,
@@ -145,6 +151,7 @@ def init(
     ctx: click.core.Context,
     type: str,
     example: bool,
+    light: bool,
     local_pdf_collection: bool,
     verbose: bool,
     force: bool,
@@ -155,10 +162,10 @@ def init(
     import colrev.ui_web.settings_editor
 
     try:
-
         colrev.review_manager.ReviewManager.get_init_operation(
             review_type=type,
             example=example,
+            light=light,
             local_pdf_collection=local_pdf_collection,
             exact_call=EXACT_CALL,
         )
@@ -248,19 +255,18 @@ def retrieve(
     """Retrieve, a high-level operation, consists of search, load, prep, and dedupe.
 
     \b
-    To retrieve search results
-    - copy files (*.bib, *.ris, *.xlsx, ...) to the directory data/search or
-    - copy PDF files to the directory data/pdfs or
-    - add an API-based search, as described in the documentation:
+    To retrieve search results:
+    - Copy files (*.bib, *.ris, *.xlsx, ...) to the directory data/search or
+    - Copy PDF files to the directory data/pdfs or
+    - Add an API-based search, as described in the documentation:
 
-    https://github.com/geritwagner/colrev/blob/main/docs/source/user_resources/2_2_metadata_retrieval/search.rst
+    https://colrev.readthedocs.io/en/latest/manual/metadata_retrieval/search.html
     """
 
     try:
         review_manager = colrev.review_manager.ReviewManager(
             verbose_mode=verbose, force_mode=force, high_level_operation=True
         )
-        review_manager.get_local_index(startup_without_waiting=True)
 
         if not any(review_manager.search_dir.iterdir()) and not any(
             review_manager.pdf_dir.iterdir()
@@ -274,8 +280,7 @@ def retrieve(
                 f"to the directory {review_manager.SEARCHDIR_RELATIVE} or\n"
                 f" - copy PDF files to the directory {review_manager.PDF_DIR_RELATIVE} or \n"
                 " - add an API-based search, as described in the documentation:\n"
-                "https://github.com/geritwagner/colrev/blob/main/"
-                "docs/source/user_resources/2_2_metadata_retrieval/search.rst"
+                "https://colrev.readthedocs.io/en/latest/manual/metadata_retrieval/search.html"
             )
             return
 
@@ -283,7 +288,6 @@ def retrieve(
         review_manager.logger.info(
             "Retrieve is a high-level operation consisting of search, load, prep, and dedupe:"
         )
-
         print()
 
         search_operation = review_manager.get_search_operation()
@@ -291,6 +295,7 @@ def retrieve(
 
         print()
 
+        review_manager.exact_call = "colrev prep"
         load_operation = review_manager.get_load_operation()
         new_sources = load_operation.get_new_sources(skip_query=True)
         load_operation = review_manager.get_load_operation(hide_load_explanation=True)
@@ -299,12 +304,13 @@ def retrieve(
         )
 
         print()
-
+        review_manager.exact_call = "colrev prep"
         prep_operation = review_manager.get_prep_operation()
         prep_operation.main()
 
         print()
 
+        review_manager.exact_call = "colrev dedupe"
         dedupe_operation = review_manager.get_dedupe_operation()
         dedupe_operation.main()
 
@@ -327,7 +333,7 @@ Format: RETRIEVE * FROM crossref WHERE title LIKE '%keyword%'
 @click.option(
     "-s",
     "--selected",
-    type=click.Path(exists=True),
+    type=str,
     help="Only retrieve search results for selected sources",
 )
 @click.option(
@@ -463,7 +469,6 @@ def load(
             force_mode=force, verbose_mode=verbose, exact_call=EXACT_CALL
         )
         # already start LocalIndex (for set_ids)
-        review_manager.get_local_index(startup_without_waiting=True)
         load_operation = review_manager.get_load_operation()
 
         new_sources = load_operation.get_new_sources(skip_query=skip_query)
@@ -494,6 +499,12 @@ def load(
     default=False,
     help="Do not change the record IDs. Useful when importing an existing sample.",
 )
+@click.option(
+    "--polish",
+    is_flag=True,
+    default=False,
+    help="Polish record metadata (includes records in md_processed or beyond).",
+)
 @click.option(
     "--reset_records",
     default="NA",
@@ -521,6 +532,11 @@ def load(
     type=str,
     help="Debug the preparation step for a selected record (can be 'all').",
 )
+@click.option(
+    "--cpu",
+    type=int,
+    help="Number of cpus (parallel processes)",
+)
 @click.option(
     "-scs",
     "--setup_custom_script",
@@ -534,6 +550,13 @@ def load(
     type=click.Path(exists=True),
     help="Debug the preparation step for a selected record (in a file).",
 )
+@click.option(
+    "--skip",
+    is_flag=True,
+    default=False,
+    help="Skip the preparation.",
+    hidden=True,
+)
 @click.option(
     "-v",
     "--verbose",
@@ -552,19 +575,23 @@ def load(
 def prep(
     ctx: click.core.Context,
     keep_ids: bool,
+    polish: bool,
     reset_records: str,
     reset_ids: bool,
     set_ids: bool,
     debug: str,
     debug_file: Path,
+    cpu: int,
     setup_custom_script: bool,
+    skip: bool,
     verbose: bool,
     force: bool,
 ) -> None:
     """Prepare records"""
 
+    # pylint: disable=too-many-branches
+    # pylint: disable=too-many-locals
     try:
-
         review_manager = colrev.review_manager.ReviewManager(
             force_mode=force, verbose_mode=verbose, exact_call=EXACT_CALL
         )
@@ -595,8 +622,10 @@ def prep(
                 "Please check and adapt its position in the settings.json and commit."
             )
             return
+        if skip:
+            prep_operation.skip_prep()
 
-        prep_operation.main(keep_ids=keep_ids)
+        prep_operation.main(keep_ids=keep_ids, cpu=cpu, polish=polish)
 
     except colrev_exceptions.ServiceNotAvailableException as exc:
         print(exc)
@@ -653,7 +682,6 @@ def prep_man(ctx: click.core.Context, stats: bool, verbose: bool, force: bool) -
 
 
 def __view_dedupe_details(dedupe_operation: colrev.ops.dedupe.Dedupe) -> None:
-
     info = dedupe_operation.get_info()
 
     if len(info["same_source_merges"]) > 0:
@@ -711,7 +739,6 @@ def dedupe(
     """Deduplicate records"""
 
     try:
-
         review_manager = colrev.review_manager.ReviewManager(
             force_mode=force, verbose_mode=verbose, exact_call=EXACT_CALL
         )
@@ -872,9 +899,17 @@ def prescreen(
         else:
             review_manager.logger.info("Prescreen")
             review_manager.logger.info(
-                "Exclude irrelevant records based on metadata (i.e., titles and abstracts). "
-                "Remaining records are retained provisionally."
+                "Exclude irrelevant records based on metadata (i.e., titles and abstracts)."
             )
+            review_manager.logger.info("Remaining records are retained provisionally")
+            review_manager.logger.info(
+                "In the screen, they can be included or excluded based on full-text documents."
+            )
+            review_manager.logger.info(
+                "See https://colrev.readthedocs.io/en/"
+                "latest/manual/metadata_prescreen/prescreen.html"
+            )
+
             prescreen_operation.main(split_str=split)
 
     except colrev_exceptions.CoLRevException as exc:
@@ -997,6 +1032,13 @@ def screen(
     default=False,
     help="Discard all missing PDFs as not_available",
 )
+@click.option(
+    "-d",
+    "--dir",
+    is_flag=True,
+    default=False,
+    help="Open the PDFs directory",
+)
 @click.option(
     "-v",
     "--verbose",
@@ -1015,6 +1057,7 @@ def screen(
 def pdfs(
     ctx: click.core.Context,
     discard: bool,
+    dir: bool,
     verbose: bool,
     force: bool,
 ) -> None:
@@ -1028,6 +1071,21 @@ def pdfs(
             exact_call=EXACT_CALL,
         )
 
+        if dir:
+            # pylint: disable=import-outside-toplevel
+            # pylint: disable=consider-using-with
+            # pylint: disable=no-member
+            import platform
+
+            path = review_manager.path / Path("data/pdfs")
+            if platform.system() == "Windows":
+                os.startfile(path)  # type: ignore
+            elif platform.system() == "Darwin":
+                subprocess.Popen(["open", path])
+            else:
+                subprocess.Popen(["xdg-open", path])
+            return
+
         if discard:
             pdf_prep_man_operation = review_manager.get_pdf_prep_man_operation()
             pdf_prep_man_operation.discard()
@@ -1305,6 +1363,12 @@ def __print_pdf_hashes(
     default=False,
     help="Prepare all PDFs again (pdf_needs_manual_preparation).",
 )
+@click.option(
+    "--tei",
+    is_flag=True,
+    default=False,
+    help="Generate TEI documents.",
+)
 @click.option(
     "-c",
     "--cover",
@@ -1345,6 +1409,7 @@ def pdf_prep(
     reprocess: bool,
     pdf_hash: Path,
     setup_custom_script: bool,
+    tei: bool,
     cover: Path,
     verbose: bool,
     force: bool,
@@ -1370,7 +1435,8 @@ def pdf_prep(
         elif setup_custom_script:
             pdf_prep_operation.setup_custom_script()
             print("Activated custom_pdf_prep_script.py.")
-
+        elif tei:
+            pdf_prep_operation.generate_tei()
         else:
             pdf_prep_operation.main(batch_size=batch_size)
 
@@ -1385,7 +1451,6 @@ def pdf_prep(
 def __delete_first_pages_cli(
     pdf_prep_man_operation: colrev.ops.pdf_prep_man.PDFPrepMan, record_id: str
 ) -> None:
-
     records = pdf_prep_man_operation.review_manager.dataset.load_records_dict()
     while True:
         if record_id in records:
@@ -1396,8 +1461,8 @@ def __delete_first_pages_cli(
                     record_dict["file"]
                 )
                 pdf_prep_man_operation.extract_coverpage(filepath=pdf_path)
-                colrev.record.Record(data=record_dict).set_pdf_man_prepared(
-                    review_manager=pdf_prep_man_operation.review_manager
+                pdf_prep_man_operation.set_pdf_man_prepared(
+                    record=colrev.record.Record(data=record_dict)
                 )
             else:
                 print("no file in record")
@@ -1511,7 +1576,7 @@ def pdf_prep_man(
     "-a",
     "--add",
     type=str,
-    help="Add a data_format endpoint (e.g., colrev_built_in.structured)",
+    help="Add a data_format endpoint (e.g., colrev.structured)",
 )
 @click.option(
     "-scs",
@@ -1551,7 +1616,7 @@ def data(
 
     try:
         review_manager = colrev.review_manager.ReviewManager(
-            force_mode=force, verbose_mode=verbose, exact_call=EXACT_CALL
+            force_mode=(force or profile), verbose_mode=verbose, exact_call=EXACT_CALL
         )
         data_operation = review_manager.get_data_operation()
 
@@ -1576,24 +1641,30 @@ def data(
             return
 
         ret = data_operation.main()
-        if ret["ask_to_commit"]:
-            if "y" == input("Create commit (y/n)?"):
+        if data_operation.review_manager.in_ci_environment():
+            if ret["ask_to_commit"]:
                 review_manager.create_commit(
                     msg="Data and synthesis", manual_author=True
                 )
-        if ret["no_endpoints_registered"]:
-            print(
-                "No data format not specified. "
-                "To register a data endpoint, "
-                "use one (or several) of the following \n"
-                "    colrev data --add colrev_built_in.manuscript\n"
-                "    colrev data --add colrev_built_in.structured\n"
-                "    colrev data --add colrev_built_in.bibliography_export\n"
-                "    colrev data --add colrev_built_in.prisma\n"
-                "    colrev data --add colrev_built_in.github_pages\n"
-                "    colrev data --add colrev_built_in.zettlr\n"
-                "    colrev data --add colrev_built_in.colrev_curation"
-            )
+        else:
+            if ret["ask_to_commit"]:
+                if "y" == input("Create commit (y/n)?"):
+                    review_manager.create_commit(
+                        msg="Data and synthesis", manual_author=True
+                    )
+            if ret["no_endpoints_registered"]:
+                print(
+                    "No data format not specified. "
+                    "To register a data endpoint, "
+                    "use one (or several) of the following \n"
+                    "    colrev data --add colrev.paper_md\n"
+                    "    colrev data --add colrev.structured\n"
+                    "    colrev data --add colrev.bibliography_export\n"
+                    "    colrev data --add colrev.prisma\n"
+                    "    colrev data --add colrev.github_pages\n"
+                    "    colrev data --add colrev.zettlr\n"
+                    "    colrev data --add colrev.colrev_curation"
+                )
 
     except colrev_exceptions.CoLRevException as exc:
         if verbose:
@@ -1674,9 +1745,6 @@ def validate(
                 validation_details=validation_details,
                 threshold=threshold,
             )
-        else:
-            print("No substantial changes.")
-            return
 
         review_manager.logger.info("%sCompleted validation%s", colors.GREEN, colors.END)
 
@@ -1788,11 +1856,8 @@ def distribute(ctx: click.core.Context, path: Path, verbose: bool, force: bool)
 def __print_environment_status(
     review_manager: colrev.review_manager.ReviewManager,
 ) -> None:
-
     environment_manager = review_manager.get_environment_manager()
-    environment_details = environment_manager.get_environment_details(
-        review_manager=review_manager
-    )
+    environment_details = environment_manager.get_environment_details()
 
     print("\nCoLRev environment status\n")
     print("Index\n")
@@ -1811,7 +1876,6 @@ def __print_environment_status(
         if "curated_metadata" not in x["repo_source_path"]
     ]
     for colrev_repo in sorted(project_repos, key=lambda d: d["repo_name"]):
-
         repo_stats = f' {colrev_repo["repo_name"]}'
         if colrev_repo["remote"]:
             if colrev_repo["behind_remote"]:
@@ -1864,12 +1928,6 @@ def __print_environment_status(
 )
 @click.option("--start", is_flag=True, default=False, help="Start environment services")
 @click.option("--stop", is_flag=True, default=False, help="Stop environment services")
-@click.option(
-    "--search",
-    is_flag=True,
-    default=False,
-    help="Start opensearch dashboard service to search the LocalIndex",
-)
 @click.option(
     "-r",
     "--register",
@@ -1912,7 +1970,6 @@ def env(
     status: bool,
     start: bool,
     stop: bool,
-    search: bool,
     register: bool,
     unregister: bool,
     update_package_list: bool,
@@ -1921,13 +1978,10 @@ def env(
 ) -> None:
     """Manage the environment"""
 
-    # pylint: disable=import-outside-toplevel
     # pylint: disable=too-many-return-statements
     # pylint: disable=too-many-branches
     # pylint: disable=too-many-locals
 
-    import webbrowser
-
     review_manager = colrev.review_manager.ReviewManager(
         force_mode=True, verbose_mode=verbose
     )
@@ -1986,18 +2040,23 @@ def env(
         return
 
     if update_package_list:
+        if "y" != input(
+            "The following process instantiates objects listed in the "
+            + "colrev/template/package_endpoints.json "
+            + "(including ones that may not be secure).\n"
+            + "Please confirm (y) to proceed."
+        ):
+            return
+
+        # pylint: disable=import-outside-toplevel
         import colrev.env.package_manager as p_manager
 
         package_manager = p_manager.PackageManager()
         package_manager.update_package_list()
 
     local_index = review_manager.get_local_index()
-    if search:
-        local_index.start_opensearch_docker_dashboards()
-        print("Started.")
-        webbrowser.open("http://localhost:5601/app/home#/", new=2)
 
-    elif index:
+    if index:
         local_index.index()
 
     elif start:
@@ -2060,7 +2119,6 @@ def settings(
         force_mode=force, verbose_mode=verbose, exact_call=EXACT_CALL
     )
     if update_hooks:
-
         print("Update pre-commit hooks")
 
         if review_manager.dataset.has_changes():
@@ -2072,7 +2130,7 @@ def settings(
                 "pre-commit",
                 "autoupdate",
                 "--repo",
-                "https://github.com/geritwagner/colrev-hooks",
+                "https://github.com/CoLRev-Ecosystem/colrev-hooks",
             ],
         ]
         for script_to_call in scripts_to_call:
@@ -2084,14 +2142,13 @@ def settings(
         return
 
     if modify:
-
         # TBD: maybe use glom.delete?
         # There is no simply append...
         # (we could replace the (last) position element with
         # keywords like prescreen.sripts.LAST_POSITION)
         # maybe prescreen.scripts.1.REPLACE/ADD/DELETE = ....
         # modify = 'dedupe.dedupe_package_endpoints='
-        # '[{"endpoint":"colrev_built_in.simple_dedupe"}]'
+        # '[{"endpoint":"colrev.simple_dedupe"}]'
 
         path, value_string = modify.split("=")
         value = ast.literal_eval(value_string)
@@ -2330,7 +2387,6 @@ def service(
     """Service for real-time reviews"""
 
     try:
-
         review_manager = colrev.review_manager.ReviewManager(
             force_mode=force, verbose_mode=verbose, exact_call=EXACT_CALL
         )
@@ -2506,8 +2562,8 @@ def upgrade(
     review_manager = colrev.review_manager.ReviewManager(
         force_mode=True, verbose_mode=verbose
     )
-    upgrad_operation = review_manager.get_upgrade()
-    upgrad_operation.main()
+    upgrade_operation = review_manager.get_upgrade()
+    upgrade_operation.main()
 
 
 @main.command(hidden=True, help_priority=30)
@@ -2656,6 +2712,41 @@ def merge(
 
 
 @main.command(help_priority=34)
+@click.argument("selection")
+@click.option(
+    "-v",
+    "--verbose",
+    is_flag=True,
+    default=False,
+    help="Verbose: printing more infos",
+)
+@click.option(
+    "-f",
+    "--force",
+    is_flag=True,
+    default=False,
+    help="Force mode",
+)
+@click.pass_context
+def undo(
+    ctx: click.core.Context,
+    selection: str,
+    verbose: bool,
+    force: bool,
+) -> None:
+    """Undo operations."""
+
+    review_manager = colrev.review_manager.ReviewManager(
+        force_mode=force, verbose_mode=verbose
+    )
+
+    if "commit" == selection:
+        colrev.operation.CheckOperation(review_manager=review_manager)
+        git_repo = review_manager.dataset.get_repo()
+        git_repo.git.reset("--hard", "HEAD~1")
+
+
+@main.command(help_priority=35)
 @click.pass_context
 def version(
     ctx: click.core.Context,
